/*==============================================================================
| NAME:    party_watchlist_header.sas
| PRODUCT: SAS Anti-Money Laundering
| SYSTEM:  UNIX/WINDOWS
================================================================================


PURPOSE
-------
- Perform data preparation functions required to support the following SAS
  Watch List scenarios

- SAS10079
- SAS10080
- SAS10082


DETAILS
-------

The party_watch_list_header.sas program constructs SAS data set lookup tables
required by the watch list scenarios. For individuals, lookups are performed
on last_name and match_code_individual while for organizations lookups are
performed on organization_name and match_code_organization.

The lookup tables are then used to directly locate matches in the
FSC_PARTY_DIM table. The lookup method directly addresses matches in the
data set tables using either an INDEX lookup method or a HASH lookup method.
The lookup method selected is set by the SAS global macro variable
WATCH_LIST_PROCESSING_METHOD located in the SAS Anti-Money Laundering
autoexec.sas file.

Setting WATCH_LIST_PROCESSING_METHOD=HASH_ARRAY results in the instantiation of
4 SAS hash objects representing the SAS data set lookup tables. The hash object
find() method is then used to locate matches.

Setting WATCH_LIST_PROCESSING_METHOD=INDEX_ARRARY results in and index lookup
mehtod to locate matches.

When a match between LAST_NAME or MATCH_CODE_INDIVIDUALS where party_type_desc
is 'INDIVDUAL' or between ORGANIZATION_NAME or MATCH_CODE_ORGANIZATION where
party_type_desc='ORGANIZATION' occurs the included watch list scenario code
will check additional data attributes for possible matches.


================================================================================
================================================================================
Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
- All rights reserved
==============================================================================*/


%macro aml_party_watchlist_header;

  options IBUFSIZE = max;

  /* Delete data sets from previous execution */
  proc datasets lib=STG_WTCH nolist nowarn;
     delete   wl_ind_: ;
     delete   wl_org_: ;
     delete   fsc_entity_watch_list_dim;
  quit;


/*****************************************************************************/
/* Create following STAGE tables:                                            */
/*                                                                           */
/* FSC_ENTITY_WATCH_LIST_DIM                                                 */
/*                                                                           */
/* - Staged version of FSC_ENTITY_WATCH_LIST_DIM. This table will contain    */
/* only those columns required for scenario processing.                      */
/*                                                                           */
/* WL_IND_NAME  - Lookup table for individual LAST_NAME                      */
/* WL_IND_MATCH - Lookup table for individual MATCH_CODE_INDIVIDUAL          */
/* WL_ORG_NAME  - Lookup table for organization ENTITY_NAME                  */
/* WL_ORG_MATCH - Lookup table for organization MATCH_CODE_ORGANIZATION      */
/*****************************************************************************/
 proc sort data=seg_kc.fsc_entity_watch_list_dim
           out=fsc_entity_watch_list_dim
     (keep=entity_watch_list_key
           entity_watch_list_number
           watch_list_name
           type_desc
           category_desc
           tax_id
           date_of_birth
           year_of_birth
           first_name
           last_name
           middle_name
           entity_name
           match_code_individual
           match_code_organization
           match_code_addr_lines
           match_code_city
           match_code_state
           match_code_country
           match_code_full_address
           citizenship_country_code
           org_country_of_business_code
           politically_exposed_person_ind
           exclude_ind
           change_current_ind
           where= (upcase(change_current_ind) eq 'Y' and upcase(exclude_ind) eq 'N'));
     by entity_watch_list_number;
 run;


data

  stg_wtch.fsc_entity_watch_list_dim(keep=entity_watch_list_key
                                          entity_watch_list_number
                                          watch_list_name
                                          type_desc
                                          category_desc
                                          tax_id
                                          date_of_birth
                                          year_of_birth
                                          first_name
                                          last_name
                                          middle_name
                                          entity_name
                                          match_code_individual
                                          match_code_organization
                                          match_code_addr_lines
                                          match_code_city
                                          match_code_state
                                          match_code_country
                                          match_code_full_address
                                          citizenship_country_code
                                          org_country_of_business_code
                                          politically_exposed_person_ind
                                          index_counter
                                     compress=yes)

  stg_wtch.wl_ind_name (keep=last_name               index_counter compress=yes)
  stg_wtch.wl_ind_match(keep=match_code_individual   index_counter compress=yes)
  stg_wtch.wl_org_name (keep=entity_name             index_counter compress=yes)
  stg_wtch.wl_org_match(keep=match_code_organization index_counter compress=yes)
  ;

  retain index_counter 0
         max_first_name 0 
         max_middle_name 0 
         max_entity_name 0;

  set fsc_entity_watch_list_dim end=eof;

  /*find longest first, last, middle, and entity names */
  if length(first_name) > max_first_name then max_first_name = length(first_name);
  if length(middle_name) > max_middle_name then max_middle_name = length(middle_name);
  if length(entity_name) > max_entity_name then max_entity_name = length(entity_name);

  if upcase(type_desc) eq 'INDIVIDUAL' and not missing(last_name) then do;
      last_name=upcase(compbl(kstrip(last_name)));
      index_counter + 1;
      output stg_wtch.fsc_entity_watch_list_dim
             stg_wtch.wl_ind_name
             stg_wtch.wl_ind_match;
   end;
   else if upcase(type_desc) eq 'ORGANIZATION' and not missing(entity_name) then do;
      entity_name=upcase(compbl(kstrip(entity_name)));
      index_counter + 1;
      output stg_wtch.fsc_entity_watch_list_dim
             stg_wtch.wl_org_name
             stg_wtch.wl_org_match;
   end;

   if eof then do;
      call symputx('array_dimension',index_counter,'g');
	  call symputx('array_dim_first_name',max_first_name,'g');
	  call symputx('array_dim_middle_name',max_middle_name,'g');
	  call symputx('array_dim_entity_name',max_entity_name,'g');
   end;

run;

proc sql noprint;
 drop table work.fsc_entity_watch_list_dim;
quit;

/* Process lookup tables */
%macro aml_process_lookup_table(table=,
                                by_columns=,
                                index_columns=);

  proc sort data=stg_wtch.&table
             out=stg_wtch.&table;
    by &by_columns;
  run;

  data stg_wtch.&table(compress=yes index=(&index_columns));
    set stg_wtch.&table;
  run;

%mend aml_process_lookup_table;


%aml_process_lookup_table(table=wl_ind_name,
                          by_columns=last_name index_counter,
                          index_columns=last_name);

%aml_process_lookup_table(table=wl_ind_match,
                          by_columns=match_code_individual index_counter,
                          index_columns=match_code_individual);

%aml_process_lookup_table(table=wl_org_name,
                          by_columns=entity_name index_counter,
                          index_columns=entity_name);

%aml_process_lookup_table(table=wl_org_match,
                          by_columns=match_code_organization index_counter,
                          index_columns=match_code_organization);

/* Create distinct version of lookup tables */
%macro aml_create_distinct_lookup_table(table=,
                                        distinct_column=);

  proc sort data=stg_wtch.&table
             out=stg_wtch.&table._d(keep=&distinct_column)
             nodupkey;
    by &distinct_column;
  run;

  data stg_wtch.&table._d(index=(&distinct_column) compress=yes);
    set stg_wtch.&table._d;
  run;

%mend aml_create_distinct_lookup_table;

%aml_create_distinct_lookup_table(table=wl_ind_name,  distinct_column=last_name);
%aml_create_distinct_lookup_table(table=wl_ind_match, distinct_column=match_code_individual);
%aml_create_distinct_lookup_table(table=wl_org_name,  distinct_column=entity_name);
%aml_create_distinct_lookup_table(table=wl_org_match, distinct_column=match_code_organization);


data &alert_fname(keep=primary_entity_number
                       scenario_id
                       rundate
                       entity_level_code
                       actual_values_text
  );

  format rundate date9.;

  length
     n 8
     create_array 8
     create_hash  8
     entity_watch_list_number_prev $ 35
     actual_values_text $ 512
     match_code_individual_r $ 80
     match_code_organization_r $ 80
  ;

  retain
     n
     create_array
     create_hash 0
     entity_watch_list_number_prev
  ;


  if upcase("&WATCH_LIST_PROCESSING_METHOD") eq 'HASH_ARRAY' then do;

    if create_hash eq 0 then do;

      if 0 then do;
         set stg_wtch.wl_ind_name_d;
         set stg_wtch.wl_ind_match_d;
         set stg_wtch.wl_org_name_d;
         set stg_wtch.wl_org_match_d;
      end;

      /* INDIVIDUAL: KEY=LAST_NAME */
      declare hash h_wl_ind_name(dataset: "stg_wtch.wl_ind_name_d", hashexp: 16);
                   h_wl_ind_name.definekey("last_name");
                   h_wl_ind_name.definedone();

      /* INDIVIDUAL: KEY=MATCH_CODE_INDIVIDUAL */
      declare hash h_wl_ind_match(dataset: "stg_wtch.wl_ind_match_d", hashexp: 16);
                   h_wl_ind_match.definekey("match_code_individual");
                   h_wl_ind_match.defineDone();

      /* ORGANIZATION: KEY=ORGANIZATION_NAME */
      declare hash h_wl_org_name(dataset: "stg_wtch.wl_org_name_d", hashexp: 16);
                   h_wl_org_name.definekey("entity_name");
                   h_wl_org_name.defineDone();

      /* ORGANIZATION: KEY=MATCH_CODE_ORGANIZATION */
      declare hash h_wl_org_match(dataset: "stg_wtch.wl_org_match_d", hashexp: 16);
                   h_wl_org_match.definekey("match_code_organization");
                   h_wl_org_match.defineDone();

      create_hash=1;

    end;
  end;

  array _entity_watch_list_key          {&array_dimension} 8    _TEMPORARY_;
  array _entity_watch_list_number       {&array_dimension} $35  _TEMPORARY_;
  array _watch_list_name                {&array_dimension} $4   _TEMPORARY_;
  array _category_desc                  {&array_dimension} $50  _TEMPORARY_;
  array _tax_id                         {&array_dimension} $35  _TEMPORARY_;
  array _date_of_birth                  {&array_dimension} 8    _TEMPORARY_;
  array _year_of_birth                  {&array_dimension} 8    _TEMPORARY_;
  array _first_name                     {&array_dimension} $&array_dim_first_name   _TEMPORARY_;
  array _middle_name                    {&array_dimension} $&array_dim_middle_name  _TEMPORARY_;
  array _entity_name                    {&array_dimension} $&array_dim_entity_name  _TEMPORARY_;
  array _match_code_individual          {&array_dimension} $80  _TEMPORARY_;
  array _match_code_organization        {&array_dimension} $80  _TEMPORARY_;
  array _match_code_addr_lines          {&array_dimension} $80  _TEMPORARY_;
  array _match_code_city                {&array_dimension} $15  _TEMPORARY_;
  array _match_code_state               {&array_dimension} $15  _TEMPORARY_;
  array _match_code_country             {&array_dimension} $15  _TEMPORARY_;
  array _match_code_full_address        {&array_dimension} $80  _TEMPORARY_;
  array _citizenship_country_code       {&array_dimension} $3   _TEMPORARY_;
  array _org_country_of_business_code   {&array_dimension} $3   _TEMPORARY_;
  array _politically_exposed_person_ind {&array_dimension} $1   _TEMPORARY_;

  do while (create_array=0);

   set stg_wtch.fsc_entity_watch_list_dim end=eof;

   n + 1;
   _entity_watch_list_key{n}=entity_watch_list_key;
   _entity_watch_list_number{n}=entity_watch_list_number;
   _watch_list_name{n}=watch_list_name;
   _category_desc{n}=category_desc;
   _tax_id{n}=kcompress(kcompress(tax_id,"-"));
   _date_of_birth{n}=date_of_birth;
   _year_of_birth{n}=year_of_birth;
   _first_name{n}=first_name;
   _middle_name{n}=middle_name;
   _entity_name{n}=entity_name;
   _match_code_individual{n}=match_code_individual;
   _match_code_organization{n}=match_code_organization;
   _match_code_addr_lines{n}=match_code_addr_lines;
   _match_code_city{n}=match_code_city;
   _match_code_state{n}=match_code_state;
   _match_code_country{n}=match_code_country;
   _match_code_full_address{n}=match_code_full_address;
   _politically_exposed_person_ind{n}=politically_exposed_person_ind;
   _citizenship_country_code{n}=citizenship_country_code;
   _org_country_of_business_code{n}=org_country_of_business_code;

   if eof then
      create_array=1;

  end;

  set seg_kc.fsc_party_dim(keep=party_number
                                party_type_desc
                                party_tax_id
                                party_date_of_birth
                                party_first_name
                                party_middle_name
                                party_last_name
                                party_name
                                match_code_organization
                                match_code_individual
                                match_code_street_address
                                match_code_street_addr_lines
                                match_code_street_city
                                match_code_street_state
                                match_code_street_country
                                match_code_mailing_address
                                match_code_mailing_addr_lines
                                match_code_mailing_city
                                match_code_mailing_state
                                match_code_mailing_country
                                citizenship_country_code
                                org_country_of_business_code
                                politically_exposed_person_ind
                                change_current_ind
                           where=(upcase(change_current_ind) eq 'Y'));

  /* track the party_key/entity_watch_list_number combos */
  /* so as not to output duplicate rows */
  length ewln $35;
  if _n_ = 1 then do;
   declare hash ptyewl();
   rc = ptyewl.defineKey('party_number','ewln');
   rc = ptyewl.defineDone();
  end;

  rundate=input(put(&runasofdate,8.),yymmdd8.);
  entity_level_code='PTY';

  if upcase(party_type_desc) eq 'INDIVIDUAL' then do;

    last_name=upcase(compbl(kstrip(party_last_name)));
    match_code_individual_r=kstrip(match_code_individual);

    if upcase("&WATCH_LIST_PROCESSING_METHOD") eq 'HASH_ARRAY' then do;
       rc_ind_n=h_wl_ind_name.find();
       rc_ind_m=h_wl_ind_match.find();
    end;
    else if  upcase("&WATCH_LIST_PROCESSING_METHOD") eq 'INDEX_ARRAY' then do;

       set stg_wtch.wl_ind_name_d key=last_name / unique;

       if _iorc_ eq 0 then
          rc_ind_n=0;
       else do;
          rc_ind_n=1;
          _error_=0;
       end;

       set stg_wtch.wl_ind_match_d key=match_code_individual / unique;

       if _iorc_ eq 0 then
          rc_ind_m=0;
       else do;
          rc_ind_m=1;
         _error_=0;
       end;
    end;
  end;
  else if upcase(party_type_desc) eq 'ORGANIZATION' then do;

    entity_name=upcase(compbl(kstrip(party_name)));
    match_code_organization_r=kstrip(match_code_organization);

    if upcase("&WATCH_LIST_PROCESSING_METHOD") eq 'HASH_ARRAY' then do;
       rc_org_n=h_wl_org_name.find();
       rc_org_m=h_wl_org_match.find();
    end;
    else if upcase("&WATCH_LIST_PROCESSING_METHOD") eq 'INDEX_ARRAY' then do;

       set stg_wtch.wl_org_name_d key=entity_name / unique;

       if _iorc_ eq 0 then
          rc_org_n=0;
       else do;
          rc_org_n=1;
          _error_=0;
       end;

       set stg_wtch.wl_org_match_d key=match_code_organization / unique;

       if _iorc_ eq 0 then
          rc_org_m=0;
       else do;
          rc_org_m=1;
         _error_=0;
       end;
    end;
  end;

%mend aml_party_watchlist_header;


%macro aml_run_party_watchlist_header;

  /* Set module */
  %let _amlmodule_=PARTY_WATCHLIST_HEADER;

  /* Set error code to 0 and message to blank */
  %let trans_rc=0;
  %let _fcfmsg_=;

  %if %sysfunc(exist(seg_kc.fsc_entity_watch_list_dim)) ne 1 %then %do;
     %let _fcfmsg_=ERROR: AML: SEG_KC.FSC_ENTITY_WATCH_LIST_DIM does not exist. Ending execution.;
     %let trans_rc=999999;
     %fcf_put_error;
     %goto EXIT;
  %end;
  %else %do;
     proc sql noprint;
       select count(1) into :numobs
         from seg_kc.fsc_entity_watch_list_dim;
     quit;
     %if %eval(&numobs) eq 0 %then %do;
        %let _fcfmsg_=ERROR: AML: SEG_KC.FSC_ENTITY_WATCH_LIST_DIM is empty. Ending execution.;
        %let trans_rc=999999;
        %fcf_put_error;
        %goto EXIT;
     %end;
  %end;
  
  %aml_party_watchlist_header;

  %EXIT:;

%mend aml_run_party_watchlist_header;

%aml_run_party_watchlist_header;
