/*==============================================================================
|                               SAS SCENARIO FILE
|
| PRODUCT: SAS Anti-Money Laundering
| SYSTEM:  UNIX/WINDOWS
================================================================================
| NAME:            | SAS10225
|                  |
| CATEGORY:        | Unusual Aggregate Behavior
|                  |
| SEVERITY:        | Risk Factor
|                  |
| FREQUENCY:       | Daily
|                  |
| DURATION:        | 1 day
|                  |
| SUBJECT:         | Customer
|                  |
| PREP FILE:       | PARTY_TRANSACTIONS
|                  |
| HEADER FILE:     | party_header
|                  |
| TRIGGER:         | Yes
|                  |
| DAYS CONSIDERED: | Business
================================================================================

SHORT DESCRIPTION
-----------------
Transactions in Round Amounts by Risk Classification

DESCRIPTION
-----------
A customer's transaction amounts are dominated by round amounts, that is, no
cents, tens of dollars, hundreds of dollars, etc.

BUSINESS NOTES
--------------
This risk factor calculates the sum of all transaction amounts and the
transaction amounts with a specified number of trailing zeroes. If a customer's
percentage of round transactions exceeds the threshold, a match is found. This
scenario considers a customer's risk classification, whereas SAS10025 does not.
This scenario considers business days.

TECHNICAL NOTES
---------------
Not Applicable

MESSAGE
-------
r10225_threshold_message = %str("Round Transactions Percentage = #1");

SAMPLE MESSAGE
--------------
Round Transactions Percentage = 100%

================================================================================
PARAMETERS

Name                              Type                Default Value
---------------------------------|-------------------|--------------------------
 p10225_account_type_desc        | Character List    | 'P'
 p10225_min_round_amount_pct     | Numeric Constant  | 50
 p10225_risk_classification      | Numeric List      | 3
 p10225_num_days                 | Numeric Constant  | 20
 p10225_significant_digits       | Numeric Constant  | 3

Descriptions
------------
- p10225_account_type_desc
List denoting valid types of accounts

- p10225_min_round_amount_pct
Minimum percentage of round transactions

- p10225_risk_classification
List denoting risk classifications for stratification

- p10225_num_days
Number of business days in interval

- p10225_significant_digits
Number of trailing zeroes for an amount to be considered round

================================================================================

================================================================================
Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
- All rights reserved
==============================================================================*/
%let this_macro_name = &sysmacroname;  /* save this macro name */
/*----------------------------------------------------------------------------*/

retain p10225_begin_date;

/* This init section should be placed before any goto statements.  This section of
   code will only be run when processing the first BY group of the prep data set.
   Any goto statements placed before it may prevent the init code from being run. */
if run_scenario_init eq 1 then do;
  p10225_begin_date = input(put(rundate_number-&p10225_num_days+1,num_to_date.),8.) ;
end;

/* Exclude accounts based on site-specific requirements */
if risk_classification in (&p10225_risk_classification) then do;
    p10225_tran_total = 0;
    p10225_round_total = 0;

    do i = n to 1 by -1 while (date_key{i} >= p10225_begin_date);
        if p10225_begin_date <= date_key{i} <= rundate and
           upcase(account_type_desc{i}) in (&p10225_account_type_desc)
        then do;
            p10225_tran_total + currency_amount{i};
            if currency_amount{i}/10**(&p10225_significant_digits-2) = int(currency_amount{i}/10**(&p10225_significant_digits-2)) then do;
                p10225_round_total + currency_amount{i};
                %fcf_save_transaction_key (calling_macro=&this_macro_name);
            end;
        end;
    end;

    /* Compare party values to parameters for a match */
    if p10225_tran_total > 0 then do;
        p10225_round_pct = p10225_round_total / p10225_tran_total;
        if p10225_round_pct >= &p10225_min_round_amount_pct/100 then do;
            actual_values_text = tranwrd(&r10225_threshold_message,'#1',kstrip(put(p10225_round_pct,nlpct8.0)) );
            output &alert_fname;
        end;
    end;
end;
else goto &sysmacroname.;
/* EXIT */
&sysmacroname.:;


/* ========================================================================== */
/* END SCENARIO - SAS10225                                                    */
/* ========================================================================== */
