/*==============================================================================
|                               SAS SAMPLE FILE
|
| NAME:    fcf_ctr_prep.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  UNIX/WINDOWS
================================================================================


PREP FILE DESCRIPTION
---------------------
Creates party transactions and household transactions for CTR prep data sets

PREP DATA SET(S) CREATED
------------------------
mst_prep.ctr_pty_transactions
mst_prep.ctr_hh_transactions
mst_prep.ctr_executing_pty_trans
mst_prep.ctr_executing_ext_pty_trans


PREP DATA SET(S) SORT ORDER
----------------------------

| Data Set                         Field                           Sort Order
|--------------------------------|-------------------------------|--------------
|ctr_pty_transactions            |PARTY_NUMBER                   |1
|ctr_pty_transactions            |DATE_KEY                       |2

| Data Set                         Field                           Sort Order
|--------------------------------|-------------------------------|--------------
|ctr_hh_transactions             |HOUSEHOLD_NUMBER               |1
|ctr_hh_transactions             |DATE_KEY                       |2

| Data Set                         Field                           Sort Order
|--------------------------------|-------------------------------|--------------
|ctr_executing_pty_trans         |PARTY_NUMBER                   |1
|ctr_executing_pty_trans         |DATE_KEY                       |2

| Data Set                         Field                           Sort Order
|--------------------------------|-------------------------------|--------------
|ctr_executing_ext_pty_trans     |PARTY_NUMBER                   |1
|ctr_executing_ext_pty_trans     |DATE_KEY                       |2



================================================================================
SCENARIO(S) SUPPORTED:

- SAS10005CO
- SAS10005CI
- SAS10005COKA
- SAS10005CIKA
- SAS10005COC
- SAS10005COTC
- SAS10005CITC

==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 ---------------------------------------------------------------------*/

%macro fcf_ctr_prep;

    /*------------------------------------------
     * Set error code to 0 and message to blank
      *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

    %fcf_get_runasofdate;

    /*------------------------------------------------
     * %let step=Cleanup mst_prep views;
      *------------------------------------------------*/

    proc datasets lib=mst_prep nolist nowarn memtype=data;
        delete ctr_pty_transactions
            ctr_hh_transactions
            ctr_executing_pty_trans
            ctr_executing_ext_pty_trans;
    quit;

    %fcf_rcset(&syserr)

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmodule_=FCF_CTR_;
        %let _fcfmsg_=ERROR: FCF: fcf_ctr_prep.sas failed check log.;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*------------------------------------------------
     * %let step=Creation of mst_prep.ctr_pty_transactions;
      *------------------------------------------------*/

    proc sql;
        connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain."
            &segKCDBConnOpts);
        create table mst_prep.ctr_pty_transactions(compress=yes) as
            select 
                account_number length=50,
                account_type_desc length=20,
                currency_based_account_ind length=1,
                party_number length=50,
                input(put(date_key,8.),yymmdd8.) as date_key format=yymmddn8. length=8,
                entity_segment_id,
                transaction_cdi_code length=1,
                primary_medium_desc length=20,
                secondary_medium_desc length=20,
                mechanism_desc length=20,
                status_desc length=20,
                currency_amount length=8,
                transaction_key length=8,
                beneficiary_ext_party_key length=8,
                executing_party_key length=8,
                executing_ext_party_key length=8,
                party_key length=8
            from connection to &dbflavor
                (select distinct
                    Innr.*
                from
                    (select
                        ACC.account_number,
                        ACC.account_type_desc,
                        ACC.currency_based_account_ind,
                        PTY.party_number,
                        PTY.party_key,
                        PTY.entity_segment_id,
                        TRA_TYP.transaction_cdi_code,
                        TRA_TYP.primary_medium_desc,
                        TRA_TYP.secondary_medium_desc,
                        TRA_TYP.mechanism_desc,
                        TRA_STA.status_desc,
                        CSHFLW.date_key,
                        CSHFLW.currency_amount,
                        CSHFLW.transaction_key,
                        CSHFLW.beneficiary_ext_party_key,
                        CSHFLW.executing_party_key,
                        CSHFLW.executing_ext_party_key
                    from
                        &segKCSchema..fsc_account_dim ACC,
                        &segKCSchema..fsc_party_dim PTY,
                        &segKCSchema..fsc_transaction_type_dim TRA_TYP,
                        &segKCSchema..fsc_transaction_status_dim TRA_STA,
                        &segKCSchema..fsc_party_account_bridge PTY_ACC,
                        &segKCSchema..fsc_cash_flow_fact CSHFLW
                where
                    ACC.account_key = CSHFLW.account_key and
                    ACC.account_key = PTY_ACC.account_key and
                    PTY.party_key = PTY_ACC.party_key and
                    PTY_ACC.change_current_ind='Y' and
                    PTY_ACC.role_key=1 and
                    TRA_TYP.transaction_type_key = CSHFLW.transaction_type_key and
                    TRA_STA.transaction_status_key = CSHFLW.transaction_status_key and
                    CSHFLW.executing_party_key = -1 and
                    CSHFLW.executing_ext_party_key = -1 and
                    CSHFLW.account_key not in
                    (select
                        account_key 
                    from
                        &segKCSchema..fsc_household_account_bridge
                    where
                        change_current_ind='Y'
                    ) and
                    CSHFLW.date_key=&runasofdate
                    ) Innr
                )
            order by party_number, date_key;
        disconnect from &dbflavor;
    quit;

    %fcf_rcset(&sqlrc)

    %if &trans_rc eq 3 or &trans_rc ge 5 %then

        %do;
            %let _fcfmodule_=FCF_CTR_;
            %let _fcfmsg_=ERROR: FCF: Error creating mst_prep.ctr_pty_transactions;

            %fcf_put_error;
            %goto MACRO_END;
        %end;

    /*------------------------------------------------
     * %let step=Creation of mst_prep.ctr_hh_transactions;
     *------------------------------------------------*/
    proc sql;
        connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain."
            &segKCDBConnOpts);
        create table mst_prep.ctr_hh_transactions(compress=yes) as
            select
                account_number length=50,
                account_type_desc length=20,
                currency_based_account_ind length=1,
                household_number length=50,
                input(put(date_key,8.),yymmdd8.) as date_key format=yymmddn8. length=8,
                transaction_cdi_code length=1,
                primary_medium_desc length=20,
                secondary_medium_desc length=20,
                mechanism_desc length=20,
                status_desc length=20,
                currency_amount length=8,
                transaction_key length=8,
                beneficiary_ext_party_key length=8,
                executing_party_key length=8,
                executing_ext_party_key length=8
            from connection to &dbflavor
                (select distinct
                    Innr.*
                from
                    (select
                        ACC.account_number,
                        ACC.account_type_desc,
                        ACC.currency_based_account_ind,
                        HHD.household_number,
                        TRA_TYP.transaction_cdi_code,
                        TRA_TYP.primary_medium_desc,
                        TRA_TYP.secondary_medium_desc,
                        TRA_TYP.mechanism_desc,
                        TRA_STA.status_desc,
                        CSHFLW.date_key,
                        CSHFLW.currency_amount,
                        CSHFLW.transaction_key,
                        CSHFLW.beneficiary_ext_party_key,
                        CSHFLW.executing_party_key,
                        CSHFLW.executing_ext_party_key
                    from
                        &segKCSchema..fsc_account_dim ACC,
                        &segKCSchema..fsc_household_dim HHD,
                        &segKCSchema..fsc_transaction_type_dim TRA_TYP,
                        &segKCSchema..fsc_transaction_status_dim TRA_STA,
                        &segKCSchema..fsc_cash_flow_fact CSHFLW,
                        &segKCSchema..fsc_household_account_bridge HHD_ACC
                    where
                        ACC.account_key = CSHFLW.account_key and
                        ACC.account_key = HHD_ACC.account_key and
                        HHD.household_key = HHD_ACC.household_key and
                        HHD_ACC.change_current_ind = 'Y' and
                        TRA_TYP.transaction_type_key = CSHFLW.transaction_type_key and
                        TRA_STA.transaction_status_key = CSHFLW.transaction_status_key and
                        CSHFLW.executing_party_key = -1 and
                        CSHFLW.executing_ext_party_key = -1 and
                        CSHFLW.date_key=&runasofdate
                    ) Innr
                )
            order by household_number, date_key;
        disconnect from &dbflavor;
    quit;

    %fcf_rcset(&sqlrc)

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmodule_=FCF_CTR_;
        %let _fcfmsg_=ERROR: FCF: Error creating mst_prep.ctr_hh_transactions;

        %fcf_put_error;
        %goto MACRO_END;
    %end;
    
    
    /*------------------------------------------------
     * %let step=mst_prep.ctr_executing_pty_trans;
     *------------------------------------------------*/
    proc sql;
        connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain."
                                           &segKCDBConnOpts);
        create table mst_prep.ctr_executing_pty_trans(compress=yes) as
        select
           account_number length=50,
           account_type_desc length=20,
           currency_based_account_ind length=1,
           party_number length=50,
           input(put(date_key,8.),yymmdd8.) as date_key format=yymmddn8. length=8,
           entity_segment_id,
           transaction_cdi_code length=1,
           primary_medium_desc length=20,
           secondary_medium_desc length=20,
           mechanism_desc length=20,
           status_desc length=20,
           currency_amount length=8,
           transaction_key length=8,
           beneficiary_ext_party_key length=8,
           executing_party_key length=8,
           executing_ext_party_key length=8,
           party_key length=8
        from connection to &dbflavor
          (select distinct Innr.*
           from
              (select
                  ACC.account_number,
                  ACC.account_type_desc,
                  ACC.currency_based_account_ind,
                  PTY.party_number,
                  PTY.party_key,
                  PTY.entity_segment_id,
                  TRA_TYP.transaction_cdi_code,
                  TRA_TYP.primary_medium_desc,
                  TRA_TYP.secondary_medium_desc,
                  TRA_TYP.mechanism_desc,
                  TRA_STA.status_desc,
                  CSHFLW.date_key,
                  CSHFLW.currency_amount,
                  CSHFLW.transaction_key,
                  CSHFLW.beneficiary_ext_party_key,
                  CSHFLW.executing_party_key,
                  CSHFLW.executing_ext_party_key
               from
                  &segKCSchema..fsc_account_dim ACC,
                  &segKCSchema..fsc_party_dim PTY,
                  &segKCSchema..fsc_transaction_type_dim TRA_TYP,
                  &segKCSchema..fsc_transaction_status_dim TRA_STA,
                  &segKCSchema..fsc_cash_flow_fact CSHFLW
                where
                  ACC.account_key = CSHFLW.account_key and
                  PTY.party_key = CSHFLW.executing_party_key and
                  PTY.change_current_ind = 'Y' and
                  TRA_TYP.transaction_cdi_code = 'D' and
                  TRA_TYP.transaction_type_key = CSHFLW.transaction_type_key and
                  TRA_STA.transaction_status_key = CSHFLW.transaction_status_key and
                  CSHFLW.executing_party_key <> -1 and
                  CSHFLW.date_key=&runasofdate
               ) Innr
        )
        order by party_number, date_key;
        disconnect from &dbflavor;
    quit;

    %fcf_rcset(&sqlrc)

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmodule_=FCF_CTR_;
        %let _fcfmsg_=ERROR: FCF: Error creating mst_prep.ctr_executing_pty_transactions;

        %fcf_put_error;
        %goto MACRO_END;
    %end;     
    
    
    
    /*------------------------------------------------
     * %let step=mst_prep.ctr_executing_ext_pty_trans;
     *------------------------------------------------*/
    proc sql;
        connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain."
                                           &segKCDBConnOpts);
    create table mst_prep.ctr_executing_ext_pty_trans(compress=yes) as
        select
           account_number length=50,
           account_type_desc length=20,
           currency_based_account_ind length=1,
           external_party_number length=50,
           input(put(date_key,8.),yymmdd8.) as date_key format=yymmddn8. length=8,
           transaction_cdi_code length=1,
           primary_medium_desc length=20,
           secondary_medium_desc length=20,
           mechanism_desc length=20,
           status_desc length=20,
           currency_amount length=8,
           transaction_key length=8,
           transaction_description length=255,
           beneficiary_ext_party_key length=8,
           executing_party_key length=8,
           put(executing_ext_party_key,15.) length=80 as executing_ext_party_key
        from connection to &dbflavor
          (select distinct Innr.*
           from
              (select
                   ACC.account_number,
                   ACC.account_type_desc,
                   ACC.currency_based_account_ind,
                   EXT.external_party_number,
                   TRA_TYP.transaction_cdi_code,
                   TRA_TYP.primary_medium_desc,
                   TRA_TYP.secondary_medium_desc,
                   TRA_TYP.mechanism_desc,
                   TRA_STA.status_desc,
                   TRA.transaction_description,
                   CSHFLW.currency_amount,
                   CSHFLW.transaction_key,
                   CSHFLW.beneficiary_ext_party_key,
                   CSHFLW.executing_party_key,
                   CSHFLW.executing_ext_party_key,
                   CSHFLW.date_key
               from
                   &segKCSchema..fsc_account_dim ACC,
                   &segKCSchema..fsc_ext_party_account_dim EXT,
                   &segKCSchema..fsc_transaction_type_dim TRA_TYP,
                   &segKCSchema..fsc_transaction_status_dim TRA_STA,
                   &segKCSchema..fsc_transaction_dim TRA,
                   &segKCSchema..fsc_cash_flow_fact CSHFLW
                   
               where
                   ACC.account_key = CSHFLW.account_key and
                   EXT.ext_party_account_key = CSHFLW.executing_ext_party_key and
                   TRA_TYP.transaction_type_key = CSHFLW.transaction_type_key and
                   TRA_TYP.transaction_cdi_code in ('D','C') and
                   TRA_STA.transaction_status_key = CSHFLW.transaction_status_key and
                   TRA.transaction_key = CSHFLW.transaction_key and
                   CSHFLW.executing_party_key = -1 and
                   CSHFLW.executing_ext_party_key <> -1 and
                   CSHFLW.date_key=&runasofdate

               ) Innr
            )
        order by external_party_number, date_key;
        disconnect from &dbflavor;
    quit;

    %fcf_rcset(&sqlrc)

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmodule_=FCF_CTR_;
        %let _fcfmsg_=ERROR: FCF: Error creating mst_prep.ctr_executing_ext_pty_trans;

        %fcf_put_error;
        %goto MACRO_END;
    %end;     
    
%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else
        %do;
            %fcf_execute_macro(fcf_success);
        %end;
%mend;

%fcf_ctr_prep;
