/*==============================================================================
|                               SAS SAMPLE FILE
|
| NAME:    transactions_prep.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================


PREP FILE DESCRIPTION
---------------------
Creates party/account/associate transactions prep data sets


PREP FILE NOTES
---------------
1. The first part of the code generates a daily prep data set containing the
denormalized transactions for the given run date. The file name is appended
with the runasofdate; e.g. account_trans20040101.  The second
portion of the program creates views across the daily data sets. The number of
data sets used for the view is determined by querying the FSK_SCENARIO,
FSK_HEADER and FSK_SCENARIO_PARAMETER tables.

2. This code assumes that the account view will be named ACCOUNT_TRANSACTIONS
and the party view named PARTY_TRANSACTIONS, and the associate view named
ASSOCIATE_TRANSACTIONS.

3. This code converts the value of runasofdate to a SAS date value to derive
date_key. The derived value of date_key is then output to the prep data set as
required by dependent scenario(s).

The value of date_key is not converted from an integer value in yyyymmdd format
as used in the where clause(s), as this value is compared to runasofdate which
is also an integer value in yyyymmdd format.


PREP DATA SET(S) CREATED
------------------------
MST_PREP.ACCOUNT_TRANSACTIONS
MST_PREP.PARTY_TRANSACTIONS
MST_PREP.ASSOCIATE_TRANSACTIONS

PREP DATA SET(S) SORT ORDER
----------------------------

| Data Set                         Field                           Sort Order
|--------------------------------|-------------------------------|--------------
|ACCOUNT_TRANSACTIONS            |ACCOUNT_NUMBER                 |1
|ACCOUNT_TRANSACTIONS            |DATE_KEY                       |2
|PARTY_TRANSACTIONS              |PARTY_NUMBER                   |1
|PARTY_TRANSACTIONS              |DATE_KEY                       |2
|ASSOCIATE_TRANSACTIONS          |ASSOCIATE_NUMBER               |1
|ASSOCIATE_TRANSACTIONS          |DATE_KEY                       |2

PREP DATA SET(S) NOTES
----------------------
None


================================================================================
SCENARIO(S) SUPPORTED:

- SAS10001
- SAS10002
- SAS10003
- SAS10004
- SAS10005
- SAS10006
- SAS10007
- SAS10009
- SAS10014
- SAS10015
- SAS10016
- SAS10017
- SAS10018
- SAS10019
- SAS10020
- SAS10021
- SAS10022
- SAS10025
- SAS10026
- SAS10027
- SAS10028
- SAS10029
- SAS10030
- SAS10031
- SAS10032
- SAS10033
- SAS10034
- SAS10035
- SAS10040
- SAS10041
- SAS10078
- SAS10081
- SAS10086
- SAS10087
- SAS10088
- SAS10089
- SAS10090
- SAS10091
- SAS10140
- SAS10206
- SAS10220
- SAS10225
- SAS10230
==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

/*------------------------------------------------
 *        Cleanup mst_prep views
 *------------------------------------------------*/
 proc datasets lib=mst_prep nolist nowarn;
    delete account_transactions;
    delete party_transactions;
    delete associate_transactions;
 quit;

 /*------------------------------------------------------
  * Find earliest day in job calendar  
  * This will be used to see to check if enough history 
  * in the job calendar to create the views
  *------------------------------------------------------*/
  
  proc sql noprint;
     select put(datepart(min(calendar_date)),best.) into :first_job_calendar_date
     from seg_kc.fsk_job_calendar;
  quit;

  %put runasofdate=&runasofdate; 
  %put first_job_calendar_date=&first_job_calendar_date;

/* Retrieve daily data set from Core database */
 proc sql;
    connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain."
                                       &segKCDBConnOpts);
    create table mst_prep.account_trans&runasofdate as

    select
       account_number length=50,
       account_type_desc length=20,
       currency_based_account_ind length=1,
       account_tax_state_code length=3,
       party_number length=50,
       acc_entity_segment_id,
       pty_entity_segment_id,
       annual_income_amount length=8,
       last_susp_actv_rpt_date length=8,
       last_cash_trans_rpt_date length=8,
       politically_exposed_person_ind length=1,
       customer_since_date length=8,
       input("&runasofdate",yymmdd8.) as date_key format=yymmdd8. length=5,
       transaction_cdi_code length=1,
       primary_medium_desc length=20,
       secondary_medium_desc length=20,
       mechanism_desc length=20,
       street_state_code length=3,
       branch_number length=25,
       branch_type_desc length=10,
       country_code_3 length=3,
       currency_code length=3,
       status_desc length=20,
       status_reason_desc length=35,
       currency_amount length=8,
       transaction_key length=8,
       risk_classification length=3,
       associate_number length=20,
       expected_incoming_amount length=8,
       expected_outgoing_amount length=8,
       expected_incoming_count length=8,
       expected_outgoing_count length=8,
       increased_behavior_credit_amt length=8 ,
       increased_behavior_debit_amt length=8,
       increased_behavior_credit_cnt length=8,
       increased_behavior_debit_cnt length=8,
       expected_credit_ceiling_amount length=8,
       expected_debit_ceiling_amount length=8

    from connection to &dbflavor
      (select
          Innr.*,
          AA.expected_incoming_amount,
          AA.expected_outgoing_amount,
          AA.expected_incoming_count,
          AA.expected_outgoing_count,
          AA.increased_behavior_credit_amt ,
          AA.increased_behavior_debit_amt,
          AA.increased_behavior_credit_cnt,
          AA.increased_behavior_debit_cnt ,
          AA.expected_credit_ceiling_amount,
          AA.expected_debit_ceiling_amount
       from
          (select
              A.account_number,
              A.account_type_desc,
              A.currency_based_account_ind,
              A.account_tax_state_code,
              B.party_number,
              A.entity_segment_id as acc_entity_segment_id,
              B.entity_segment_id as pty_entity_segment_id,
              B.annual_income_amount,
              B.last_susp_actv_rpt_date,
              B.last_cash_trans_rpt_date,
              B.politically_exposed_person_ind,
              B.customer_since_date,
              B.risk_classification,
              C.transaction_cdi_code,
              C.primary_medium_desc,
              C.secondary_medium_desc,
              C.mechanism_desc,
              D.street_state_code,
              D.branch_number,
              D.branch_type_desc,
              E.country_code_3,
              F.currency_code,
              G.status_desc,
              G.status_reason_desc,
              I.currency_amount,
              I.transaction_key,
              L.associate_number
           from
              &segKCSchema..fsc_account_dim A,
              &segKCSchema..fsc_party_dim B,
              &segKCSchema..fsc_transaction_type_dim C,
              &segKCSchema..fsc_branch_dim D,
              &segKCSchema..fsc_country_dim E,
              &segKCSchema..fsc_currency_dim F,
              &segKCSchema..fsc_transaction_status_dim G,
              &segKCSchema..fsc_party_account_bridge H,
              &segKCSchema..fsc_cash_flow_fact I,
              &segKCSchema..fsc_associate_dim L
           where
              I.account_key = A.account_key and
              A.account_key = H.account_key and
              B.party_key = H.party_key and
              I.transaction_type_key = C.transaction_type_key and
              I.transaction_status_key = G.transaction_status_key and
              I.country_key = E.country_key and
              I.branch_key = D.branch_key and
              I.transaction_currency_key = F.currency_key and
              I.date_key = &runasofdate and
              H.role_key = 1 and
              H.change_current_ind = 'Y' and
              I.associate_key = L.associate_key
          ) Innr

       left join &segKCSchema..fsc_account_analysis_dim AA
       on AA.account_number=Innr.account_number

       union all

       select
          Innr.*,
          AA.expected_incoming_amount,
          AA.expected_outgoing_amount,
          AA.expected_incoming_count,
          AA.expected_outgoing_count,
          AA.increased_behavior_credit_amt ,
          AA.increased_behavior_debit_amt,
          AA.increased_behavior_credit_cnt,
          AA.increased_behavior_debit_cnt ,
          AA.expected_credit_ceiling_amount,
          AA.expected_debit_ceiling_amount
       from (select
                A.account_number,
                A.account_type_desc,
                A.currency_based_account_ind,
                A.account_tax_state_code,
                B.party_number,
                A.entity_segment_id as acc_entity_segment_id,
                B.entity_segment_id as pty_entity_segment_id,
                B.annual_income_amount,
                B.last_susp_actv_rpt_date,
                B.last_cash_trans_rpt_date,
                B.politically_exposed_person_ind,
                B.customer_since_date,
                B.risk_classification,
                C.transaction_cdi_code,
                C.primary_medium_desc,
                C.secondary_medium_desc,
                C.mechanism_desc,
                D.street_state_code,
                D.branch_number,
                D.branch_type_desc,
                E.country_code_3,
                '' as currency_code,
                G.status_desc,
                G.status_reason_desc,
                0 as currency_amount,
                I.transaction_key,
                'UNKNOWN' as associate_number
             from
                &segKCSchema..fsc_account_dim A,
                &segKCSchema..fsc_party_dim B,
                &segKCSchema..fsc_transaction_type_dim C,
                &segKCSchema..fsc_branch_dim D,
                &segKCSchema..fsc_country_dim E,
                &segKCSchema..fsc_transaction_status_dim G,
                &segKCSchema..fsc_party_account_bridge H,
                &segKCSchema..fsc_account_event_fact I

             where
                I.account_key = A.account_key and
                A.account_key = H.account_key and
                B.party_key = H.party_key and
                I.transaction_type_key = C.transaction_type_key and
                I.transaction_status_key = G.transaction_status_key and
                I.country_key = E.country_key and
                I.branch_key = D.branch_key and
                I.date_key = &runasofdate and
                H.role_key = 1 and
                H.change_current_ind = 'Y'
            ) Innr

       left join &segKCSchema..fsc_account_analysis_dim AA
       on AA.account_number=Innr.account_number
       )
       order by account_number, date_key;

    disconnect from &dbflavor;
 quit;

/* Sort to create party daily */
 proc sort data=mst_prep.account_trans&runasofdate
           out=mst_prep.party_trans&runasofdate;
   by party_number date_key;
  run;


/*----------------------------------
 * Create ACCOUNT_TRANSACTIONS view
 *----------------------------------*/
 %macro fcf_create_acct_trans_view;

    /* Determine number of days to chain daily prep files together.*/
     proc sql noprint;
        select max(input(sp.parm_value,best32.)) into :numdays
          from seg_kc.fsk_header h,
               seg_kc.fsk_scenario s,
               seg_kc.fsk_scenario_parameter sp
         where h.header_id=s.header_id and
               upcase(h.denorm_table_name) = "ACCOUNT_TRANSACTIONS" and
               upcase(s.scenario_status_code) = "ACT" and
               upcase(s.current_ind) = "Y" and
               upcase(s.logical_delete_ind) ne 'Y' and
               s.scenario_id = sp.scenario_id and
               sp.parm_name like 'p%_num_days'
               ;
     quit;
     %let numdays=&numdays;  /*trim blanks*/
     /*Some scenarios (for example SAS10001) don't have a num_days
       parameter but still need one day of transactions.*/
     %if &numdays eq . %then %let numdays = 1;
     %put Account numdays: &numdays;

     proc sql noprint;
        select count(*) into :default_num_days_zero
          from seg_kc.fsk_header h,
               seg_kc.fsk_scenario s,
               seg_kc.fsk_scenario_parameter sp
         where h.header_id=s.header_id and
               upcase(h.denorm_table_name) = "ACCOUNT_TRANSACTIONS" and
               upcase(s.scenario_status_code) = "ACT" and
               upcase(s.current_ind) = "Y" and
               upcase(s.logical_delete_ind) ne 'Y' and
               s.scenario_id = sp.scenario_id and
               sp.parm_name like 'p%_num_days' and
               sp.parm_value = '0';
               ;
     quit;
     %let default_num_days_zero=&default_num_days_zero;  /*trim blanks*/
     %if &default_num_days_zero eq . %then %let default_num_days_zero = 0;
     %put Account_header scenarios with default number of days equal to zero: &default_num_days_zero;

     %if &default_num_days_zero gt 0 %then %do;
       %put These scenarios need month-to-runasofdate number of days of transactions.;
       %put Adjust numdays if necessary.;
       data _null_;
         runasofdate = input(put(&runasofdate,8.),yymmdd8.);
         put 'runasofdate: ' runasofdate nldate.;
         startmonth = runasofdate - day(runasofdate) + 1;
         put 'Start of month: ' startmonth nldate.;
         num_days = runasofdate - startmonth + 1;
         put 'Days month-to-runasofdate: ' num_days;
         final_num_days = max(num_days, &numdays);
         call symput('numdays',kstrip(put(final_num_days, best32.)));
        run;
        %put Final Account numdays: &numdays;
     %end;
     %else %do;
         %put Final Account numdays: &numdays;
     %end;

    /* Check for existence of numdays.  Ensure that it is >= 1      */
    /* If numdays exists, check the job calendar has enough history */
     data _null_;
        if symexist("numdays") then do;
           /* find first day of view */
           currentday = input(put(&runasofdate,8.),yymmdd8.);
           currentday_number = input(put(currentday,date_to_num.),8.);
           first_day = input(put(currentday_number - &numdays,num_to_date.),8.);

           /* Check data needed for scenario */
           if &numdays le 0 then do;
              put "LOADNOTE:  Prep file for account_transactions could not be built.";
              put "LOADNOTE:  Possible cause: No ACTive ACCOUNT scenarios.";
              call symput('active',"NO");
           end;
           else if first_day < &first_job_calendar_date then do;
              put "LOADNOTE:  Prep file for account_transactions could not be built.";
              put "WARNING:   Job Calendar does not include enough history.";
              call symput('active',"NO");
           end;
           else call symput('active',"YES");
        end;
        else do;
           call symput('active',"NO");
           put "LOADNOTE:  Prep file for account_transactions could not be built";
        end;
     run;

     /* Check for existence of numdays.  Ensure that it is >= 1 */

     %if &active = YES %then %do;

       /* Build list of daily prep files for merge */
        data _null_;
          currentday = input(put(&runasofdate,8.),yymmdd8.);

          do i = 0 to &numdays - 1;
            currentday_number = input(put(currentday,date_to_num.),8.);
            if currentday_number eq -1 then do;    /*non-business day*/
              currentday_prev = currentday;
              do until (currentday_number ne -1);
                currentday_prev = currentday_prev - 1;
                currentday_number = input(put(currentday_prev,date_to_num.),8.);
              end;
            end;

            current_business_day = input(put(currentday_number, num_to_date.),8.);
            call symput('daydate'||left(trim(put(i,best32.))),'mst_prep.account_trans'||put(current_business_day,yymmddn8.));

            currentday = current_business_day - 1;
          end;
         run;

        %let no_ds_count = 0;
        %do i = 0 %to (&numdays - 1);
           %if not %sysfunc(exist(&&daydate&i)) %then %do;
             %put WARNING: Cannot find account daily transaction data set &&daydate&i;
             %let no_ds_count = %eval(&no_ds_count+1); 
           %end;
        %end;
        %if &no_ds_count = &numdays %then %do;
          %put ERROR: Cannot find any of the account daily transaction data sets.;
          %abort cancel 4;
        %end;

       /* Build and execute merge code that creates view across each daily
          trans file */
       /* Drop party_key and country_code_3 -- used by party prep file */
        data mst_prep.account_transactions(rename=(acc_entity_segment_id = entity_segment_id)
                                           drop=country_code_3
                                                party_number
                                                annual_income_amount
                                                currency_based_account_ind
                                                customer_since_date
                                                last_cash_trans_rpt_date
                                                last_susp_actv_rpt_date
                                                risk_classification
                                                associate_number
                                                politically_exposed_person_ind
                                                pty_entity_segment_id
                                                )
                                          /view=mst_prep.account_transactions;
           merge
           %do i = 0 %to (&numdays - 1);
              %if %sysfunc(exist(&&daydate&i)) %then  &&daydate&i ;
           %end;
           ;
           by account_number date_key;
        run;
     %end; /* End active=YES */

 %mend fcf_create_acct_trans_view;
 %fcf_create_acct_trans_view;


/*----------------------------------
 * Create PARTY_TRANSACTIONS view
 *----------------------------------*/
 %macro fcf_create_pty_trans_view;

   /* Determine number of days to chain daily prep files together.*/
     proc sql noprint;
        select max(input(sp.parm_value,best32.)) into :numdays
          from seg_kc.fsk_header h,
               seg_kc.fsk_scenario s,
               seg_kc.fsk_scenario_parameter sp
         where h.header_id=s.header_id and
               upcase(h.denorm_table_name) = "PARTY_TRANSACTIONS" and
               upcase(s.scenario_status_code) = "ACT" and
               upcase(s.current_ind) = "Y" and
               upcase(s.logical_delete_ind) ne 'Y' and
               s.scenario_id = sp.scenario_id and
               sp.parm_name like 'p%_num_days' and
               /* these scenarios don't need transactions */
               scenario_name not in ('SAS10032','SAS10034','SAS10035')
               ;
     quit;
     %let numdays=&numdays;  /*trim blanks*/
     /*Some scenarios don't have a num_days parameter
       but still need one day of transactions.*/
     %if &numdays eq . %then %let numdays = 1;
     %put Party numdays: &numdays;

    /* Check for existence of numdays.  Ensure that it is >= 1      */
    /* If numdays exists, check the job calendar has enough history */

     data _null_;
        if symexist("numdays") then do;
           /* find first day of view */
           currentday = input(put(&runasofdate,8.),yymmdd8.);
           currentday_number = input(put(currentday,date_to_num.),8.);
           first_day = input(put(currentday_number - &numdays,num_to_date.),8.); 

           /* Check data needed for scenario */
           if &numdays le 0 then do;
              put "LOADNOTE:  Prep file for account_transactions could not be built.";
              put "LOADNOTE:  Possible cause: No ACTive PARTY scenarios.";
              call symput('active',"NO");
           end;
           else if first_day < &first_job_calendar_date then do;
              put "LOADNOTE:  Prep file for party_transactions could not be built.";
               put "WARNING:  Job Calendar does not include enough history.";
              call symput('active',"NO");
           end;
           else call symput('active',"YES");
        end;
        else do;
           call symput('active',"NO");
           put "LOADNOTE:  Prep file for party_transactions could not be built";
        end;
     run;

    %if &active = YES %then %do;

      /* Build list of daily prep files for merge */
       data _null_;
         currentday = input(put(&runasofdate,8.),yymmdd8.);

         do i = 0 to &numdays - 1;
           currentday_number = input(put(currentday,date_to_num.),8.);
           if currentday_number eq -1 then do;    /*non-business day*/
             currentday_prev = currentday;
             do until (currentday_number ne -1);
               currentday_prev = currentday_prev - 1;
               currentday_number = input(put(currentday_prev,date_to_num.),8.);
             end;
           end;

           current_business_day = input(put(currentday_number, num_to_date.),8.);
           call symput('daydate'||left(trim(put(i,best32.))),'mst_prep.party_trans'||put(current_business_day,yymmddn8.));

           currentday = current_business_day - 1;
         end;
        run;

       %let no_ds_count = 0;
       %do i = 0 %to (&numdays - 1);
          %if not %sysfunc(exist(&&daydate&i)) %then %do;
            %put WARNING: Cannot find party daily transaction data set &&daydate&i;
            %let no_ds_count = %eval(&no_ds_count+1); 
          %end;
       %end;
       %if &no_ds_count = &numdays %then %do;
         %put ERROR: Cannot find any of the party daily transaction data sets.;
         %abort cancel 4;
       %end;

      /* Build and execute merge code that creates view across each daily
         trans file */
       data mst_prep.party_transactions (rename=(pty_entity_segment_id = entity_segment_id)
                                         drop=account_tax_state_code
                                              branch_number
                                              street_state_code
                                              branch_type_desc
                                              status_reason_desc
                                              associate_number
                                              expected_incoming_amount
                                              expected_outgoing_amount
                                              expected_incoming_count
                                              expected_outgoing_count
                                              increased_behavior_credit_amt
                                              increased_behavior_debit_amt
                                              increased_behavior_credit_cnt
                                              increased_behavior_debit_cnt
                                              expected_credit_ceiling_amount
                                              expected_debit_ceiling_amount
                                              acc_entity_segment_id)
                                        /view=mst_prep.party_transactions;
          merge
          %do i = 0 %to (&numdays - 1);
             %if %sysfunc(exist(&&daydate&i)) %then &&daydate&i;
          %end;
          ;
          by party_number date_key;
       run;
    %end; /* End active = YES */

 %mend fcf_create_pty_trans_view;
 %fcf_create_pty_trans_view;

 
/*------------------------------------
 * Create ASSOCIATE_TRANSACTIONS view
 *------------------------------------*/ 
 %macro fcf_create_asc_trans_view;

   /* Determine number of days to chain daily prep files together.*/
     proc sql noprint;
        select max(input(sp.parm_value,best32.)) into :numdays
          from seg_kc.fsk_header h,
               seg_kc.fsk_scenario s,
               seg_kc.fsk_scenario_parameter sp
         where h.header_id=s.header_id and
               upcase(h.denorm_table_name) = "ASSOCIATE_TRANSACTIONS" and
               upcase(s.scenario_status_code) = "ACT" and
               upcase(s.current_ind) = "Y" and
               upcase(s.logical_delete_ind) ne 'Y' and
               s.scenario_id = sp.scenario_id and
               sp.parm_name like 'p%_num_days'
               ;
     quit;
     %let numdays=&numdays;  /*trim blanks*/
     /*Some scenarios don't have a num_days parameter
       but still need one day of transactions.*/
     %if &numdays eq . %then %let numdays = 1;
     %put Associate numdays: &numdays;

     /* Check for existence of numdays.  Ensure that it is >= 1      */
    /* If numdays exists, check the job calendar has enough history */
     data _null_;
        if symexist("numdays") then do;
           /* find first day of view */
           currentday = input(put(&runasofdate,8.),yymmdd8.);
           currentday_number = input(put(currentday,date_to_num.),8.);
           first_day = input(put(currentday_number - &numdays,num_to_date.),8.);

           /* Check data needed for scenario */
           if &numdays le 0 then do;
              put "LOADNOTE:  Prep file for associate_transactions could not be built.";
              put "LOADNOTE:  Possible cause: No ACTive ASSOCIATE scenarios.";
              call symput('active',"NO");
           end;
           else if first_day < &first_job_calendar_date then do;
              put "LOADNOTE:  Prep file for associate_transactions could not be built.";
               put "WARNING:  Job Calendar does not include enough history.";
              call symput('active',"NO");
           end;
           else call symput('active',"YES");
        end;
        else do;
           call symput('active',"NO");
           put "LOADNOTE:  Prep file for account_transactions could not be built";
        end;
     run;

    %if &active = YES %then %do;

      data assoc_trans&runasofdate;
       set mst_prep.account_trans&runasofdate;
        if associate_number ne 'UNKNOWN';
       run;
       
      proc sort data=assoc_trans&runasofdate
            out=mst_prep.assoc_trans&runasofdate;
        by associate_number date_key;
       run;

      /* Build list of daily prep files for merge */
       data _null_;
         currentday = input(put(&runasofdate,8.),yymmdd8.);

         do i = 0 to &numdays - 1;
           currentday_number = input(put(currentday,date_to_num.),8.);
           if currentday_number eq -1 then do;    /*non-business day*/
             currentday_prev = currentday;
             do until (currentday_number ne -1);
               currentday_prev = currentday_prev - 1;
               currentday_number = input(put(currentday_prev,date_to_num.),8.);
             end;
           end;

           current_business_day = input(put(currentday_number, num_to_date.),8.);
           call symput('daydate'||left(trim(put(i,best32.))),'mst_prep.assoc_trans'||put(current_business_day,yymmddn8.));

           currentday = current_business_day - 1;
         end;
        run;

       %let no_ds_count = 0;
       %do i = 0 %to (&numdays - 1);
          %if not %sysfunc(exist(&&daydate&i)) %then %do;
            %put WARNING: Cannot find associate daily transaction data set &&daydate&i;
            %let no_ds_count = %eval(&no_ds_count+1); 
          %end;
       %end;
       %if &no_ds_count = &numdays %then %do;
         %put ERROR: Cannot find any of the associate daily transaction data sets.;
         %abort cancel 4;
       %end;

      /* Build and execute merge code that creates view across each daily
         trans file */
       data mst_prep.associate_transactions (drop=account_tax_state_code
                                                  branch_number
                                                  street_state_code
                                                  branch_type_desc
                                                  status_reason_desc
                                                  expected_incoming_amount
                                                  expected_outgoing_amount
                                                  expected_incoming_count
                                                  expected_outgoing_count
                                                  increased_behavior_credit_amt
                                                  increased_behavior_debit_amt
                                                  increased_behavior_credit_cnt
                                                  increased_behavior_debit_cnt
                                                  expected_credit_ceiling_amount
                                                  expected_debit_ceiling_amount
                                                  ACC_ENTITY_SEGMENT_ID
                                                  PTY_ENTITY_SEGMENT_ID)
                                            /view=mst_prep.associate_transactions;
          merge
          %do i = 0 %to (&numdays - 1);
             %if %sysfunc(exist(&&daydate&i)) %then &&daydate&i;
          %end;
          ;
          by associate_number date_key;
       run;
    %end; /* End active = YES */

 %mend fcf_create_asc_trans_view;
 %fcf_create_asc_trans_view;
