
/******************************************************************************/
/*                  S A S   S A M P L E   L I B R A R Y                       */
/*                                                                            */
/*    NAME: fcf_alert_investigation_extract.sas                               */
/*   TITLE: fcf_alert_investigation_extract                                   */
/* PRODUCT: SAS Anti-Money Laundering                                         */
/*  SYSTEM: UNIX/WINDOWS                                                      */
/******************************************************************************/

/* ************************************************************************** */
/*  Copyright (c) 2014 by SAS Institute Inc., Cary, NC 27513 USA              */
/*  ---All Rights Reserved.                                                   */
/* ************************************************************************** */
/*---------------------------------------------------------
 *  Host and Site specific customizations may be REQUIRED
 *---------------------------------------------------------*/

%macro fcf_alert_investigation_extract;

/*----------------------------------------------------------------------------*/
/* Configurable macro variable for unassinged alerts.
/*----------------------------------------------------------------------------*/
  %let NO_OWNER = No Owner;


/*----------------------------------------------------------------------------*/
/* Retrieve all AML Alerts from ECM.
/*----------------------------------------------------------------------------*/

  %if %sysfunc(exist(ecm_db.incident_live)) %then %do;
        proc sql;
              create table selected_alerts as
              select INCIDENT_RK,
               /* Derive ALERT_ID from INCIDENT_ID - key to FSK_ALERT Table   */
                     ((input((substrn(INCIDENT_ID,5)),best.))) AS ALERT_ID, 
                     VALID_from_DTTM,
                     VALID_TO_DTTM,
                     CASE_RK,
                     INCIDENT_ID,
                     SOURCE_SYSTEM_CD,
                     INCIDENT_TYPE_CD,
                     INCIDENT_CATEGORY_CD,
                     INCIDENT_SUBCATEGORY_CD,
                     INCIDENT_DESC,
                     INCIDENT_from_DT,
                     INCIDENT_from_TM,
                     INCIDENT_TO_DT,
                     INCIDENT_TO_TM,
                     DETECTION_DT,
                     DETECTION_TM,
                     NOTIFICATION_DT,
                     NOTIFICATION_TM,
                     UI_DEF_FILE_NM,
                     CREATE_USER_ID,
                     /* derive YEAR from CREATE_DTTM */
                     YEAR(datepart(CREATE_DTTM)) as YEAR label='Year', 
                     /* derive QUARTER from CREATE_DTTM */
                     left(put(datepart(CREATE_DTTM),yyq10.)) as QUARTER label='Quarter',
                     /* derive Month from CREATE_DTTM */
                     datepart(CREATE_DTTM) as MONTH format monyy7. Label='Month',
                     /* derive WEEK in mulitple formats for reporting display from CREATE_DTTM */
                     put(datepart(CREATE_DTTM), weeku5.) as WEEK label='Week format weeku5.',
                     intnx('week',datepart(CREATE_DTTM),0) as WEEKBEGINDATE format Worddate12. label='Week',
                     /* reformat CREATE_DTTM for date calculations and display */
                     datepart(CREATE_DTTM) format=date9. as CREATE_DTTM,
                     /* reformat CREATE_DTTM for date calculations and display */
                     datepart(CLOSE_DTTM) format=date9. as CLOSE_DTTM,
                     UPDATE_USER_ID,
                     VERSION_NO,
                     DELETE_FLG,
                     INVESTIGATOR_USER_ID,
                     INCIDENT_DISPOSITION_CD,
                     INCIDENT_STATUS_CD
              from ecm_db.incident_live
              where incident_type_cd = 'AMLALT';
              quit;
  %end;
  %else %do;
      %let trans_rc = 900;
      %let _fcfmsg_ = ecm_db.incident_live table not found, processing will end;  
      %fcf_put_error;
      %goto MACRO_END;
  %end;

/*----------------------------------------------------------------------------*/
/* Determine earliest date, total number of weeks and weeks span into 
/* macro varibles minDate, nbrWeeks and weekDelta respectively from
/* selected alerts.
/*----------------------------------------------------------------------------*/

  proc sql noprint;
      select min(create_dttm) into :minDate from selected_alerts;
      select count(distinct(week)) into :nbrWeeks from selected_alerts;
      select intck('week',minDate,today())+1 into :weekDelta
         from (select min(create_dttm) as minDate 
               from selected_alerts);
  quit;

/*----------------------------------------------------------------------------*/
/* Create Age groups buckets and logic to bucket selected alerts accordingly
/* from macro varibles minDate, nbrWeeks and weekDelta.
/*----------------------------------------------------------------------------*/

 %let DaysInPeriod = 7;
 DATA alert_age_range_list;
    label AGE_GRP_IN_DAYS = 'Age in Days'    ;
    length AGE_GRP_IN_DAYS $9. AGEGRPCHECK $250.;
    WEEK_NBR=1;
    tmpDate = &minDate + ((&weekDelta-1) * 7);
    START = 0;
    END = &DaysInPeriod - 1;
    AGE_GRP_PAD  = 4;
    AGEGRPCHECK = ' ALERT_AGE GE ' || trim(left(put(START,8.))) || ' and  ALERT_AGE LE ' || trim(left(put(END,8.)));
    AGE_GRP_IN_DAYS = '    ' || trim(left(put(START,8.))) || '-' || trim(left(put(END,8.)));
    output;

    do WEEK_NBR= 2 to &weekDelta;
        tmpDate = &minDate + ((&weekDelta- WEEK_NBR) * 7);
        START = END + 1;
        END = START + &DaysInPeriod - 1;
        AGEGRPCHECK = ' ALERT_AGE>= ' || trim(left(put(START,8.))) || ' and  ALERT_AGE=< ' || trim(left(put(END,8.)));
        /* single digits - pad with 4 spaces for VA .....#-# */
        if  START < 10 then do; 
           AGE_GRP_PAD  = 4;
           AGE_GRP_IN_DAYS = '    ' || trim(left(put(START,8.))) || '-' || trim(left(put(END,8.)));
        end;
        /* double digits - pad with 3 places for VA ...##-## */
        if START > 9 and START < 100 then do;
           AGE_GRP_PAD  = 3;  
           AGE_GRP_IN_DAYS = '   ' || trim(left(put(START,8.))) || '-' || trim(left(put(END,8.)));
        end;
        /* triple digits - pad with 1 places for VA .###-### */
        if START > 99 and START < 1000 then do; 
           AGE_GRP_PAD  = 2;
           AGE_GRP_IN_DAYS = ' ' ||trim(left(put(START,8.))) || '-' || trim(left(put(END,8.)));
        end;
        /* quad digits - no pad  ####-#### */
        if START > 1000 then do;
           AGE_GRP_PAD  = 0;
           AGE_GRP_IN_DAYS = trim(left(put(START,8.))) || '-' || trim(left(put(END,8.)));
        end;
        output;
    end;
    run;

/*----------------------------------------------------------------------------*/
/* Select Age groups, logic, number of padding spaces for VA and week number
/* into dynamic macro variable ranges:
/*             :ageGrp1 - :ageGrp%trim(&nbrAgeBuckets)
/*             :ageChk1 - :ageChk%trim(&nbrAgeBuckets)
/*             :agePad1 - :agePad%trim(&nbrAgeBuckets)
/*             :ageWeek1 - :ageWeek%trim(&nbrAgeBuckets)
/* from alert_age_range_list
/*----------------------------------------------------------------------------*/
    proc sql noprint;
        select count(*) into :nbrAgeBuckets from alert_age_range_list;
        select AGE_GRP_IN_DAYS, AGEGRPCHECK, AGE_GRP_PAD , WEEK_NBR
                         into :ageGrp1 - :ageGrp%trim(&nbrAgeBuckets),
                              :ageChk1 - :ageChk%trim(&nbrAgeBuckets),
                              :agePad1 - :agePad%trim(&nbrAgeBuckets),
                              :ageWeek1 - :ageWeek%trim(&nbrAgeBuckets)
        from alert_age_range_list;
    quit;
 
/*----------------------------------------------------------------------------*/
/* Bucket alerts into age grous (number of days old range)
/*----------------------------------------------------------------------------*/

    data selected_alerts;
      set selected_alerts;
      label ALERT_AGE= 'Alert Age in Days'
            AGE_GRP_IN_DAYS = 'Age in Days'
            DURATION = 'Duration of Alert';
      length AGE_GRP_IN_DAYS $9.;
      
      if INCIDENT_DISPOSITION_CD = 'ACT' then do; 
           ALERT_AGE= today() - create_dttm;
           DURATION = ALERT_AGE;
      end;
      else do;
           ALERT_AGE = .;
           DURATION = close_dttm - create_dttm;
      end;
      /* Leverage dynamic marco ranges to generate code for age bucketing */
      %do i = 1 %to &nbrAgeBuckets;
          if (&&ageChk&i) then do;
              AGE_GRP_IN_DAYS = 
              %if &&agePad&i > 0 %then %do;
                   %do p = 1 %to &&agePad&i;
                      ' ' || 
                   %end;
              %end;
               "&&ageGrp&i";
               WEEK_NBR= &&ageWeek&i;
          end;
      %end;
    run;

/*----------------------------------------------------------------------------*/
/* Summarize at the incident_disposition_cd, incident_category_cd level
/* to determine number of alerts, average age and maximum age per classification
/*----------------------------------------------------------------------------*/

  proc summary data=selected_alerts nway;
  class incident_disposition_cd incident_category_cd;
  var ALERT_AGE DURATION;
  output  out=ALERTS_SUMMARY
            n(DURATION) =   NUM_ALERTS
            mean(ALERT_AGE) = AVG_AGE
            max(ALERT_AGE) = MAX_AGE 
            mean(DURATION) = AVG_DURATION
            max(DURATION) = MAX_DURATION;
  run;

/*----------------------------------------------------------------------------*/
/* Join number of alerts, average age and maximum age per classification
/* to selected alerts.
/* Note: Leverage proc sql nowarn option to suppress: 
/*    WARNING: This CREATE TABLE statement recursively references the target table
/*----------------------------------------------------------------------------*/ 

   proc sql nowarn;
      create table work.selected_alerts as
      select a.INCIDENT_RK,
             a.ALERT_ID  , 
             a.VALID_from_DTTM,
             a.VALID_TO_DTTM,
             a.CASE_RK,
             a.INCIDENT_ID,
             a.SOURCE_SYSTEM_CD,
             a.INCIDENT_TYPE_CD,
             a.INCIDENT_CATEGORY_CD,
             a.INCIDENT_SUBCATEGORY_CD,
             a.INCIDENT_DESC,
             a.INCIDENT_from_DT,
             a.INCIDENT_from_TM,
             a.INCIDENT_TO_DT,
             a.INCIDENT_TO_TM,
             a.DETECTION_DT,
             a.DETECTION_TM,
             a.NOTIFICATION_DT,
             a.NOTIFICATION_TM,
             a.UI_DEF_FILE_NM,
             a.CREATE_USER_ID,
             a.CREATE_DTTM,
             a.WEEK,
             a.AGE_GRP_IN_DAYS,
             a.WEEK_NBR,
             a.WEEKBEGINDATE,
             a.YEAR,
             a.QUARTER,
             a.MONTH,
             a.UPDATE_USER_ID,
             a.VERSION_NO,
             a.DELETE_FLG,
             a.INVESTIGATOR_USER_ID,
             a.INCIDENT_DISPOSITION_CD,
             a.CLOSE_DTTM,
             a.INCIDENT_STATUS_CD,
             a.ALERT_AGE,
             a.DURATION,
             b.NUM_ALERTS as CAT_NUM_ALERTS label='Nbr. Alerts by Category',
             b.MAX_AGE label = 'Maximum Age in Days',
             b.AVG_AGE label = 'Average Age in Days' format 8.2,
             b.MAX_DURATION label = 'Maximum Duration in Days',
             b.AVG_DURATION label = 'Average Duration in Days' format 8.2
     from selected_alerts a, ALERTS_SUMMARY b
     where a.INCIDENT_DISPOSITION_CD = b.INCIDENT_DISPOSITION_CD and        
            a.INCIDENT_CATEGORY_CD = b.INCIDENT_CATEGORY_CD;
  quit;

/*----------------------------------------------------------------------------*/
/* Select Summary of number of alerts (NUM_ALERTS) into macro variables by 
/* alert disposition for VA report display
/*       :totalACT, totalCLS, totalSUP
/* from ALERTS_SUMMARY
/*----------------------------------------------------------------------------*/   
  proc sql noprint;
      select sum(NUM_ALERTS) into :totalACT 
          from ALERTS_SUMMARY 
          where INCIDENT_DISPOSITION_CD = 'ACT';
      select sum(NUM_ALERTS) into :totalCLS  
          from ALERTS_SUMMARY 
          where INCIDENT_DISPOSITION_CD = 'CLS';
      select sum(NUM_ALERTS) into :totalSUP
          from ALERTS_SUMMARY 
          where INCIDENT_DISPOSITION_CD = 'SUP';
   quit;

/*----------------------------------------------------------------------------*/
/* Build Duration Display column for Duration of All Alerts report 
/*----------------------------------------------------------------------------*/

  data selected_alerts;
      set work.selected_alerts;
      label DURATION_DISPLAY = 'Duration of All Alerts';
      length DURATION_DISPLAY $60.;
      select (INCIDENT_DISPOSITION_CD);
          when ('ACT') do; 
                DURATION_DISPLAY = 'Days Active (Number of active alerts = ' || trim(left(&totalACT)) || ')';
          end;
          when ('CLS') do; 
                DURATION_DISPLAY = 'Days to Close (Number of closed alerts = ' || trim(left(&totalCLS)) || ')';
          end;
          otherwise do; 
                DURATION_DISPLAY = 'Days to Suppress (Number of suppressed alerts = ' || trim(left(&totalSUP)) ||')';
          end;
      end;
  run;

 /*----------------------------------------------------------------------------*/
 /* Get long display values INCIDENT_DISPOSITION_CD, INCIDENT_CATEGORY_CD
 /* and INCIDENT_SUBCATEGORY_CD.
 /*----------------------------------------------------------------------------*/
  %if %sysfunc(exist(ecm_db.ref_table_value)) %then %do;
        data RT_ALERT_STATUS_DISPLAY (keep= value_cd value_desc)
             RT_INCIDENT_CATEGORY_DISPLAY (keep=value_cd value_desc) 
             RT_INCIDENT_SUBCATEGORY_DISPLAY (keep=value_cd value_desc);
        set ecm_db.ref_table_value;
             if ref_table_nm = 'RT_ALERT_STATUS' then output RT_ALERT_STATUS_DISPLAY;
             if ref_table_nm = 'RT_INCIDENT_CATEGORY' then output RT_INCIDENT_CATEGORY_DISPLAY;
             if ref_table_nm = 'RT_INCIDENT_SUBCATEGORY' then output RT_INCIDENT_SUBCATEGORY_DISPLAY;
        run;
        /*----------------------------------------------------------------------------*/
        /* Update incident_disposition_cd  
        /*----------------------------------------------------------------------------*/
        %fcf_update_display_data(tabName=selected_alerts,
                       LUTAB=RT_ALERT_STATUS_DISPLAY,
                       existCol=INCIDENT_DISPOSITION_CD,
                       LUKey=VALUE_CD,
                       LUVal=VALUE_DESC
                       );
        /*----------------------------------------------------------------------------*/
        /* Update incident_category_cd  
        /*----------------------------------------------------------------------------*/
        %fcf_update_display_data(tabName=selected_alerts,
                       LUTAB=RT_INCIDENT_CATEGORY_DISPLAY,
                       existCol=INCIDENT_CATEGORY_CD,
                       LUKey=VALUE_CD,
                       LUVal=VALUE_DESC
                       );     
        /*----------------------------------------------------------------------------*/
        /* Update incident_subcategory_cd  
        /*----------------------------------------------------------------------------*/
         %fcf_update_display_data(tabName=selected_alerts,
                       LUTAB=RT_INCIDENT_SUBCATEGORY_DISPLAY,
                       existCol=INCIDENT_SUBCATEGORY_CD,
                       LUKey=VALUE_CD,
                       LUVal=VALUE_DESC
                       ); 
     %end; 
     %else %do;
          %let _fcfmsg_ = WARNNG: ecm_db.ref_table_value not found codes will be displayed in report.;  
          %fcf_put_error;
     %end;

 /*----------------------------------------------------------------------------*/
 /* Retrieve Scenario Data from DB_KC.FSK_ALERT.
 /*----------------------------------------------------------------------------*/
    %if %sysfunc(exist(DB_KC.FSK_ALERT)) %then %do;
       PROC SQL;
       CREATE TABLE selected_alerts AS 
       SELECT a.ALERT_ID  , 
              b.PRIMARY_ENTITY_LEVEL_CODE, 
              b.PRIMARY_ENTITY_NAME, 
              b.PRIMARY_ENTITY_NUMBER, 
              b.SCENARIO_NAME, 
              b.ACTUAL_VALUES_TEXT, 
              b.ALERT_DESCRIPTION, 
              b.MONEY_LAUNDERING_RISK_SCORE, 
              b.TERROR_FINANCING_RISK_SCORE, 
              b.ALERT_STATUS_CODE, 
              b.RUN_DATE, 
              a.CREATE_DTTM,
              a.INCIDENT_RK, 
              b.SCENARIO_ID, 
              a.INVESTIGATOR_USER_ID,
              a.VALID_from_DTTM, 
              a.VALID_TO_DTTM, 
              a.CASE_RK, 
              a.INCIDENT_ID, 
              a.SOURCE_SYSTEM_CD, 
              a.INCIDENT_TYPE_CD, 
              a.INCIDENT_CATEGORY_CD, 
              a.INCIDENT_SUBCATEGORY_CD, 
              a.INCIDENT_DESC, 
              a.INCIDENT_from_DT, 
              a.INCIDENT_from_TM, 
              a.INCIDENT_TO_DT, 
              a.INCIDENT_TO_TM, 
              a.DETECTION_DT, 
              a.DETECTION_TM, 
              a.NOTIFICATION_DT, 
              a.NOTIFICATION_TM, 
              a.UI_DEF_FILE_NM, 
              a.CREATE_USER_ID, 
              a.WEEK, 
              a.WEEK_NBR label='Week Number', 
              a.WEEKBEGINDATE,
              a.YEAR, 
              a.QUARTER, 
              a.MONTH, 
              a.AGE_GRP_IN_DAYS,
              a.UPDATE_USER_ID, 
              a.VERSION_NO, 
              a.DELETE_FLG, 
              a.INCIDENT_DISPOSITION_CD, 
              a.CLOSE_DTTM, 
              a.INCIDENT_STATUS_CD, 
              a.Duration_Display, 
              a.CAT_NUM_ALERTS, 
              a.ALERT_AGE,
              a.DURATION, 
              a.MAX_AGE, 
              a.AVG_AGE,
              a.MAX_DURATION,
              a.AVG_DURATION
          from selected_alerts a,  DB_KC.FSK_ALERT b
          where a.ALERT_ID   = B.ALERT_ID  ;
        quit;
   %end;
   %else %do;
      %let trans_rc = 900;
      %let _fcfmsg_ =ERROR: DB_KC.FSK_ALERT table not found, processing will end;  
      %fcf_put_error;
      %goto MACRO_END;
   %end;
 
   /*----------------------------------------------------------------------------*/
   /* Get labels for INCIDENT_LIVE columns
   /*----------------------------------------------------------------------------*/


   %let fcf_config_locale_opt = %sysfunc(sasmsg(sashelp.fcflocale, &fcf_config_locale, N));


    proc sql;
       create table incident_live_ecm_column_labels as 
       select SOURCE_NM, 
              OBJECT_NM, 
              TABLE_NM, 
              COLUMN_NM, 
              LOCALE, 
              LABEL_TXT
          from ECM_DB.ECM_COLUMN_LABEL
          where SOURCE_NM = 'application' and TABLE_NM = 'incident' 
          and LOCALE = "&fcf_config_locale_opt";
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let trans_rc = 900;
         %let _fcfmsg_=ERROR: Could not retrieve incident_live_ecm_column_labels.;
         %fcf_put_error;
         %goto MACRO_END;
    %end;

    proc sql noprint;
        select count (*) into :nbrLabelsRtrn from INCIDENT_LIVE_ECM_COLUMN_LABELS ;
    quit;

   %if &nbrLabelsRtrn > 0 %then %do;
      %fcf_modify_report_labels(inTable=selected_alerts,
                                    labelTable=incident_live_ecm_column_labels);
   %end;
   %else %do;
      %let trans_rc = 909;
      %let _fcfmsg_ = ERROR: No Incident Live columns labels returned.;  
      %fcf_put_error;
      %goto MACRO_END;
   %end;


   /*----------------------------------------------------------------------------*/
   /* Get labels for scenario columns
   /*----------------------------------------------------------------------------*/

    proc sql;
       create table alert_column_labels as 
       select  COLUMN_EXPRESSION_TEXT as COLUMN_NM, 
               COLUMN_LABEL_DESC as LABEL_TXT 
       from DB_KC.FSK_UI_COLUMN
       where TABLE_UI_NAME = 'alert' and
            COLUMN_UI_NAME IN 
     ('alertDisplay', 'primaryEntityLevelCode','primaryEntityNumber',
      'primaryEntityName','scenarioName','actualValuesText','alertDescription',
      'moneyLaunderingRiskScore','terrorFinancingRiskScore','alertStatusCode',
      'runDate','scenarioId');
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let trans_rc = 900;  
         %let _fcfmsg_=ERROR: Failed to return alert table columns.;
         %fcf_put_error;
         %goto MACRO_END;
    %end;


    proc sql noprint;
        select count (*) into :nbrLabelsRtrn from alert_column_labels;
    quit;

   %if &nbrLabelsRtrn > 0 %then %do;
      %fcf_modify_report_labels(inTable=selected_alerts,
                                labelTable=alert_column_labels);
   %end;
   %else %do;
      %let trans_rc = 909;
      %let _fcfmsg_ =ERROR: No FSK_ALERT table columns labels found, processing will end.;  
      %fcf_put_error;
      %goto MACRO_END;
   %end;


   /*----------------------------------------------------------------------------*/
   /* Get current users from metadata.
   /*----------------------------------------------------------------------------*/
   %mduextr(libref=work);
   
   /*----------------------------------------------------------------------------*/
   /* GET CURRENT OWNER OF ALERT DISPLAY NAME
   /*----------------------------------------------------------------------------*/
      %if %sysfunc(exist(work.person)) %then %do;
      proc sql;
      create table fcf_rpt.ALERT_INVESTIGATION_EXTRACT %fcf_get_ds_opts(lib_name=fcf_rpt, ds_name=ALERT_INVESTIGATION_EXTRACT, output_ds=Y, add_parens=Y)  as
      select a.ALERT_ID, 
             a.PRIMARY_ENTITY_LEVEL_CODE, 
             a.PRIMARY_ENTITY_NAME, 
             a.PRIMARY_ENTITY_NUMBER, 
             a.SCENARIO_NAME, 
             a.ACTUAL_VALUES_TEXT, 
             a.ALERT_DESCRIPTION, 
             a.MONEY_LAUNDERING_RISK_SCORE, 
             a.TERROR_FINANCING_RISK_SCORE, 
             a.ALERT_STATUS_CODE,
             a.RUN_DATE, 
             a.CREATE_DTTM, 
             a.INCIDENT_RK, 
             a.SCENARIO_ID, 
             a.INVESTIGATOR_USER_ID,
             a.VALID_from_DTTM, 
             a.VALID_TO_DTTM, 
             a.CASE_RK, 
             a.INCIDENT_ID, 
             a.SOURCE_SYSTEM_CD, 
             a.INCIDENT_TYPE_CD, 
             a.INCIDENT_CATEGORY_CD, 
             a.INCIDENT_SUBCATEGORY_CD, 
             a.INCIDENT_DESC, 
             a.INCIDENT_from_DT, 
             a.INCIDENT_from_TM, 
             a.INCIDENT_TO_DT, 
             a.INCIDENT_TO_TM, 
             a.DETECTION_DT, 
             a.DETECTION_TM, 
             a.NOTIFICATION_DT, 
             a.NOTIFICATION_TM, 
             a.UI_DEF_FILE_NM, 
             a.CREATE_USER_ID,
             a.WEEK, 
             a.WEEK_NBR, 
             a.WEEKBEGINDATE, 
             a.YEAR,
             a.QUARTER, 
             a.MONTH, 
             a.AGE_GRP_IN_DAYS,
             a.UPDATE_USER_ID, 
             a.VERSION_NO, 
             a.DELETE_FLG, 
             CASE WHEN  b.NAME ^= "" THEN b.DISPLAYNAME 
             ELSE "&no_owner" END AS OWNER label = 'Owner',
             a.INCIDENT_DISPOSITION_CD, 
             a.CLOSE_DTTM, 
             a.INCIDENT_STATUS_CD, 
             a.DURATION_DISPLAY, 
             a.CAT_NUM_ALERTS, 
             a.ALERT_AGE,
             a.DURATION, 
             a.MAX_AGE, 
             a.AVG_AGE,
             a.MAX_DURATION,
             a.AVG_DURATION
      from SELECTED_ALERTS as a
      left join PERSON as b ON (a.INVESTIGATOR_USER_ID = b.NAME);
      quit;
      %end;
      %else %do;
          %let trans_rc = 900;
          %let _fcfmsg_ =ERROR: work.person table not found, processing will end;  
          %fcf_put_error;
          %goto MACRO_END;
      %end;


%MACRO_END:

%if &trans_rc in (3 5 900 909) %then
     %fcf_execute_macro(fcf_error);
%else %do;
     %fcf_execute_macro(fcf_success);
%end;

%mend fcf_alert_investigation_extract;
