/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    cdd_crr_save_to_kc.sas
| PRODUCT: SAS Customer Due Diligence
| SYSTEM:  Windows Unix
| DESC:    Standardize CDD scoring result. 
|          If it is not a test, permanently save the result to
|          DB_KC.FSK_CDD_SCORING_EVENT, DB_KC.FSK_CDD_TRIGGERING_VALUES
|          DB_KC.FSK_CDD_ENTITY_FACT
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS
|           in_libname= SAS library containing the in_memname data set
|                       e. g. WORK
|           in_memname= table name of the input dataset
|                       e. g. CDD_SCORING_EVENT_ALL      
|           testYN=     test flag. Y for generating out_test_ds dataset 
|           out_test_ds= name of the output test dataset 
|                        if testYN is not Y, the parameter is ignored
|
| Although the data model of FSK_CDD_TRIGGERING_VALUES 
| support multiple triggering values for the same score event,
| it is not supported in the CRR process for this release.
| The limitation of the macro is scoring result records in a single CRR execution process should have the same
| PRIMARY_ENTITY_LEVEL_CODE 
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2013-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro cdd_crr_save_to_kc(   in_libname=WORK
                           , in_memname=CDD_SCORING_EVENT_ALL
                           , testYN=
                           , out_test_ds=cdd_crr_test_out);

    /*------------------------------------------
    * Set error code to 0 and message to blank
    *------------------------------------------*/
    %let trans_rc = 0;
    %let _fcfmsg_=;
 
    %let in_libname=%upcase(&in_libname);
    %let in_memname=%upcase(&in_memname);
  
    %let testYN=%upcase(&testYN);
    %if &testYN=Y %then %let out_ds=&out_test_ds;
    %else %let out_ds=_t_outds;

    /* Find the column names of FSK_CDD_REVIEW */
    proc contents data=DB_KC.FSK_CDD_REVIEW out=_t_cdd_review_columns (keep=name) noprint;
    run;

    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /* Find the column names of FSK_CDD_SCORING_EVENT */
    proc contents data=DB_KC.FSK_CDD_SCORING_EVENT out=_t_scoring_columns (keep=name) noprint;
    run;

    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
 
    /* Find the column names of FSK_CDD_TRIGGER_VALUES */
    proc contents data=DB_KC.FSK_CDD_TRIGGERING_VALUES out=_t_triggering_columns (keep=name) noprint;
    run;

    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
        %fcf_put_error;
        %goto MACRO_END;
    %end;  

    /* Find the special __CRR column names of the input table */
    proc contents data=&in_libname..&in_memname out=_t_crr_columns (keep=name) noprint;
    run;

    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
        %fcf_put_error;
        %goto MACRO_END;
    %end;   

   
    proc sql noprint;
        %let _t_crr_keep_list=;
        SELECT distinct upcase(name) INTO :_t_crr_keep_list separated by ' '
            FROM _t_crr_columns
            WHERE substr(upcase(name),1,5)='__CRR';

        SELECT distinct upcase(name) INTO :_t_keep_list separated by ' '
            FROM (select name from _t_scoring_columns
                  union
                  select name from _t_triggering_columns
            );

        %let _t_cdd_review_keep_list=;
        SELECT distinct upcase(name) INTO :_t_cdd_review_keep_list separated by ' '
            FROM _t_cdd_review_columns
        ;

        %let _t_max_scoring_key=0;
        SELECT put(max(cdd_scoring_event_key),best.) INTO :_t_max_scoring_key
            FROM DB_KC.FSK_CDD_SCORING_EVENT;
        %if &_t_max_scoring_key < 0 %then %let _t_max_scoring_key=0;

        %let _t_max_triggering_key=0;
        SELECT put(max(cdd_triggering_value_key),best.) INTO :_t_max_triggering_key
            FROM DB_KC.FSK_CDD_TRIGGERING_VALUES;
        %if &_t_max_triggering_key < 0 %then %let _t_max_triggering_key=0;
    quit;


    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

        
    /* Create data table containing all data needed for FSK_CDD_SCORING_EVENT,  */
    /*   FSK_CDD_TRIGGERING_VALUES and FSK_CDD_ENTITY_FACT                      */

    %let cdd_fact_updt_flg=0;

    data &out_ds  (keep= &_t_keep_list &_t_crr_keep_list __crr_save_trigger_flg)
        _TMP_CDD_ENTITY_FACT (keep=  primary_entity_level_code
                                     primary_entity_number
                                     last_scoring_event_key
                                     curr_review_scoring_event_key
                                     last_change_date
                                     __crr_new_flg
                                     cdd_review_flg
                                     investigation_ind
                               )
        _TMP_CDD_REVIEW (keep=&_t_cdd_review_keep_list)
        ;

        if 0 then
            SET DB_KC.FSK_CDD_SCORING_EVENT
                DB_KC.FSK_CDD_TRIGGERING_VALUES
                DB_KC.FSK_CDD_REVIEW
                DB_KC.FSK_CDD_ENTITY_FACT (keep=/* override_score_class_code */
                                                 last_scoring_event_key
                                                 curr_review_scoring_event_key
                                                 last_change_date
                                            )
        ;

        RETAIN _tmp_cdd_fact_updt_flg '0';

        SET &in_libname..&in_memname end=eof;

        cdd_scoring_event_key=&_t_max_scoring_key + _n_;
        cdd_triggering_value_key=&_t_max_triggering_key + _n_;
        output &out_ds;

        /* Create a record for temporary cdd entity fact for the final score only */
        if score_type_code='FINAL' then do;
            /* Store current scoring event key as last_scoring_event_key        */
            if last_scoring_event_key > 0 then do;
                __crr_new_flg='0';
                _tmp_cdd_fact_updt_flg='1';
            end;
            else
                __crr_new_flg='1';

            last_scoring_event_key=cdd_scoring_event_key;

            /* Store current scoring event key as curr_review_scoring_event_key */
            /* if the score triggers an investigation.                          */
            if curr_review_key =. and investigation_ind='Y' then do;
                curr_review_scoring_event_key=cdd_scoring_event_key;
            end;

            /* store event date as last_change_date */
            last_change_date=event_date;
            output _tmp_cdd_entity_fact;
        end;

        /* create review records for new scores that require investigation */
        if score_type_code='FINAL' and curr_review_key =. and investigation_ind='Y' then do;
            /* all FSK_CDD_REVIEW column must have a value assigned, otherwise it will be inherited
             * from prep data set */
            CREATE_DATE=event_date;
            CREATE_USER_ID='CRR';
            /* EMPLOYEE_IND - copy from prep data set */
            FINAL_DISPOSITION_CODE="";
            INITIAL_AUTO_HIGH_IND=auto_high_ind;
            INITIAL_RISK_RATING=score_classification_code;
            INITIAL_RISK_SCORE=rule_score;
            INITIAL_SCORING_EVENT_KEY=cdd_scoring_event_key;
            /* LAST_REVIEW_DATE - from prep data set - the same as FSK_CDD_ENTITY_FACT */
            LSTUPDATE_DATE=event_date;
            LSTUPDATE_USER_ID='CRR';
            /* NEXT_REVIEW_DATE - from prep data set - the same as FSK_CDD_ENTITY_FACT */
            OWNER_USER_LONG_ID="";
            PERIODIC_REVIEW_DATE=.; /* not used */
            /* from prep data set
             * PRIMARY_ENTITY_LEVEL_CODE
             * PRIMARY_ENTITY_NUMBER
             */
            RATING_OVERRIDE = "";
            REVIEW_DUE_DATE=.;
            /* REVIEW_KEY - will be assigned later in this macro */
            STATUS_CODE="O";
            VERSION_NUMBER=1;

            /* update current review key for FSK_CDD_ENTITY_FACT */
            CURR_REVIEW_SCORING_EVENT_KEY=cdd_scoring_event_key;

            output _TMP_CDD_REVIEW;
        end;

        if eof then do;
            call symput('CDD_FACT_UPDT_FLG',_tmp_cdd_fact_updt_flg);
            call symput('PRIMARY_ENTITY_LEVEL_CODE_VALUE', "'"||strip(primary_entity_level_code)||"'");
        end;
    run;

    %put cdd_fact_updt_flg=&cdd_fact_updt_flg;

    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /**************************************/
    /* Stop here if it is a test          */
    /**************************************/

    %if %str(&testYN) ne Y %then %do;

        /* append new events */
        %cdd_proc_append(   base_ds=DB_KC.FSK_CDD_SCORING_EVENT
                         , data_ds=&out_ds
                         , option=force nowarn
                        );
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %goto MACRO_END;

        /* append triggering values */

        %cdd_proc_append(   base_ds=DB_KC.FSK_CDD_TRIGGERING_VALUES
                         , data_ds=&out_ds
                         , where_cl=%quote(__crr_save_trigger_flg='1' or __crr_save_trigger_flg='Y')
                         , option=force nowarn
                        );
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %goto MACRO_END;

        /* If there are existing parties in FSK_CDD_ENTITY_FACT that need to be update */
        %if &cdd_fact_updt_flg = 1 %then %do;
             %cdd_crr_entity_fact_updt(   in_dsn=WORK._TMP_CDD_ENTITY_FACT
                                        , primary_entity_level_code_value=&primary_entity_level_code_value
                                       );
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %goto MACRO_END;
        %end;

        %cdd_proc_append(   base_ds=DB_KC.FSK_CDD_ENTITY_FACT
                           , data_ds=_tmp_cdd_entity_fact
                           , where_cl=%quote(__crr_new_flg='1')
                           , option=force nowarn
                        );
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %goto MACRO_END;

        /* create new reviews */
        %let num_reviews = %fcf_get_observation_count(table=_TMP_CDD_REVIEW, nonotes=N, sasdataset=Y);
        %put NOTE: Number of review ids requested: &num_reviews;
        %if &num_reviews ne 0 %then %do;
            /*--------------------------------
            * get sequence IDs for new records
            *--------------------------------*/
            %fcf_get_sequence_ids(numIds=&num_reviews, globals=first);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _fcfmsg_=ERROR: FCF: Unable to obtain review ids from &seg_kc_libname..FSK_SEQ.;
                %fcf_put_error;
                %goto MACRO_END;
            %end;

            data _TMP_CDD_REVIEW;
                set _TMP_CDD_REVIEW;
                REVIEW_KEY = _n_ + &firstSeqID -1;
            run;
            /*-------------------------------------------------------------------------------
            * If Auto Routing preference set, then Route CDD Reviews using Round Robin Method
            *--------------------------------------------------------------------------------*/
            %fcf_get_property_value(property=fcf.svr.cdd.autoRouteCDDReviews,
                                  output_property_value=AUTO_ROUTE_CDD_REVIEW,
                                  output_status=property_status); 
            %if &property_status ne 0 %then %do;
              %let _fcf_msg_=WARNING:  The autoRouteCDDReviews property is not set.  No Auto Routing will be done.;
              %let trans_rc=4;
              %fcf_put_error;
            %end;
            %else %if &AUTO_ROUTE_CDD_REVIEW ne 0 %then %do;
             %fcf_get_property_value(property=fcf.svr.cdd.routeCDDReviewsRole,
                                     output_property_value=ROUTE_CDD_REVIEW_TO_ROLE,
                                     output_status=property_status); 
             %if &property_status ne 0 %then %do;
                 %let _fcf_msg_=WARNING:  The RouteCDDReviewsRole property is not set.  No Auto Routing will be done.;
                 %let trans_rc=4;
                 %fcf_put_error;
             %end; 
             %else %do;
                /* route reviews */
                %fcf_get_property_value(property=fcf.svr.aml.useRoutingToQueues,output_property_value=CUSTOM_ROUTING,
                output_status=property_status);

                %if &property_status ne 0 %then %do;
                    %let _fcfmsg_=ERROR: FCF: Unable to get property fcf.svr.aml.useRoutingToQueues from SEG_KC.FSK_PREFERENCE;
                    %let trans_rc = 999999;
                    %fcf_put_error;
                    %goto MACRO_END;
                %end;

                %let CUSTOM_ROUTING=&CUSTOM_ROUTING;
                %if &CUSTOM_ROUTING. eq false %then %do;    /* call round robin             */
                    %fcf_route_entity_cdd(inLib=work,
                                      inDS=_TMP_CDD_REVIEW,
                                      pk_key=REVIEW_KEY,
                                      bank_users=&bankUsersGroup,
                                      role_name=&ROUTE_CDD_REVIEW_TO_ROLE,
                                      column_name=OWNER_USER_LONG_ID,
                                      sasdataset=Y
                                      );
                %end;  /* end of round robin */
                %else %do; /* custom routing */
                    %fcf_route_entity_queue(inLib=work,
                                             inDS=_TMP_CDD_REVIEW,
                                             pk_key=REVIEW_KEY,
                                             bank_users=&bankUsersGroup,
                                             role_name=&ROUTE_CDD_REVIEW_TO_ROLE,
                                             column_name=OWNER_USER_LONG_ID,
                                             sasdataset=Y
                                             );
                %end;

            
                %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                   %goto MACRO_END;
                %end;
             %end;
            %end;

            /* create EDD review rows for each CDD review */
            proc sql;
                create table _TMP_CDD_EDD_REVIEW_ROWS
                   as select c.REVIEW_KEY,
                             l.LOV_TYPE_CODE as EDD_STEP_CODE,
                             l.LOV_SORT_POSITION_NUMBER as EDD_STEP_SORT_POSITION_NUMBER
                   from  _TMP_CDD_REVIEW c
                         inner join DB_KC.FSK_LOV l
                            on l.LOV_TYPE_NAME = 'CDD_EDD_STEP'
                               and l.ACTIVE_FLG='Y'
                               and l.LOV_LANGUAGE_DESC='en'
                ;
            quit;
            proc sql;
                create table _TMP_CDD_EDD_REVIEW like DB_KC.FSK_CDD_EDD_REVIEW;
            quit;
            proc append base=_TMP_CDD_EDD_REVIEW data=_TMP_CDD_EDD_REVIEW_ROWS force nowarn;
            run;

            %cdd_proc_append(   base_ds=DB_KC.FSK_CDD_REVIEW
                             , data_ds=_TMP_CDD_REVIEW
                             , option=force nowarn
                            );
            %cdd_proc_append(   base_ds=DB_KC.FSK_CDD_EDD_REVIEW
                             , data_ds=_TMP_CDD_EDD_REVIEW
                             , option=force nowarn
                            );

            %if &trans_rc eq 3 or &trans_rc ge 5 %then %goto MACRO_END;
        %end;
    %end;

%MACRO_END:
    %if &trans_rc eq 3 or &trans_rc ge 5 %then
     %cdd_error(testYN=&testYN);
    %else %do;
     %cdd_success(testYN=&testYN);
    %end;
%mend cdd_crr_save_to_kc;
 
