/******************************************************************************/
/*    NAME: cdd_crr_test_display_results.sas                                  */
/* PRODUCT: SAS Customer Due Diligence                                        */
/*  SYSTEM: UNIX/WINDOWS                                                      */
/*                                                                            */
/* -------------------------------------------------------------------------- */
/*
 DESC: This macro is called at the end of the stored process that executes
           the rules only in case if the execution was successful. It is invoked
           after %STPBEGIN macro was executed, so it allows customization of the
           the output that will be sent back to the browser using ODS output.
 PARAMETERS:
   score_ds_nm - dataset name containing all scoring events saved by CRR
             process. Data set is sorted by  PRIMARY_ENTITY_NUMBER.
   display_all_scores - if Y then user requested full score table to be
                             displayed, otherwise just score counts will be
                             sent back.
   cddlib - libname with the following datasets:
                    FSK_CDD_RULE
                    FSK_CDD_RULE_GROUP
                    FSK_CDD_RULE_PARAMETER
*/
/******************************************************************************/
/*-------------------------------------------------------------------
 * Copyright (c) 2013-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

/* full list of available columns in the data set with scores:
 PRIMARY_ENTITY_KEY    RULE_SCORE   EVENT_DATE     SCORE_CLASSIFICATION_CODE       AUTO_HIGH_IND   INVESTIGATION_IND
 PRIMARY_ENTITY_LEVEL_CODE   PRIMARY_ENTITY_NUMBER   CDD_RULE_ID     CATEGORY_CODE   VERSION_NO      SCORE_TYPE_CODE
 TRIGGERING_VALUE_ID        TRIGGERING_CHAR_VALUE   TRIGGERING_DATE_VALUE   TRIGGERING_NUM_VALUE    TRIGGERING_VALUE_TYPE_CODE
 __CRR_SAVE_TRIGGER_FLG;
*/
%macro cdd_crr_test_display_results(score_ds_nm=, display_all_scores=, cddlib=test);
    proc sort data=cdd_crr_test_out;
      by PRIMARY_ENTITY_KEY SCORE_TYPE_CODE;
    run;
    /************************/
    /* Display final counts */
    /************************/
    %fcf_get_aml_lookup_table(tabName=CDD_SCORE_CLASSIFICATION, locale=&fcf_user_locale);
    %fcf_get_aml_lookup_table(tabName=CDD_SCORE_TYPE, locale=&fcf_user_locale);
    %fcf_get_aml_lookup_table(tabName=CDD_CATEGORY, locale=&fcf_user_locale);

    proc sql;
          create table final_scores as
          select  r.LOV_TYPE_DESC, count(PRIMARY_ENTITY_KEY) as Count
          from &score_ds_nm s
          left join CDD_SCORE_CLASSIFICATION  r
               on r.lov_type_name = "CDD_SCORE_CLASSIFICATION" and s.score_classification_code = r.lov_type_code
          where SCORE_TYPE_CODE = "FINAL"
          group by LOV_TYPE_DESC;
    quit;
        %let cdd_sm_value_desc_label= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_value_desc_label, N));
        %let cdd_sm_count_label = %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_count_label, N));
    proc print data=final_scores NOOBS LABEL;
          Label LOV_TYPE_DESC="&cdd_sm_value_desc_label";
          Label COUNT="&cdd_sm_count_label";
    run;

    %let cdd_sm_CDD_RULE_NAME= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_CDD_RULE_NAME, N));
    %let cdd_sm_SCORE_TYPE_CODE= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_SCORE_TYPE_CODE, N));
    %let cdd_sm_CATEGORY= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_CATEGORY, N));
    %let cdd_sm_RULE_ID= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_RULE_ID, N));
    %let cdd_sm_VERSION_NO= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_VERSION_NO, N));
    %let cdd_sm_score_count= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_score_count, N));
    %let cdd_sm_min_score= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_min_score, N));
    %let cdd_sm_max_score= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_max_score, N));
    %let cdd_sm_avg_score= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_avg_score, N));
    %let cdd_sm_auto_high_count= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_auto_high_count, N));

    proc sql;

        /************************/
        /* Get auto high counts */
        /************************/

        create table crr_result_auto_high_count as
        select distinct    cdd_rule_id
                        ,  version_no
                        ,  count (*) as auto_high_count
            from &score_ds_nm
            where auto_high_ind='Y'
            group by cdd_rule_id
            order by cdd_rule_id
        ;



        /*********************************************************/
        /* Get rule score statistics (Max, Min, Mean, Frequency) */
        /*********************************************************/

        create table crr_result_sum as
        select distinct   st.LOV_TYPE_DESC as SCORE_TYPE
                        , cc.LOV_TYPE_DESC as category
                        , cdd_rule_id
                        , version_no
                        , count (*) as score_count
                        , min(rule_score) as score_min
                        , max(rule_score) as score_max
                        , avg(rule_score) as score_avg
            from &score_ds_nm s
            left join CDD_SCORE_TYPE  st
                 on st.lov_type_name = "CDD_SCORE_TYPE" and s.SCORE_TYPE_CODE = st.lov_type_code
            left join CDD_CATEGORY  cc
                 on cc.lov_type_name = "CDD_CATEGORY" and s.CATEGORY_CODE = cc.lov_type_code
        group by cdd_rule_id
        order by cdd_rule_id
        ;
    quit;

    /**********************************/
    /* Merge two CRR result tables    */
    /* by rule_id                     */
    /**********************************/
    data crr_result_combine;
         merge crr_result_sum
               crr_result_auto_high_count
               ;
             by cdd_rule_id;
    run;

    proc sql;
       create table crr_result_display as
       select b.cdd_rule_name, a.*
       from crr_result_combine as a,
            &cddlib..fsk_cdd_rule as b
       where a.cdd_rule_id=b.cdd_rule_id and a.version_no=b.version_no
       order by score_type, category, cdd_rule_name
       ;
    quit;

    data crr_result_display;
        set crr_result_display;
        label
          CDD_RULE_NAME="&cdd_sm_CDD_RULE_NAME"
          SCORE_TYPE="&cdd_sm_SCORE_TYPE_CODE"
          CATEGORY="&cdd_sm_CATEGORY"
          CATEGORY="&cdd_sm_CATEGORY"
          CDD_RULE_ID="&cdd_sm_RULE_ID"
          VERSION_NO="&cdd_sm_VERSION_NO"
          score_count="&cdd_sm_score_count"
          score_min="&cdd_sm_min_score"
          score_max="&cdd_sm_max_score"
          score_avg="&cdd_sm_avg_score"
          auto_high_count="&cdd_sm_auto_high_count"
          ;
    run;

    proc print data=crr_result_display NOOBS LABEL;
    run;

    /************************/
    /* Display all scores   */
    /************************/
    %if &display_all_scores eq Y %then %do;
        %fcf_get_aml_lookup_table(tabName=CDD_SCORE_CLASSIFICATION, locale=&fcf_user_locale);
        proc sql;
            create table score_full as
            select s.PRIMARY_ENTITY_NUMBER, s.RULE_SCORE, st.LOV_TYPE_DESC as SCORE_TYPE_CODE, s.AUTO_HIGH_IND, u.CDD_RULE_NAME
                from &score_ds_nm s
                left join CDD_SCORE_TYPE  st
                     on st.lov_type_name = "CDD_SCORE_TYPE" and s.SCORE_TYPE_CODE = st.lov_type_code
                left join &cddlib..fsk_cdd_rule u
                on s.CDD_RULE_ID = u.CDD_RULE_ID and s.VERSION_NO = u.VERSION_NO
                order by s.PRIMARY_ENTITY_NUMBER, SCORE_TYPE_CODE
            ;
        quit;

        %let cdd_sm_PRIMARY_ENTITY_KEY= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_PRIMARY_ENTITY_KEY, N));
        %let cdd_sm_PRIMARY_ENTITY_NUMBER= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_PRIMARY_ENTITY_NUMBER, N));
        %let cdd_sm_RULE_SCORE= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_RULE_SCORE, N));
        %let cdd_sm_VALUE_DESC= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_VALUE_DESC, N));
        %let cdd_sm_AUTO_HIGH_IND= %sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_AUTO_HIGH_IND, N));
        /* rule name could be localized if needed */

        data score_full;
            set score_full;
            label
              PRIMARY_ENTITY_NUMBER="&cdd_sm_PRIMARY_ENTITY_NUMBER"
              RULE_SCORE="&cdd_sm_RULE_SCORE"
              SCORE_TYPE_CODE="&cdd_sm_SCORE_TYPE_CODE"
              LOV_TYPE_DESC="&cdd_sm_VALUE_DESC"
              AUTO_HIGH_IND="&cdd_sm_AUTO_HIGH_IND"
              CDD_RULE_NAME="&cdd_sm_CDD_RULE_NAME"
              ;
        run;
        proc print data=score_full NOOBS LABEL;
        run;
    %end;
%mend cdd_crr_test_display_results;
