/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    cdd_prep_chk_pty_chg.sas
| PRODUCT:  SAS Customer Due Diligence
| SYSTEM:  Windows, Unix
| DESC:    flag the list of parties who have changes that should trigger re-scoring 
|    
+------------------------------------------------------------------------------+
| USAGE: %cdd_prep_chk_pty_chg;
+------------------------------------------------------------------------------+
| COMMENTS:
|   Input:
|   INDSN = table with the list of party numbers 
|           it must contain PARTY_NUMBER CDD_SCORING_FLG
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2013-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro cdd_prep_chk_pty_chg(indsn=work.master_entity_list);
        /************************************/
        /* check party dim columns:         */
        /* change_begin_date - SCD2 change  */
        /* lst_update_date - SCD1 change     */
        /************************************/
	    %let chg_obs=0;
	    proc sql noprint;
            create table pty_dim_chg as
                select distinct party_number
                    from  seg_kc.fsc_party_dim
                    where change_begin_date >  &cdd_last_asofdttm
                          or lst_update_date >  &cdd_last_asofdttm
            ;
        quit;
		%fcf_rcset(&syserr)
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
        %end;  
		%let chg_obs=&sqlobs;

		%if &chg_obs > 0 %then %do;
             proc sql noprint;
	              update &indsn 
	              set cdd_scoring_flg='1' 
	              where party_number in (select distinct party_number from pty_dim_chg) 
	         ;
	         quit;
			 %fcf_rcset(&syserr)
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                 %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
                 %fcf_put_error;
                 %goto MACRO_END;
             %end;
        %end; /* end chg_obs > 0 */

        /**********************/
		/* check party assoc  */
		/**********************/
	    %let chg_obs=0;
	    /* select all parties if the relationship has changed
	     * or when one of the parties has changed
	     */
	    proc sql noprint;
            create table pty_assoc_chg as
                select distinct party_number, related_party_number
                    from  seg_kc.fsc_party_assoc
                    where change_begin_date >  &cdd_last_asofdttm
            UNION
                select distinct a.party_number, a.related_party_number
                    from  seg_kc.fsc_party_assoc a
                    inner join pty_dim_chg c
                         on a.party_number = c.party_number or a.related_party_number = c.party_number
            ;
        quit;
		%let chg_obs=&sqlobs;
		%fcf_rcset(&syserr)
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
        %end;  

		%if &chg_obs > 0 %then %do;
             proc sql noprint;
	              update &indsn 
	              set cdd_scoring_flg='1' 
	              where party_number in (select distinct party_number from pty_assoc_chg)
                        or party_number in (select distinct related_party_number from pty_assoc_chg)
	         ;
	         quit;
			 %fcf_rcset(&syserr)
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                  %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
                  %fcf_put_error;
                  %goto MACRO_END;
             %end;
        %end; /* end chg_obs > 0 */

		/******************************/
		/* check party assoc attribute */
		/*******************************/
	    %let chg_obs=0;
	    proc sql noprint;
            create table pty_assoc_attr_chg as
            select distinct party_number, related_party_number
                from  seg_kc.fsc_party_assoc_attribute
                where change_begin_date >  &cdd_last_asofdttm
                ;
        quit;
		%let chg_obs=&sqlobs;
		%fcf_rcset(&syserr)
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
        %end;  

		%if &chg_obs > 0 %then %do;
             proc sql noprint;
	              update &indsn 
	              set cdd_scoring_flg='1' 
	              where party_number in (select distinct party_number from pty_assoc_attr_chg)
                        or party_number in (select distinct related_party_number from pty_assoc_attr_chg)
	              ;
	         quit;
			 %fcf_rcset(&syserr)
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                  %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
                  %fcf_put_error;
                  %goto MACRO_END;
            %end;
        %end; /* end chg_obs > 0 */

     	/*************************/
		/* check account change  */
		/*************************/
	    %let chg_obs=0;
	    proc sql noprint;
            create table pty_acct_dim_chg as
            select distinct c.party_number
            from (select distinct account_key
                   from  seg_kc.fsc_account_dim
                   where change_begin_date > &cdd_last_asofdttm
                  ) as a
                  , seg_kc.fsc_party_account_bridge as b
                  , seg_kc.fsc_party_dim as c
            where a.account_key=b.account_key and b.party_key=c.party_key
			;
        quit;
     
		%let chg_obs=&sqlobs;

		%fcf_rcset(&syserr)
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
        %end;  

		%if &chg_obs > 0 %then %do;
             proc sql noprint;
	              update &indsn
                      set cdd_scoring_flg='1'
                      where party_number in (select distinct party_number from pty_acct_dim_chg)
	              ;
	         quit;
   		     %fcf_rcset(&syserr)
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                  %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
                  %fcf_put_error;
                  %goto MACRO_END;
             %end;
        %end; /* end chg_obs > 0 */

         /********************************/
		 /* check account bridge change  */
		 /********************************/
	    %let chg_obs=0;
	    proc sql noprint;
            create table pty_acct_bridge_chg as
            select distinct b.party_number
            from   seg_kc.fsc_party_account_bridge as a
                 , seg_kc.fsc_party_dim as b
            where  a.change_begin_date >  &cdd_last_asofdttm
                   and a.party_key=b.party_key
			;
        quit;
		%fcf_rcset(&syserr)
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
        %end;  
     
		%let chg_obs=&sqlobs;

		%if &chg_obs > 0 %then %do;
             proc sql noprint;
	              update &indsn
	              set cdd_scoring_flg='1' 
	              where party_number in (select distinct party_number from pty_acct_bridge_chg)
	             ;
	         quit;
			 %fcf_rcset(&syserr)
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                  %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
                  %fcf_put_error;
                  %goto MACRO_END;
                  quit;
             %end;
        %end; /* end chg_obs > 0 */


        /****************************/
	    /* check address dim change  */
	    /*****************************/
	    %let chg_obs=0;
	    proc sql noprint;
            create table address_chg as
            select distinct primary_entity_number as party_number
            from   seg_kc.fsc_address_dim
            where  primary_entity_level_code='PTY'
                   and change_begin_date >  &cdd_last_asofdttm
			;
        quit;
		%fcf_rcset(&syserr)
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
        %end;  
     
		%let chg_obs=&sqlobs;

		%if &chg_obs > 0 %then %do;
             proc sql noprint;
	              update &indsn
	              set cdd_scoring_flg='1' 
	              where party_number in (select distinct party_number from address_chg)
	             ;
	         quit;
			 %fcf_rcset(&syserr)
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                  %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
                  %fcf_put_error;
                  %goto MACRO_END;
             %end;
        %end; /* end chg_obs > 0 */

 %MACRO_END:
   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %cdd_error;
   %else %do;
      %cdd_success;
   %end;
%mend;
 
