/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_agp.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Alert Generation Process (AGP)
+------------------------------------------------------------------------------+
| USAGE:   %fcf_agp
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------
 *  Host and Site specific customizations may be REQUIRED
 *---------------------------------------------------------*/

%macro fcf_agp;
  %let aml_inv_type=AML;                         /* AML case type */
  %let aml_sar90_cat=SAR90;                      /* AML SAR90 case category */
  %let aml_pty_level_list='PTY' 'EXT';                 /* List of entity levels for report suspects*/

 /*----------------
  * Enable Formats
  *----------------*/
  %macro fcf_enable_formats;
        %if &enable_formats eq Y %then %do;
            %fcf_create_formats;
        %end;
        %else %do;
            %put NOTE: FCF: Formats are not enabled.;
        %end;
    %mend fcf_enable_formats;
  %fcf_enable_formats;


  /*------------------------------------------------------------------
   * Store start datetime of the FCF AGP process in a data set.
   * Note:  This sets the create date to the current date time.
   *        The create date is used for suppression so this date
   *        must be updated if running more than one runasofday on
   *        a single calendar day
   *------------------------------------------------------------------*/
  %fcf_start_datetime_init;
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  /*------------------------------------------------------------------
   * Update the start datetime of the process to match the runaofdate
   * Optional:  Leave commented out if not running muliple AGP run
   *            dates on a single calendar date.
   *------------------------------------------------------------------*/
   /*
   proc sql noprint;
   select calendar_date into :start_dt_time
   from db_core.fsc_date_dim
   where date_key=&runasofdate;
   quit;

   %fcf_start_datetime_update(start_dt="&start_dt_time"dt);
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %goto MACRO_END;
   %end;
  */

  /*------------------------------------------------------------------
   * Make sure the FCF start datetime has been set before proceeding.
   *------------------------------------------------------------------*/
  %fcf_start_datetime_read;
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  /*----------------------------------------------------------------------
   * Notes:
   *  When parameters are updated for a scenario, the header code must be
   *  re-generated and the Alert Generation Process (AGP) re-executed.
   *  If re-executing for the same RUNDATE in the same sessions, then
   *  be certain to clear the STG_ALER directory before executing.
   *  Otherwise the STG_ALER data sets will contain duplicate alerts.
   *  Set get_scenario_performance_stats to Y in the autoexec so
   *  fcf_agp_codegen generates code to gather performance statistics
   *  for scenarios.
   *----------------------------------------------------------------------*/
  %fcf_agp_codegen(outLib=STG_ALER,
                   outDS=generated_headers);
   
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto MACRO_END;
  %end;

  %fcf_run_generated_headers(inLib=STG_ALER,
                             inDS=generated_headers,
                             exclude_header=empty_header);
   
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto MACRO_END;
  %end;

  /*------------------------------------------------
   * An example of how to include other headers not
   * processed by today's run of fcf_agp_codegen
   * and fcf_run_generated_headers:
   *------------------------------------------------*/
  /*
  %fcf_section_header(Section=ANOTHER_HEADER);
     %include '!FCFROOT/scenario/codegen/another_header.sas';
     %fcf_rcset(&syserr)
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmodule_=ANOTHER_HEADER;
        %let _fcfmsg_=ERROR: FCF: another_header.sas failed check log.;
        %fcf_put_error;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=ANOTHER_HEADER);
  */

  /*-----------------------------------------------------------
   *  An example of generating alerts for VSD scenarios.
   *  This example assumes the vsd_deployment_name is set in 
   *  the vsd_autoexec or from the command line, the VSD deployment
   *  has been downloaded and registered using the macro 
   *  fcf_vsd_register_deployment and the required data has
   *  been generated and uploaded to the lasr library using
   *  the macro fcf_vsd_data_generation.  The alert and triggering
   *  transaction data generated defaults to the STG_ALER library.
   *  To change the location of the output, modify the calls to
   *  fcf_vsd_process_alerts and fcf_vsd_process_trig_txn in
   *  the ucmacro fcf_vsd_agp.
   *----------------------------------------------------------*/
  /*
  %fcf_vsd_agp;
   
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %goto MACRO_END;
  %end;
  */

  /*--------------------------
   * Process generated alerts
   *--------------------------*/


  /*----------------
   * Combine alerts
   *----------------*/
  %fcf_section_header(Section=AGP_COMBINE_ALERTS);
     %fcf_combine_alerts(inlib=STG_ALER,
                         outLib=STG_ALER,
                         outDS=combined_alerts,
                         outRiskLib=STG_ALER,
                         outRiskDS=risk_factor_alerts);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_COMBINE_ALERTS);

  /*------------------------------------------------------------
   * Test for zero alerts generated.  If no alerts end process.
   *------------------------------------------------------------*/
  %if %sysfunc(exist(STG_ALER.combined_alerts)) %then %do;
     %let dsid = %sysfunc(open(STG_ALER.combined_alerts(%fcf_get_ds_opts(lib_name=STG_ALER, ds_name=combined_alerts))));
     %let nobs = %sysfunc(attrn(&dsid, nobs));
     %let rc = %sysfunc(close(&dsid));
     %if &nobs eq 0 %then %do;
        %let _fcfmsg_=NOTE: FCF: No Alerts generated for &runasofdate..;
        %fcf_put_error;
        %goto MACRO_END;
     %end;
  %end;
  %else %goto MACRO_END;

  /*--------------
   * Risk ranking
   *--------------*/
  %fcf_section_header(Section=AGP_RISKRANKING);
     %fcf_risk_ranking(inLib=STG_ALER,
                       inDS=combined_alerts,
                       inRiskLib=STG_ALER,
                       inRiskDS=risk_factor_alerts,
                       outLib=STG_ALER,
                       outDS=ranked_alerts,
                       outRiskLib=MST_ALER,
                       outRiskDS=ranked_risk_factor_alerts,
                       riskonly=Y,
                       %if %upcase(&dbflavor) eq TERADATA %then %do;
                        bl_temp=NO,
                       %end;
                       %else %do;
                        bl_temp=YES,
                       %end;
                       bl_tempload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_RISKRANKING);

  /*-------------
   * Suppression
   *-------------*/
  %fcf_section_header(Section=AGP_SUPPRESSION);
     %fcf_suppression(inLib=STG_ALER,
                      inDS=ranked_alerts,
                      outLib=STG_ALER,
                      outDS=suppressed_alerts);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_SUPPRESSION);

  /*-----------------------
   *Get entity information
   *-----------------------*/
  %fcf_section_header(Section=AGP_GET_ENTITY_INFO);
     %fcf_get_entity_info(inLIB=STG_ALER,
                          inDS=suppressed_alerts,
                          outLIB=MST_ALER,
                          outDS=entity_alerts,
                          %if %upcase(&dbflavor) eq TERADATA %then %do;
                           bl_temp=NO,
                          %end;
                          %else %do;
                           bl_temp=YES,
                          %end;
                          bl_tempload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_GET_ENTITY_INFO);

    /**********************************************************\
    | Route alerts to queues                                   |
    \**********************************************************/

    %fcf_get_property_value(property=fcf.svr.aml.useRoutingToQueues,output_property_value=CUSTOM_ROUTING,
    output_status=property_status);

    %if &property_status ne 0 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to get property fcf.svr.aml.useRoutingToQueues from SEG_KC.FSK_PREFERENCE;
        %let trans_rc = 999999;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %let CUSTOM_ROUTING=&CUSTOM_ROUTING;
    %if &CUSTOM_ROUTING. eq true %then %do;    /* route alerts to queues    */
        %fcf_route_alert_queue(Lib=MST_ALER,
                               DS=entity_alerts);
    %end;


  /*-----------------------
   * Close alerts for SAR90 suspects
   *-----------------------*/
  %fcf_section_header(Section=AGP_ADD_ALERTS_TO_SAR90_CASES);
     %fcf_add_to_sar90_cases(inLIB=MST_ALER,
                          inDS=entity_alerts,
                          outLIB=MST_ALER,
                          outDS=entity_sar90_alerts
                          );
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_ADD_ALERTS_TO_SAR90_CASES);

  /*----------------------------------
   * Make the UI unavailable to users
   *----------------------------------*/
  %fcf_set_ui_availability(availability_ind=N);

  /*-------------------------
   * Load alerts into the KC
   *-------------------------*/
  %fcf_section_header(Section=AGP_LOADALERTS);
     %fcf_load_alerts(routedAlertLib=MST_ALER,
                      routedAlertTable=entity_sar90_alerts);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_LOADALERTS);

  /*-------------------------------
   * Load risk factors into the KC
   *-------------------------------*/
  %fcf_section_header(Section=AGP_LOADRISKFACTORS);
     %fcf_load_risk_factors(riskFactorLib=MST_ALER,
                            riskFactorTable=ranked_risk_factor_alerts);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_LOADRISKFACTORS);
  
  /*-------------------------------------
   * Load enhanced alert data into the KC
   *-----------------------------------*/
  %fcf_section_header(Section=AGP_LOAD_ENHANCED_ALERT_DATA);
  %fcf_load_enhanced_alert_data;
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_LOAD_ENHANCED_ALERT_DATA);

  /*-------------------------------------------------------- 
   * Create replication temporary tables
   * %fcf_get_replicate_info must be run before
   * %fcf_transaction_replication and %fcf_replicate_profile
   *--------------------------------------------------------*/
  %fcf_section_header(Section=AGP_GET_REPLICATE_INFO);
     %fcf_get_replicate_info(routedAlertLib=MST_ALER,
                             routedAlertTable=entity_sar90_alerts,
                             %if %upcase(&dbflavor) eq TERADATA %then %do;
                              bl_temp=NO,
                             %end;
                             %else %do;
                              bl_temp=YES,
                             %end;
                             bl_tempload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_GET_REPLICATE_INFO);

  /*---------------------------------------------------
   * Replicate transactions
   *---------------------------------------------------*/
  %fcf_section_header(Section=AGP_TRANSACTION_REPLICATION);
     %fcf_transaction_replication(routedAlertLib=MST_ALER,
                                  routedAlertTable=entity_sar90_alerts);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_TRANSACTION_REPLICATION);


  /*------------------------------------------------
   * Replicate profile for FSC_ACCOUNT_PROFILE_FACT
   *------------------------------------------------*/
  %fcf_section_header(Section=AGP_REPLICATE_ACC_PROFILE);
     %fcf_replicate_profile(routedAlertLib=MST_ALER,
                            routedAlertTable=entity_sar90_alerts,
                            coreFactTable=fsc_account_profile_fact,
                            kcFactTable=fsk_account_profile_alert,
                            bulkload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_REPLICATE_ACC_PROFILE);

  /*----------------------------------------------
   * Replicate profile for FSC_PARTY_PROFILE_FACT
   *----------------------------------------------*/
  %fcf_section_header(Section=AGP_REPLICATE_PTY_PROFILE);
     %fcf_replicate_profile(routedAlertLib=MST_ALER,
                            routedAlertTable=entity_sar90_alerts,
                            coreFactTable=fsc_party_profile_fact,
                            kcFactTable=fsk_party_profile_alert,
                            bulkload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_REPLICATE_PTY_PROFILE);

  /*----------------------------------------------
   * Load Entity Triage Data Mart
   *----------------------------------------------*/
  %fcf_section_header(Section=AGP_LOAD_ENTITY_DATAMART);
     %fcf_ent_load_dm_process;
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_LOAD_ENTITY_DATAMART);

   /*----------------------
   * Update alert history
   *----------------------*/
  %fcf_section_header(Section=AGP_UPDATE_ALERT_HISTORY);
     %fcf_update_alert_history(inLib=MST_ALER,
                               inDS=entity_sar90_alerts,
                               bulkload=);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;
  %fcf_section_footer(Section=AGP_UPDATE_ALERT_HISTORY);

  /*--------------------------------
   * Make the UI available to users
   *--------------------------------*/
  %fcf_set_ui_availability(availability_ind=Y);

  /*------------------------------------------------------------------
   * Clear the start datetime of the FCF AGP process.
   *------------------------------------------------------------------*/
  %fcf_start_datetime_clear;
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %goto MACRO_END;
  %end;

  %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_agp;
