/*==============================================================================
| NAME:    fcf_ent_load_dm_process
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

MODULE TYPE         - PROCESS
MODULE PURPOSE      - Builds a datamart for use by the Entity Triage UI.
                      Aggregates alert information at the entity level then 
                      extracts and aggregates transactions information for 
                      each entitity and loads into the datamart.  

                      ACC, PTY, and HHD alerts are all aggregated to the PTY level.
                      Other alert types are not aggregated.

MODULE NOTES
------------
This macro is run as part of the Alert Generation Process.  The fcf_start_datetime  
macro variable must be initialized prior to calling this macro since it is used to
set the create_date. 

INPUTS
------
 None

PARAMETERS
------------
None

USAGE
-----
%fcf_ent_load_dm_process;

==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
%macro fcf_ent_load_dm_process;

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/
     %let trans_rc=0;
     %let _fcfmsg_=;

    /*-------------------------------------------------
     * Clean-up from previous run
     * Note: fsk_alerted_entities will be deleted in
     *       %fcf_ent_load_alerted_entities because the
     *        routing information needs to be saved
     *------------------------------------------------*/
    %put NOTE: FCF: Truncating Customer Explorer data mart tables;
    proc sql noprint;
      delete from seg_kc.fsk_entity_transaction;
      delete from seg_kc.fsk_alert_transaction;
      delete from seg_kc.fsk_entity_tx_agg;
      delete from seg_kc.fsk_alert_tx_agg;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _fcfmsg_=ERROR: FCF: Error deleting data from entity triage datamart.;
      %fcf_put_error;
      %goto MACRO_END;
    %end;

    /* Load Entities with alerts */
    %put NOTE: FCF: Loading Entities with alerts;
    %fcf_ent_load_alerted_entities;
    %fcf_get_observation_count(table=seg_kc.fsk_alerted_entity);
      %if &fcf_obs_count eq -1 %then %do;
         %let _fcfmsg_=NOTE: FCF: seg_kc.fsk_alerted_entity does not exist.;
         %fcf_put_error;
         %put &_fcfmsg_;
         %goto MACRO_END;
      %end;
      %else %if &fcf_obs_count eq 0 %then %do;
         %let _fcfmsg_=NOTE: FCF: No records to process.  Skipping execution.;
         %fcf_put_error;
         %goto MACRO_END;
      %end;

    /* Load transactions from the fact tables for all alerted entities */
    /* Call fcf_get_replicate_info to update the FSK_ACCOUNT_KEYS_xxx  */
    /* tables to include accounts for all active alerts prior to       */
    /* calling fcf_ent_load_entity_txn.  The key tables currently      */
    /* contain info on new alerts for use by transaction replication   */
 
    %put NOTE: FCF: Loading Transactions for Alerted Entities;

    /* Save value for macro variable replicate_suppressed_alerts then  */
    /* set the macro variable to N.  The datamart should not contain   */
    /* suppressed alerts.                                              */

    %let replicate_suppressed_alerts_val=&replicate_suppressed_alerts;
    %let replicate_suppressed_alerts=N;
    %fcf_get_replicate_info(routedAlertLib=seg_kc,routedAlertTable=fsk_alert,
      %if %upcase(&dbflavor) eq TERADATA %then %do;
        bl_temp=NO
      %end;
      %else %do;
        bl_temp=YES
      %end;
    );
    %let replicate_suppressed_alerts=&replicate_suppressed_alerts_val;

    %fcf_ent_load_entity_txn;

    %fcf_get_observation_count(table=seg_kc.fsk_entity_transaction);
      %if &fcf_obs_count eq -1 %then %do;
         %let _fcfmsg_=NOTE: FCF: seg_kc.fsk_entity_transaction does not exist.;
         %fcf_put_error;
         %put &_fcfmsg_;
         %goto MACRO_END;
      %end;
      %else %if &fcf_obs_count eq 0 %then %do;
         %let _fcfmsg_=NOTE: FCF: No records to process.  Skipping execution.;
         %fcf_put_error;
         %goto MACRO_END;
      %end;

    /* Load transactions replicated for entity alerts */
    %put NOTE: FCF: Loading alerted entity replicated transactions;

    %fcf_ent_load_alert_txn;
     
    %fcf_get_observation_count(table=seg_kc.fsk_alert_transaction);
      %if &fcf_obs_count eq -1 %then %do;
         %let _fcfmsg_=NOTE: FCF: seg_kc.fsk_alert_transaction does not exist.;
         %fcf_put_error;
         %put &_fcfmsg_;
         %goto MACRO_END;
      %end;
      %else %if &fcf_obs_count eq 0 %then %do;
         %let _fcfmsg_=NOTE: FCF: No records to process.  Skipping execution.;
         %fcf_put_error;
         %goto MACRO_END;
      %end;

      /* Load transaction aggregates for entities */
    %put NOTE: FCF: Loading transaction aggregates for entities;
    %fcf_ent_load_entity_tx_agg;
    %fcf_get_observation_count(table=seg_kc.fsk_entity_tx_agg);
      %if &fcf_obs_count eq -1 %then %do;
         %let _fcfmsg_=NOTE: FCF: seg_kc.fsk_entity_tx_agg does not exist.;
         %fcf_put_error;
         %put &_fcfmsg_;
         %goto MACRO_END;
      %end;
      %else %if &fcf_obs_count eq 0 %then %do;
         %let _fcfmsg_=NOTE: FCF: No records to process.  Skipping execution.;
         %fcf_put_error;
         %goto MACRO_END;
      %end;

    /* Load transaction aggregates for customer alerts */
    %put NOTE: FCF: Loading transaction aggregates for customer alerts;
    %fcf_ent_load_alert_tx_agg;
    %fcf_get_observation_count(table=seg_kc.fsk_alert_tx_agg);
      %if &fcf_obs_count eq -1 %then %do;
         %let _fcfmsg_=NOTE: FCF: seg_kc.fsk_alert_tx_agg does not exist.;
         %fcf_put_error;
         %put &_fcfmsg_;
         %goto MACRO_END;
      %end;
      %else %if &fcf_obs_count eq 0 %then %do;
         %let _fcfmsg_=NOTE: FCF: No records to process.  Skipping execution.;
         %fcf_put_error;
         %goto MACRO_END;
      %end;

    /* Calculate and update transactions amount and count for entities with alerts */
    %put NOTE: FCF: Updating transactions amount and count for entities with alerts;
    proc sql noprint;
        create table entity_tx_aggregates as 
             Select alerted_entity_level_code, 
                    alerted_entity_number, 
                    count(*) As cnt,
                    sum(transaction_amt) as amt
             from seg_kc.fsk_alert_transaction
             where triggering_indicator_cd='Y'
                    group by alerted_entity_level_code, alerted_entity_number;

        create table alerted_entity_aggs as
             select A.alerted_entity_level_code,
                    A.alerted_entity_number,
                    coalesce(B.cnt,0) as cnt,
                    coalesce(B.amt,0) as amt
             from seg_kc.fsk_alerted_entity A
             left join entity_tx_aggregates B
             on A.alerted_entity_level_code=B.alerted_entity_level_code
             and A.alerted_entity_number=B.alerted_entity_number;

        update seg_kc.fsk_alerted_entity as A 
             set aggregate_amt = (select amt from alerted_entity_aggs as B
                                  where A.alerted_entity_level_code = B.alerted_entity_level_code
                                    and A.alerted_entity_number = B.alerted_entity_number) 
                                    ;

        update seg_kc.fsk_alerted_entity as A 
             set transactions_cnt = (select cnt from alerted_entity_aggs as B
                                     where A.alerted_entity_level_code = B.alerted_entity_level_code
                                    and A.alerted_entity_number = B.alerted_entity_number) 
                                    ;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
       %let _fcfmsg_=ERROR: FCF: Cannot delete data from datamart.;
       %fcf_put_error;
       %goto MACRO_END;
    %end;

    /* Clean-up: Delete temporary customer_tx_aggregates dataset */
    %put NOTE: FCF: Deleting temporary customer_tx_aggregates dataset;
    proc datasets library=work nolist;
        delete entity_tx_aggregates;
        delete alerted_entity_aggs;
    quit;

    %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(fcf_success);
   %end;
%mend fcf_ent_load_dm_process;
