/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_purge_from_table.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Purge from a database table
+------------------------------------------------------------------------------+
| USAGE: 
|   %fcf_purge_from_table(
|      table=&db_lib_kc..fsk_comment,
|      entities_ds=ras_to_delete,
|      key_vars=risk_assessment_id_char,
|      one_to_many_ind=Y,
|      rename_var_from=risk_assessment_id_char,
|      rename_var_to=object_id,
|      db_key_vars=object_type_cd object_id,
|      db_key_elc_var=object_type_cd,
|      db_key_elc_value=RAS,
|      other_output_ds=comments_to_delete,
|      other_output_ds_vars=object_id object_type_cd comment_id);
+------------------------------------------------------------------------------+
| COMMENTS: 
|   table= Database table (including schema) to purge from.
|   entities_ds= Data set containing entities (their keys) to purge
|   key_vars= Entity key
|   one_to_many_ind= Is this a one to one or one to many relationship between
|      the key and what's in the table.
|   other_output_ds= If set, the macro will output what was purged, or what
|      would have been purged, to this data set.
|   other_output_ds_vars= The variables to include in the other_output_ds
|   rename_var_from= If set, will rename this variable to whatever is set in
|      the rename_var_to parameter.
|   rename_var_to= See rename_var_from
|   db_key_vars= The column(s) used to search through the table for data to
|      purge.  Not necessarily the same as those specified in key_vars.
|   db_key_elc_var= Some key data sets only have a party_number for example,
|      but we need to use that key to purge from tables which also have a
|      entity level code such as PTY.  This variable specifies the entity
|      level code in the table being purged from.
|   db_key_elc_value= The value of the entity level code variable, such as
|      PTY.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2019 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_purge_from_table(
                            table=,
                            entities_ds=,
                            key_vars=,
                            one_to_many_ind=,
                            other_output_ds=,
                            other_output_ds_vars=,
                            rename_var_from=,
                            rename_var_to=,
                            db_key_vars=,
                            db_key_elc_var=,
                            db_key_elc_value=
                           );
  /*------------
   * Set Module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=FCF_PURGE_FROM_TABLE;

  /*------------------------------------------
   * Call prefcf_purge_from_table stub macro.
   *------------------------------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*-------------------------------------------
   * Set error code to 0 and message to blank.
   *-------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;

  /*------------------
   * Purge from table
   *------------------*/
  %if &one_to_many_ind eq Y %then
  %do;
     /* one to many */
     data &table
          %if &other_output_ds ne  %then
          %do;
             &other_output_ds
                             %if &other_output_ds_vars ne  %then
                             %do;
                                (keep=&other_output_ds_vars rec_purged_by_key_counter)
                             %end;
          %end;
          ;
      %if &db_key_elc_var ne  %then
      %do;
         if 0 then set &table(keep=&db_key_elc_var);
      %end;
      set &entities_ds(keep=&key_vars
                       %if &rename_var_from ne  %then
                       %do;
                          rename=(&rename_var_from=&rename_var_to)
                       %end;
                      );
       %if &db_key_elc_var ne  %then
       %do;
          &db_key_elc_var = "&db_key_elc_value";
       %end;
       rec_purged_by_key_counter = 0;
       do until (_iorc_ ne %sysrc(_sok));
          modify &table(cntllev=record
                          dbkey=(
                                 %if &db_key_vars ne  %then
                                 %do;
                                    &db_key_vars
                                 %end;
                                 %else
                                 %do;
                                    &key_vars
                                 %end;
                                )
                          %if &db_key_elc_var ne  %then
                          %do;
                             dbsastype=(&db_key_elc_var = 'char(3)')
                          %end;
                       ) key=dbkey;
          if _iorc_ = %sysrc(_sok) then
          do;
             %if %upcase(&test_run) eq N %then
             %do;
                remove &table;
             %end;
             rec_purged_by_key_counter + 1;
             %if &other_output_ds ne  %then
             %do;
                output &other_output_ds;
             %end;
          end;
          else
          do;
             _error_ = 0;
          end;
       end;
      run;
  %end;
  %else
  %do;
     /* one to one */
     data &table
          %if &other_output_ds ne  %then
          %do;
             &other_output_ds
                             %if &other_output_ds_vars ne  %then
                             %do;
                                (keep=&other_output_ds_vars rec_purged_by_key_counter)
                             %end;
          %end;
          ;
      set &entities_ds(keep=&key_vars);
       rec_purged_by_key_counter = 0;
       modify &table(cntllev=record dbkey=(&key_vars)) key=dbkey;
       if _iorc_ = %sysrc(_sok) then
       do;
          %if %upcase(&test_run) eq N %then
          %do;
             remove &table;
          %end;
          rec_purged_by_key_counter + 1;
          %if &other_output_ds ne  %then
          %do;
             output &other_output_ds;
          %end;
       end;
       else
       do;
          _error_ = 0;
       end;
      run;
  %end;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _fcfmsg_=ERROR: FCF: Error purging from &table;
      %fcf_put_error;
      %goto MACRO_END;
  %end;


  %MACRO_END:

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
     %fcf_execute_macro(fcf_error);
  %else %do;
     %fcf_execute_macro(post&_fcfmodule_);
     %fcf_execute_macro(fcf_success);
  %end;

%mend fcf_purge_from_table;

