/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_transaction_replication.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Handle custom and regular transaction replication for AML AGP
+------------------------------------------------------------------------------+
| USAGE:   %fcf_transaction_replication( routedAlertLib=, routedAlertTable=);
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS
|             routedAlertLib=    SAS Library containing routed alerts
|                                i.e. MST_ALER
|             routedAlertTable=  SAS Data Set that contains routed alerts
|                                i.e. ROUTED_ALERTS
+-------------+----------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2015-2017 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_transaction_replication(
    routedAlertLib=MST_ALER,
    routedAlertTable=entity_alerts);

    /*------------
    * Set Module
    *------------*/
    %local _fcfmodule_;
    %let _fcfmodule_=FCF_TRANSACTION_REPLICATION;


    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
     %end;

    /* Check for existance of custom replication control dataset. If not found, go to MACRO_BYPASS */

    %if %sysfunc(exist(stg_ctrl.fcf_custom_replication)) %then %do;

        /* Check for entries in custom replication control dataset. If no entries are found, go to MACRO_BYPASS */
        
        proc sql noprint;
            select count(*) into :cust_rep_num
              from stg_ctrl.fcf_custom_replication(%fcf_get_ds_opts(lib_name=stg_ctrl, ds_name=fcf_custom_replication));
        quit;

        %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: FCF: Unable to retrieve observation count from STG_CTRL.FCF_CUSTOM_REPLICATION for Custom Transaction Replication;
            %fcf_put_error;
            %goto MACRO_END;
        %end; 
    
        %if &cust_rep_num=0 %then %do;
            %let _fcfmsg_=NOTE: FCF: Custom replication dataset STG_CTRL.FCF_CUSTOM_REPLICATION is empty. Proceeding with regular transaction replication;
            %fcf_put_error;

            %goto MACRO_BYPASS;
        %end;
        %else %do;

            /* Sort the custom replication control dataset and incoming alert dataset for subsequent merge split */

            proc sort data=stg_ctrl.fcf_custom_replication(%fcf_get_ds_opts(lib_name=stg_ctrl, ds_name=fcf_custom_replication))
                       out=custom_replication_sorted;
                by scenario_name;
            run;

            %fcf_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _fcfmsg_=ERROR: FCF: Unable to sort STG_CTRL.FCF_CUSTOM_REPLICATION for Custom Transaction Replication;
                %fcf_put_error;
                %goto MACRO_END;
            %end; 

            proc sort data=&routedAlertLib..&routedAlertTable.(%fcf_get_ds_opts(lib_name=&routedAlertLib, ds_name=&routedAlertTable))
                       out=routed_alerts_sorted;
                by scenario_name;
            run;

            %fcf_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _fcfmsg_=ERROR: FCF: Unable to sort STG_CTRL.FCF_CUSTOM_REPLICATION for Custom Transaction Replication;
                %fcf_put_error;
                %goto MACRO_END;
            %end; 

            /* Merge/split the incoming alerts according to their scenario_name value existing in custom replication control dataset */
            /* Regular replication alerts go into first subset in STG_ALER.REGULAR_REP_ALERTS */
            /* Custom replication alerts go into second subset in STG_ALER.CUSTOM_REP_ALERTS with replication macro name */

            data stg_aler.regular_rep_alerts(%fcf_get_ds_opts(lib_name=stg_aler, ds_name=regular_rep_alerts, output_ds=Y))
                 stg_aler.custom_rep_alerts(%fcf_get_ds_opts(lib_name=stg_aler, ds_name=custom_rep_alerts, output_ds=Y));
                merge routed_alerts_sorted (in=reg) custom_replication_sorted (in=custom);
                by scenario_name;
                if custom and scenario_id ne . then output stg_aler.custom_rep_alerts;
                if reg and not custom and scenario_id ne . then output stg_aler.regular_rep_alerts;
            run;

            %fcf_rcset(&syserr);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _fcfmsg_=ERROR: FCF: Unable to create Alert subsets STG_ALER.REGULAR_REP_ALERTS and CUSTOM_REP_ALERTS for Custom Transaction Replication;
                %fcf_put_error;
                %goto MACRO_END;
            %end; 

            /* Run regular replication on first subset of incoming alerts */

            %fcf_replicate_transactions(
                routedAlertLib=stg_aler,
                routedAlertTable=regular_rep_alerts,
                coreFactTable=fsc_account_event_fact,
                kcFactTable=fsk_account_event_alert
            );

            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
             %end;

            %fcf_replicate_transactions(
                routedAlertLib=stg_aler,
                routedAlertTable=regular_rep_alerts,
                coreFactTable=fsc_account_trade_fact,
                kcFactTable=fsk_account_trade_alert,
                date_key=%str(settlement_date_key)
            );

            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
             %end;

            %fcf_replicate_transactions(
                routedAlertLib=stg_aler,
                routedAlertTable=regular_rep_alerts,
                coreFactTable=fsc_cash_flow_fact,
                kcFactTable=fsk_cash_flow_alert
            );

            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
             %end;

             %fcf_replicate_ext_transactions(
                 routedAlertLib=stg_aler,
                 routedAlertTable=regular_rep_alerts,
                 coreFactTable=fsc_cash_flow_fact,
                 kcFactTable=fsk_cash_flow_alert
             );

            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
            %end;

            /* If second subset is empty, bypass and go to MACRO_END */

            proc sql noprint;
                select count(*) into :cust_rep_num
                  from stg_aler.custom_rep_alerts(%fcf_get_ds_opts(lib_name=stg_aler, ds_name=custom_rep_alerts));
            quit;

            %fcf_rcset(&sqlrc);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _fcfmsg_=ERROR: FCF: Unable to retrieve STG_ALER.CUSTOM_REP_ALERTS observation count for Custom Transaction Replication;
                %fcf_put_error;
                %goto MACRO_END;
            %end; 
            
            %if &cust_rep_num=0 %then %do;
                %let _fcfmsg_=NOTE: FCF: No alerts found in STG_ALER.CUSTOM_REP_ALERTS. Bypassing custom transaction replication.;
                %fcf_put_error;

                %goto MACRO_END;
            %end;
            %else %do;

                /* Run custom replication on second subset of incoming alerts */

                %put now running new replication;

                proc sql;
                    create table work.cust_rep_to_run as
                     (select distinct macro_name,
                             count(*) as scenarios
                        from stg_aler.custom_rep_alerts(%fcf_get_ds_opts(lib_name=stg_aler, ds_name=custom_rep_alerts))
                       group by macro_name);
                quit;

                %fcf_rcset(&sqlrc);
                %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                    %let _fcfmsg_=ERROR: FCF: Unable to create CUST_REP_TO_RUN for Custom Transaction Replication;
                    %fcf_put_error;
                    %goto MACRO_END;
                %end; 
                
                proc sql noprint;
                    select count(macro_name) into :num_cust_rep_macros from work.cust_rep_to_run;
                quit;

                %fcf_rcset(&sqlrc);
                %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                    %let _fcfmsg_=ERROR: FCF: Unable to retrieve CUST_REP_TO_RUN observation count for Custom Transaction Replication;
                    %fcf_put_error;
                    %goto MACRO_END;
                %end; 

                %let num_cust_rep_macros=&num_cust_rep_macros;

                /* Create sequentially named macro variables containing the names of the macros to be run */

                proc sql noprint;
                       select lowcase(compress(macro_name))
                     into :macro1-:macro&num_cust_rep_macros
                    from work.cust_rep_to_run;
                quit;

                %fcf_rcset(&sqlrc);
                %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                    %let _fcfmsg_=ERROR: FCF: Unable to create macro variables for Custom Transaction Replication execution;
                    %fcf_put_error;
                    %goto MACRO_END;
                %end;
 
                /*  Loop through all macro variables, and execute the variable value as individual macros */
                %do i = 1 %to &num_cust_rep_macros;
                    %let macro_name=&&macro&i;

                     %&macro_name.;

                    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                        %let _fcfmsg_=ERROR: FCF: Unable to execute custom macros for Custom Transaction Replication;
                        %fcf_put_error;
                        %goto MACRO_END;
                    %end;

                %end;
            %end;
        %end;

        %goto MACRO_END;

    %end;
    %else %do;
        %let _fcfmsg_=WARNING: FCF: Transaction replication dataset STG_CTRL.FCF_TRANSACTION_REPLICATION cannot be found. Defaulting to regular transaction replication.;
        %fcf_put_error;

        %goto MACRO_BYPASS;
    %end;

/* Regular transaction replication which will be defaulted to depending on existence and content of STG_CTRL.FCF_TRANSACTION_REPLICATION */

%MACRO_BYPASS:

        %fcf_replicate_transactions(
            routedAlertLib=&routedAlertLib,
            routedAlertTable=&routedAlertTable,
            coreFactTable=fsc_account_event_fact,
            kcFactTable=fsk_account_event_alert
        );

        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
         %end;

        %fcf_replicate_transactions(
            routedAlertLib=&routedAlertLib,
            routedAlertTable=&routedAlertTable,
            coreFactTable=fsc_account_trade_fact,
            kcFactTable=fsk_account_trade_alert,
            date_key=%str(settlement_date_key)
        );

        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
         %end;

        %fcf_replicate_transactions(
            routedAlertLib=&routedAlertLib,
            routedAlertTable=&routedAlertTable,
            coreFactTable=fsc_cash_flow_fact,
            kcFactTable=fsk_cash_flow_alert
        );

        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
         %end;

         %fcf_replicate_ext_transactions(
            routedAlertLib=&routedAlertLib,
            routedAlertTable=&routedAlertTable,
            coreFactTable=fsc_cash_flow_fact,
            kcFactTable=fsk_cash_flow_alert
        );

        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
         %end;

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(fcf_success);
    %end;

%mend;
