/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_update_fsk_lov.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Update FSK_LOV table from a sashelp data set defined by DS parameter.
+------------------------------------------------------------------------------+
| NOTE:
|     Execution of this macro requires SAS Unicode session and the database
|     must support all the languages listed in the LOCALE parameter.
|
|		Tables from the data set defined by DS parameter used to update/
|     append the tables in FSk_LOV table. If a
|     row exists it is replaced, otherwise it is added to the table.
|
|
|     key_name
|          specifies the variable in the input dataset, that is used to populate
|              lov_type_name, it could be different from key variable if needed,
|              but it must have the same structure as the key variable ad described below
|
|     The key in the smd file must follow the syntax:
|         <REF_TABLE_NM>.<VALUE_CD>[.<DISPLAY_ORDER_NUMBER>.<ACTIVE_FLAG>]
|     The last two components are optional and used to specify display
|     order and active flag.
|
|     This macro is intended to be used by fcf_install.sas programs,
|     i.e. when it runs the records are added FSK_LOV
|     However the macro also supports a mode where
|     fcf_install.sas has to be re-run because of errors, in this case records
|     are deleted and then added again.
| FCFKC.FSK_LOV
|	LOV_TYPE_NAME            CHAR(18) NOT NULL
|	LOV_TYPE_CODE            CHAR(3) NOT NULL
|	LOV_TYPE_DESC            VARCHAR2(65) NOT NULL
|	LOV_SORT_POSITION_NUMBER INTEGER NULL
|	LOV_LANGUAGE_DESC        VARCHAR2(20) NULL
|   ACTIVE_FLG           CHAR(1) NOT NULL
+------------------------------------------------------------------------------+
| USAGE:
|     %fcf_update_fsk_lov
|         (DS=sashelp.amlval,
            key_name=key_orig);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016 by SAS Institute Inc., Cary, NC 27513 USA
 * All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_update_fsk_lov
           (DS=,
            defaultLanguage=en,
            key_name=key);

 /* get the default locale */
 %let fcf_config_locale_opt = %sysfunc(sasmsg(sashelp.fcflocale, &fcf_config_locale, N));

 /*workaround for Oracle bug 440366 (cannot serialize) */
 %let oracle_upd_nowhere_option=;
 %macro fcf_oracle_options();
    %if %upcase(&dbflavor) = ORACLE %then %do;
        %let oracle_upd_nowhere_option=(or_upd_nowhere=no);
    %end;
  %mend;
  %fcf_oracle_options

/* need to process each locale separately so that values are converted correctly */
 
  %let LOCALE=;
  proc sql NOPRINT;
    select distinct locale  into :LOCALE separated by ' '
      from &DS(%fcf_get_ds_opts(ds_name=&DS));
  quit;

  %let delimiter=%STR( );
  %let locale_count = %fcf_word_count(&LOCALE, &delimiter);

  %do l=1 %to &locale_count;
      %let loc = %qkscan(&LOCALE, &l, &delimiter);
      options locale=&loc;
      data fsk_lov&l;
        if 0 then set seg_kc.fsk_lov;
        set &DS.(%fcf_get_ds_opts(ds_name=&DS));
        /* keep only fsk_lov columns */
        KEEP    LOV_TYPE_NAME
                LOV_TYPE_CODE
                LOV_TYPE_DESC
                LOV_SORT_POSITION_NUMBER
                LOV_LANGUAGE_DESC
                ACTIVE_FLG
                ;
        ;
        if LOCALE = "&loc" then do;
              LOV_TYPE_NAME=kscan(key,1,'.');
              LOV_TYPE_CODE=kscan(&key_name,2,'.');
              LOV_TYPE_DESC=sasmsg("&DS", key, 'NOQUOTE');
              LOV_SORT_POSITION_NUMBER=input(kscan(key,3,'.'), BEST32.); /* could be NULL */
              LOV_LANGUAGE_DESC=LOCALE;
              ACTIVE_FLG=substr(kscan(key,4,'.'),1,1);
              if ACTIVE_FLG = "" or ACTIVE_FLG ne "N"  then ACTIVE_FLG = "Y";
              output;
        end;
      run;
      proc sql;
            delete from seg_kc.fsk_lov &oracle_upd_nowhere_option l
                where exists (select lov_type_code from fsk_lov&l d
                                where   l.LOV_TYPE_NAME =   d.LOV_TYPE_NAME and
                                        l.LOV_TYPE_CODE =   d.LOV_TYPE_CODE and
                                        l.LOV_LANGUAGE_DESC =   d.LOV_LANGUAGE_DESC
                             )
            ;
      quit;
      proc append base=seg_kc.fsk_lov data=fsk_lov&l;
      run;
  %end;
  /* restore the locale to base language */
  options locale=&fcf_config_locale_opt;

%mend fcf_update_fsk_lov;
