/*==============================================================================
| NAME:    fcf_wl_dfat_transform
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

WATCH LIST PROVIDER - Australian Department of Foreign Affairs and Trade
WATCH LIST NAME     - Consolidated List
WATCH LIST CODE     - DFAT
WEB LINK            - http://www.dfat.gov.au/icat/UNSC_financial_sanctions.html

MODULE TYPE         - TRANSFORM
MODULE PURPOSE      - Transform data in prepartion for data load.

MODULE NOTES
------------
None

==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_wl_dfat_transform;

  /*------------
   * Set module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=FCF_WL_DFAT_TRANSFORM;

  /*---------------------
   * Call pre macro stub
   *---------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*------------------------------------------
   * Set error code to 0 and message to blank
   *------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;


  /****************************************************************************/
  /* STEP 1                                                                   */
  /*                                                                          */
  /* DESCRIPTION:                                                             */
  /*                                                                          */
  /* Initial process of indvidual and organization records.                   */
  /* Output distinct tables for date_of_birth, citizenship and address values */
  /* as these data values require additional processing.                      */
  /*                                                                          */
  /* NOTES:                                                                   */
  /*                                                                          */
  /* 1. The source column 'additional_information' may contain information    */
  /*    such as title, designation, passport and national identification      */
  /*    numbers.  The 'additional_information' column is not processed due to */
  /*    the complexities involved in parsing the semi-free form text          */
  /*    correctly.  Users may wish to add their own routines which perform    */
  /*    the parsing required to process information in this column.           */
  /*                                                                          */
  /* INPUTS(S);                                                               */
  /*                                                                          */
  /* STG_WTCH.wl_dfat_extract - DFAT extract file                             */
  /*                                                                          */
  /* OUTPUT(S):                                                               */
  /*                                                                          */
  /* STG_WTCH.wl_dfat_transform_01 - Individuals                              */
  /* STG_WTCH.wl_dfat_transform_02 - Organizations                            */
  /* STG_WTCH.wl_dfat_transform_03 - Individuals (date_of_birth)              */
  /* STG_WTCH.wl_dfat_transform_04 - Individuals (citizenship)                */
  /* STG_WTCH.wl_dfat_transform_05 - Individuals-Organizations (address)      */
  /* STG_ERR.wl_dfat_errors_01     - Invalid data values                      */
  /*                                                                          */
  /****************************************************************************/
  data
    stg_wtch.wl_dfat_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_01, output_ds=Y) keep=
      entity_watch_list_number
      watch_list_name
      type_desc
      category_desc
      programs
      entity_name
      update_date
      exclude_ind
      )

    stg_wtch.wl_dfat_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_02, output_ds=Y) keep=
      entity_watch_list_number
      watch_list_name
      type_desc
      category_desc
      programs
      entity_name
      update_date
      exclude_ind
      place_of_birth
      date_of_birth
      year_of_birth
      citizenship_country_name
      )

    stg_wtch.wl_dfat_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_03, output_ds=Y) keep=
      watch_list_name
      entity_watch_list_number
      date_of_birth_
      )

    stg_wtch.wl_dfat_transform_03a(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_03a, output_ds=Y) keep=
      entity_watch_list_number
      place_of_birth_)

    stg_wtch.wl_dfat_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_04, output_ds=Y) keep=
      entity_watch_list_number
      citizenship_
      )

    stg_wtch.wl_dfat_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_05, output_ds=Y) keep=
      entity_watch_list_number
      address_
      )

    stg_err.wl_dfat_errors_01(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_dfat_errors_01, output_ds=Y) keep=
      entity_watch_list_number
      watch_list_name
      error_file
      error_record_number
      error_field
      error_value
      error_message
      error_datetime
      );

    retain error_flag 0;

    attrib
      /* ERROR FIELDS */
      error_file         length=$200
                         informat=$200.
                         format=$200.
                         label='Error File'

      error_record_number length=8
                         informat=best32.
                         format=best32.
                         label='Error Record Number'

      error_field        length=$32
                         informat=$32.
                         format=$32.
                         label='Error Field'

      error_value        length=$1000
                         informat=$1000.
                         format=$1000.
                         label='Error Value'

      error_message      length=$200
                         informat=$200.
                         format=$200.
                         label='Error Message'

      error_datetime     length=8
                         informat=nldatm.
                         format=nldatm.
                         label='Error Datetime'
      ;

    /* FSC_ENTITY_WATCH_LIST_DIM */
    /* Sets variable attribs to match variables in DBMS */
    if 1 eq 0 then
      set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

    set stg_wtch.wl_dfat_extract(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_extract)
                                 rename=(date_of_birth=date_of_birth_
                                         place_of_birth=place_of_birth_
                                         citizenship=citizenship_
                                         address=address_)
      where=(upcase(name_type) ne 'ORIGINAL SCRIPT' and reference ne '')) end=eof;

    /* ENTITY_WATCH_LIST_NUMBER */
   /*--------------------------------------------------------------------------------------
    * K Function note:  Stays compress() since kcompress doesn't support modifier argument
    *--------------------------------------------------------------------------------------*/
    entity_watch_list_number=kstrip(compress(reference,,"a"));

    /* WATCH_LIST_NAME */
    watch_list_name='DFAT';

    /* CATEGORY_DESC */
    category_desc='';

    /* PROGRAMS */
	programs='';
    if upcase(committees) in ("1267 (AL-QAIDA AND TALIBAN)",
                              "1518 (IRAQ)",
                              "1521 (LIBERIA)",
                              "1533 (DEMOCRATIC REPUBLIC OF THE CONGO)",
                              "1572 (COTE D'IVOIRE)",
                              "1591 (SUDAN)",
                              "1718 (DPRK)",
                              "1737 (IRAN)") then
      programs='UNSC -' || kstrip(upcase(committees));

    /* ENTITY_NAME */
    entity_name=upcase(compbl(kstrip(name_of_individual_or_entity)));

    /* EXCLUDE_IND */
    exclude_ind='N';

    /* UPDATE_DATE */
    update_date=.;  * reset value for each record in extract data set ;
    if not missing(control_date) then
      update_date=dhms(control_date,00,00,00);

    /* ENTITY_NAME */
    if verify(kcompress(entity_name), '?') = 0 then do;
      watch_list_name='DFAT';
      error_file="&dfat_file";
      error_record_number=_N_;
      error_field='ENTITY_NAME';
      error_value=kstrip(type);
      error_message='ERROR: Invalid entity name.';
      error_datetime=datetime();

      putlog 'WARNING: FCF: Invalid entity_name. ' entity_watch_list_number= entity_name=;
      error_flag = 1;

      output stg_err.wl_dfat_errors_01;
    end;
    else do;
    
      /* TYPE */
      if upcase(type) not in ('ENTITY','INDIVIDUAL','NGO') then do;

        watch_list_name='DFAT';
        error_file="&dfat_file";
        error_record_number=_N_;
        error_field='TYPE';
        error_value=kstrip(type);
        error_message='ERROR: Unknown TYPE.';
        error_datetime=datetime();

        putlog 'WARNING: FCF: Unknown TYPE. ' entity_watch_list_number= type=;
        error_flag = 1;

        output stg_err.wl_dfat_errors_01;
      end;
      else if upcase(type) eq 'INDIVIDUAL' then do;

        /* TYPE_DESC */
        type_desc='INDIVIDUAL';

        output stg_wtch.wl_dfat_transform_01
               stg_wtch.wl_dfat_transform_03
               stg_wtch.wl_dfat_transform_03a
               stg_wtch.wl_dfat_transform_04
               stg_wtch.wl_dfat_transform_05;

      end;
      else if upcase(type) in ('ENTITY','NGO') then do;

        /* TYPE_DESC */
        type_desc='ORGANIZATION';

		/* PLACE_OF_BIRTH */
        place_of_birth='';

        output stg_wtch.wl_dfat_transform_02
               stg_wtch.wl_dfat_transform_05;

      end;
    end;

    if (eof) and (error_flag) then do;
      putlog ' ';
      putlog 'WARNING: FCF: Invalid values were encountered processing the watch list data';
      putlog 'WARNING: FCF: Data values were written to the STG_ERR.WL_DFAT_ERRORS_01 table.';
    end;

  run;

  /****************************************************************************/
  /* STEP 2                                                                   */
  /*                                                                          */
  /* DESCRIPTION:                                                             */
  /*                                                                          */
  /* Process DATE_OF_BIRTH and/or YEAR_OF_BIRTH.                              */
  /*                                                                          */
  /* NOTES:                                                                   */
  /* Depending on the engine used to extract the data, some date fields will  */
  /* be text and others numeric.  For text fields, some Excel values will be  */
  /* their numeric representation, days since 1900.  Care must be taken when  */
  /* converting dates.                                                        */
  /*                                                                          */
  /* INPUTS(S);                                                               */
  /*                                                                          */
  /* stg_wtch.wl_dfat_transform_03 - Individuals (date_of_birth)              */
  /*                                                                          */
  /* OUTPUT(S):                                                               */
  /*                                                                          */
  /* STG_WTCH.wl_dfat_transform_06 - Individuals (date_of_birth/year_of_birth)*/
  /* STG_ERR.wl_dfat_errors_02 - Invalid date_of_birth values                 */
  /*                                                                          */
  /****************************************************************************/
  %macro fcf_parse_dob;

        /* Remove ALT DOB if present: ALT DOB: JAN 1, 1964 */
        dob_temp = prxchange('s/(.*ALT DOB:*\W*(\w+))/$2/',-1,dob_temp);

        /* Check for year range, i.e. 1955-1958 */
        position = prxmatch(year_range_re, dob_temp);
        if position > 0 then do;
          first_dob=prxposn(year_range_re, 1, dob_temp);
          last_dob=prxposn(year_range_re, 2, dob_temp);
          do j=input(first_dob,8.) to input(last_dob,8.);
             date_of_birth=.;
             year_of_birth=j;
             output stg_wtch.wl_dfat_transform_06;
          end;
        end; /* End of year range */

        else do;
          /* YEAR ONLY */
          position = prxmatch(year_only_re, dob_temp);
          if position > 0 then do;
            yob = prxposn(year_only_re, 1, dob_temp);
            date_of_birth=.;
            year_of_birth=input(yob,best32.);
          end;
		  if lengthn(dob_temp) eq 4 then do;
		     date_of_birth=.;
			 year_of_birth=input(yob,best32.);
		  end;

          /* MORE THAN YEAR */
          else do;
		  /* Convert from Excel format, i.e. 31028 => 12DEC1984 */
            if prxmatch(excel_re, dob_temp) > 0 then do;
              date_of_birth = dhms(input(dob_temp,best.)-21916,0,0,0);
		      year_of_birth=year(datepart(date_of_birth));
            end;
			else do;
            /* Regular conversion attempt */
            date_of_birth=input(dob_temp,??anydtdte32.);
            if not missing(date_of_birth) then do;
               date_of_birth=dhms(date_of_birth,00,00,00);
               year_of_birth=year(datepart(date_of_birth));
            end;
			end;
          end;

          if (year_of_birth < 1900 or year_of_birth > 2012) then do;

            year_of_birth=.;
            date_of_birth=.;

            watch_list_name='DFAT';
            error_file="&dfat_file";
            error_record_number=_N_;
            error_field='DATE_OF_BIRTH';
            error_value=kstrip(date_of_birth_);
            error_message='WARNING: Invalid date of birth.';
            error_datetime=datetime();

            putlog 'WARNING: FCF: Invalid date of birth. ' entity_watch_list_number= dob_temp=;
            error_flag = 1;

            output stg_err.wl_dfat_errors_02;
          end;

          /* Valid date of birth */
          else if not missing(dob_temp) and
              not missing(year_of_birth) then do;
              output stg_wtch.wl_dfat_transform_06;
          end;
          /* Badly formatted date of birth */
          else if not missing(dob_temp) and missing(year_of_birth) then do;

              watch_list_name='DFAT';
              error_file="&dfat_file";
              error_record_number=_N_;
              error_field='DATE_OF_BIRTH';
              error_value=kstrip(date_of_birth_);
              error_message='WARNING: Invalid date of birth.';
              error_datetime=datetime();

              putlog 'WARNING: FCF: Invalid date of birth. ' entity_watch_list_number= dob_temp=;
              error_flag = 1;

              output stg_err.wl_dfat_errors_02;
          end;
        end;
  %mend fcf_parse_dob;


  data
   stg_wtch.wl_dfat_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_06, output_ds=Y) keep=
      date_of_birth
      year_of_birth
      entity_watch_list_number
      watch_list_name)

   stg_err.wl_dfat_errors_02(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_dfat_errors_02, output_ds=Y) keep=
      entity_watch_list_number
      watch_list_name
      error_file
      error_record_number
      error_field
      error_value
      error_message
      error_datetime)
      ;

   attrib
      first_dob     length=$100 informat=$100.    format=$100.
      last_dob      length=$100 informat=$100.    format=$100.
	  dob_temp      length=$200

   /* ERROR FIELDS */
      error_file          length=$200
                          informat=$200.
                          format=$200.
                          label='Error File'

      error_record_number length=8
                          informat=best32.
                          format=best32.
                          label='Error Record Number'

      error_field         length=$32
                          informat=$32.
                          format=$32.
                          label='Error Field'

      error_value         length=$1000
                          informat=$1000.
                          format=$1000.
                          label='Error Value'

      error_message       length=$200
                          informat=$200.
                          format=$200.
                          label='Error Message'

      error_datetime      length=8
                          informat=nldatm.
                          format=nldatm.
                          label='Error Datetime'
   ;

   retain between_re year_range_re year_only_re excel_re;
   drop date_of_birth_orig i j yob error_flag dob_temp position between_re
        year_range_re year_only_re first_dob last_dob;

  /* FSC_ENTITY_WATCH_LIST_DIM */
  /* Sets variable attribs to match variables in DBMS */
  if 1 eq 0 then
    set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

  if _n_ eq 1 then do;
    * call prxdebug(1);
    between_re    = prxparse('/BETWEEN\s*(\d{4})\s*(TO|AND)\s*(\d{4})/');
    year_only_re  = prxparse('/^(\d{4})\D/');
    year_range_re = prxparse('/(\d{4})\s*-\s*(\d{4})/');
    excel_re      = prxparse('/^(\d{5})\D/');
  end;

  set stg_wtch.wl_dfat_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_03)
                                    where=(upcase(date_of_birth_) ne 'NA' and date_of_birth_ ne '')) end=eof;

  /* Necessary to upcase to catch keywords such as CIRCA and BETWEEN */
  date_of_birth_orig = date_of_birth_;
  date_of_birth_=upcase(date_of_birth_);

  date_of_birth_=tranwrd(date_of_birth_,'JANUARY','JAN');
  date_of_birth_=tranwrd(date_of_birth_,'FEBRUARY','FEB');
  date_of_birth_=tranwrd(date_of_birth_,'MARCH','MAR');
  date_of_birth_=tranwrd(date_of_birth_,'APRIL','APR');
  date_of_birth_=tranwrd(date_of_birth_,'MAY','MAY');
  date_of_birth_=tranwrd(date_of_birth_,'JUNE','JUN');
  date_of_birth_=tranwrd(date_of_birth_,'JULY','JUL');
  date_of_birth_=tranwrd(date_of_birth_,'AUGUST','AUG');
  date_of_birth_=tranwrd(date_of_birth_,'SEPTEMBER','SEP');
  date_of_birth_=tranwrd(date_of_birth_,'OCTOBER','OCT');
  date_of_birth_=tranwrd(date_of_birth_,'NOVEMBER','NOV');
  date_of_birth_=tranwrd(date_of_birth_,'DECEMBER','DEC');

  /* Remove APPROXIMATELY or APPROX */
  date_of_birth_=tranwrd(date_of_birth_,'APPROXIMATELY','');
  date_of_birth_=tranwrd(date_of_birth_,'APPROX','');

  /* Remove POSSIBLY */
  date_of_birth_=tranwrd(date_of_birth_,'POSSIBLY','');

  /* Remove CIRCA or C. or C at start of date, i.e. CIRCA 1956 */
  date_of_birth_= prxchange('s/^(CIRCA|C\.|C)//',-1,date_of_birth_);

  /* Remove DOB: at start of date, i.e. DOB 01-01-1956 */
  date_of_birth_= prxchange('s/^DOB://',-1,date_of_birth_);

  /* Reformat delimiters in form A-Z) to '|' to indicate multiple dates */
  if indexw(date_of_birth_,'A)') then
    substr(date_of_birth_,indexw(date_of_birth_,'A)'),2)='';
  date_of_birth_ = prxchange('s/\s+[A-Z]\)\s+/|/',-1,date_of_birth_);

  /* Reformat dates with OR to indicated multiple dates */
  if indexw(date_of_birth_,'OR') and not kindex(date_of_birth_,'|') then
    date_of_birth_=tranwrd(date_of_birth_,'OR','|');

	/* Reformat dates separated by ; to indicate multiple dates */
	/* Reformat dates separated by , to indicate multiple dates */
	/*  -- can cause errors because of dates such as September 1, 1966 */
   if kindex(date_of_birth_,';') and not kindex(date_of_birth_,'|') then
      date_of_birth_=translate(date_of_birth_,'|',';');
   else if kindex(date_of_birth_,';|') then
      date_of_birth_=translate(date_of_birth_,'',';');
   else if kindex(date_of_birth_,',|') then
      date_of_birth_=translate(date_of_birth_,'',',');

    /* Remove '.' characters, i.e. DEC. 25 */
   date_of_birth_=kstrip(kcompress(date_of_birth_,'.'));

   /* Reformat dates separated by '>>' to indicate multiple dates */
   if kindex(date_of_birth_,'>>') and not kindex(date_of_birth_,'|') then
      date_of_birth_=tranwrd(date_of_birth_,'>>','|');

   /* Reformat following date formats to indicate multiple dates */
   /* 22 MAY 1953, 23 MAY 1953 */
   /* 01/31/1956, 02/01/1956 */
   date_of_birth_ = prxchange('s/(\d{2}\s+[A-Z]{3}\s+\d{4}),(.+)/$1|$2/',-1,date_of_birth_);
   date_of_birth_ = prxchange('s!(\d{1,2}/\d{2}/\d{4}),(.+)!$1|$2!',-1,date_of_birth_);

   /* Reformat dates separated by '(' to indicate multiple dates */
   /* (01JAN1960) (02JAN1958) (03FEB1955) */
   if kindex(date_of_birth_,'(') and not kindex(date_of_birth_,'|') then do;
      date_of_birth_=tranwrd(date_of_birth_,'(','|');
      date_of_birth_=tranwrd(date_of_birth_,')','');
   end;

   /* Debug */
   * put / entity_watch_list_number= dob_orig=;

   /* Handle multiple dates specified */
   if kindex(date_of_birth_,'|') then do;
      do i=1 to countc(date_of_birth_,'|') + 1;

	     dob_temp=kstrip(kscan(date_of_birth_,i,'|'));
		 dob_temp=kcompress(dob_temp,'|');

         /* Check for BETWEEN rage, change to dash separated */
         /* Between 1959 and 1960 => 1959-1960 */
         position = prxmatch(between_re, dob_temp);
         if position > 0 then do;
            first_dob=prxposn(between_re, 1, dob_temp);
            last_dob=prxposn(between_re,  3, dob_temp);
            dob_temp = catx('-',first_dob ,last_dob);
         end; /* End of 'between' range */

         /* Debug */
         * put dob_temp=;

         %fcf_parse_dob;

		 /* reset values */
		 date_of_birth=.;  year_of_birth=.;

      end; /* End of loop through multiple years */
   end; /* End of multiple dates detected */

   /* Single date given */
   else do;
      dob_temp = kstrip(date_of_birth_);
      
      /* Check for BETWEEN rage, change to dash separated */
      /* Between 1959 and 1960 => 1959-1960 */
      position = prxmatch(between_re, date_of_birth_);
      if position > 0 then do;
         first_dob=prxposn(between_re, 1, date_of_birth_);
         last_dob=prxposn(between_re,  3, date_of_birth_);
         dob_temp = catx('-',first_dob ,last_dob);
      end; /* End of 'between' range */

      /* Debug */
      * put dob_temp=;

      %fcf_parse_dob;
   end;

   if (eof) and (error_flag) then do;
      putlog ' ';
      putlog 'WARNING: FCF: Invalid values were encountered processing the watch list data';
      putlog 'WARNING: FCF: Data values were written to the STG_ERR.WL_DFAT_ERRORS_02 table.';
   end;
   call prxdebug(0);

  run;

  proc sort data=stg_wtch.wl_dfat_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_06))
            out=stg_wtch.wl_dfat_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_06, output_ds=Y))
            nodupkey;
    by entity_watch_list_number
       date_of_birth
       year_of_birth;
   run;

  /****************************************************************************/
  /* STEP 2.a                                                                 */
  /*                                                                          */
  /* DESCRIPTION:                                                             */
  /*                                                                          */
  /* Process Place_of_Birth                                                   */
  /*                                                                          */
  /* NOTES:                                                                   */
  /* Multiple addresses are sometimes stored in a single address field        */
  /* separated with a), b), etc.                                              */
  /* LIMITATION:Only 5 places will be parsed                                  */
  /*                                                                          */
  /* INPUTS(S);                                                               */
  /*                                                                          */
  /* stg_wtch.wl_dfat_transform_03a - Individuals (place_of_birth)            */
  /*                                                                          */
  /* OUTPUT(S):                                                               */
  /*                                                                          */
  /* STG_WTCH.wl_dfat_transform_06a - Individuals (place_of_birth)            */
  /*                                                                          */
  /****************************************************************************/

   data STG_WTCH.wl_dfat_transform_06a (%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_06a, output_ds=Y)
                                        keep=
                                             entity_watch_list_number 
                                             place_of_birth)
        stg_err.wl_dfat_errors_02a(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_dfat_errors_02a, output_ds=Y)
                                   keep=
                                        entity_watch_list_number
                                        watch_list_name
                                        error_file
                                        error_record_number
                                        error_field
                                        error_value
                                        error_message
                                        error_datetime)
                                     ;


      set STG_WTCH.wl_dfat_transform_03a(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_03a));
      
    attrib 
   /* ERROR FIELDS */
      error_file          length=$200
                          informat=$200.
                          format=$200.
                          label='Error File'

      error_record_number length=8
                          informat=best32.
                          format=best32.
                          label='Error Record Number'

      error_field         length=$32
                          informat=$32.
                          format=$32.
                          label='Error Field'

      error_value         length=$1000
                          informat=$1000.
                          format=$1000.
                          label='Error Value'

      error_message       length=$200
                          informat=$200.
                          format=$200.
                          label='Error Message'

      error_datetime      length=8
                          informat=nldatm.
                          format=nldatm.
                          label='Error Datetime'
   ;

	   /* replace list numbering with pipe */
	   place_of_birth_string=tranwrd(place_of_birth_,"(a) ","");
	   place_of_birth_string=tranwrd(place_of_birth_,"a) ","");
	   place_of_birth_string=tranwrd(place_of_birth_string," (b) ","|");
	   place_of_birth_string=tranwrd(place_of_birth_string," b) ","|");
	   place_of_birth_string=tranwrd(place_of_birth_string," (c) ","|");
	   place_of_birth_string=tranwrd(place_of_birth_string," c) ","|");
	   place_of_birth_string=tranwrd(place_of_birth_string," (d) ","|");
	   place_of_birth_string=tranwrd(place_of_birth_string," d) ","|");
       place_of_birth_string=tranwrd(place_of_birth_string," (e) ","|");
	   place_of_birth_string=tranwrd(place_of_birth_string," e) ","|");

       if kindex(place_of_birth_string,'|') then do;
	      do i=1 to countc(place_of_birth_string,'|') +1;
		     place_of_birth=compbl(kstrip(kscan(place_of_birth_string,i,'|')));
			 place_of_birth=tranwrd(place_of_birth,";","");
		     if not missing (place_of_birth) then 
			    if length(place_of_birth) <= 100 then 
			        output STG_WTCH.wl_dfat_transform_06a;
				else do;
				    watch_list_name='DFAT';
                    error_file="&dfat_file";
                    error_record_number=_N_;
                    error_field='PLACE_OF_BIRTH';
                    error_value=kstrip(place_of_birth);
                    error_message='WARNING: Invalid place of birth.';
                    error_datetime=datetime();

                    putlog 'WARNING: FCF: Too long place_of_birth. ' entity_watch_list_number= place_of_birth=;
                    error_flag = 1;

                    output stg_err.wl_dfat_errors_02a;
				end;
          end;
	   end;
	   else do;
            place_of_birth=compbl(kstrip(place_of_birth_string));
		     if not missing (place_of_birth) then 
			    if length(place_of_birth) <= 100 then 
			        output STG_WTCH.wl_dfat_transform_06a;
				else do;
				    watch_list_name='DFAT';
                    error_file="&dfat_file";
                    error_record_number=_N_;
                    error_field='PLACE_OF_BIRTH';
                    error_value=kstrip(place_of_birth);
                    error_message='WARNING:  Too long place of birth.';
                    error_datetime=datetime();

                    putlog 'WARNING: FCF: Invalid place_of_birth. ' entity_watch_list_number= place_of_birth=;
                    error_flag = 1;

                    output stg_err.wl_dfat_errors_02a;
				end;
	   end;
   run;

   proc sort data=STG_WTCH.wl_dfat_transform_06a(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_06a)) nodup;
      by entity_watch_list_number place_of_birth;
   run;

  /****************************************************************************/
  /* STEP 3                                                                   */
  /*                                                                          */
  /* DESCRIPTION:                                                             */
  /*                                                                          */
  /* Process CITIZENSHIP_COUNTRY_NAME                                         */
  /*                                                                          */
  /* NOTES:                                                                   */
  /*                                                                          */
  /* INPUTS(S);                                                               */
  /*                                                                          */
  /* STG_WTCH.wl_dfat_transform_04 - Individuals (citizenship)                */
  /*                                                                          */
  /* OUTPUT(S):                                                               */
  /*                                                                          */
  /* STG_WTCH.wl_dfat_transform_07 - Individuals (citizenship_country_name)   */
  /*                                                                          */
  /****************************************************************************/
  data
   stg_wtch.wl_dfat_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_07, output_ds=Y) keep=
      entity_watch_list_number
      citizenship_country_name
  );

  /* FSC_ENTITY_WATCH_LIST_DIM */
  /* Sets variable attribs to match variables in DBMS */
  if 1 eq 0 then
    set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

   set stg_wtch.wl_dfat_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_04)
                                     where=(upcase(citizenship_) ne 'NA' and
                                                   citizenship_ ne ''));

   citizenship_=upcase(citizenship_);

   citizenship_=tranwrd(citizenship_,'JANUARY','');
   citizenship_=tranwrd(citizenship_,'FEBRUARY','');
   citizenship_=tranwrd(citizenship_,'MARCH','');
   citizenship_=tranwrd(citizenship_,'APRIL','');
   citizenship_=tranwrd(citizenship_,'MAY','');
   citizenship_=tranwrd(citizenship_,'JUNE','');
   citizenship_=tranwrd(citizenship_,'JULY','');
   citizenship_=tranwrd(citizenship_,'AUGUST','');
   citizenship_=tranwrd(citizenship_,'SEPTEMBER','');
   citizenship_=tranwrd(citizenship_,'OCTOBER','');
   citizenship_=tranwrd(citizenship_,'NOVEMBER','');
   citizenship_=tranwrd(citizenship_,'DECEMBER','');
   citizenship_=tranwrd(citizenship_,'CITIZENSHIP','');
   citizenship_=tranwrd(citizenship_,'WITHDRAWN','');
   citizenship_=tranwrd(citizenship_,'OFFICIALLY','');
   citizenship_=tranwrd(citizenship_,'NATIONAL','');
   citizenship_=tranwrd(citizenship_,'THOUGHT','');
   citizenship_=tranwrd(citizenship_,'SINCE','');
   citizenship_=tranwrd(citizenship_,'UNRESOLVED','');
   citizenship_=tranwrd(citizenship_,'ORIGIN','');
   citizenship_=tranwrd(citizenship_,'BASED','');
   citizenship_=tranwrd(citizenship_,'PASSPORT','');
   citizenship_=tranwrd(citizenship_,'DIPLOMATIC','');
   citizenship_=tranwrd(citizenship_,'VALID','');
   citizenship_=tranwrd(citizenship_,'RESIDENT','');
   citizenship_=tranwrd(citizenship_,'NUMBER','');

   if indexw(citizenship_,'A') then
      substr(citizenship_,indexw(citizenship_,'A'),1)='';

   if indexw(citizenship_,'AN') then
      substr(citizenship_,indexw(citizenship_,'AN'),2)='';

   if indexw(citizenship_,'TO') then
      substr(citizenship_,indexw(citizenship_,'TO'),2)='';

   if indexw(citizenship_,'BE') then
      substr(citizenship_,indexw(citizenship_,'BE'),2)='';

   if indexw(citizenship_,'IN') then
      substr(citizenship_,indexw(citizenship_,'IN'),2)='';

   if indexw(citizenship_,'NOW') then
      substr(citizenship_,indexw(citizenship_,'NOW'),3)='';

   if indexw(citizenship_,'AND') then
      substr(citizenship_,indexw(citizenship_,'AND'),3)='';

   if indexw(citizenship_,'JAN') then
      substr(citizenship_,indexw(citizenship_,'JAN'),3)='';

   if indexw(citizenship_,'FEB') then
      substr(citizenship_,indexw(citizenship_,'FEB'),3)='';

   if indexw(citizenship_,'MAR') then
      substr(citizenship_,indexw(citizenship_,'MAR'),3)='';

   if indexw(citizenship_,'APR') then
      substr(citizenship_,indexw(citizenship_,'APR'),3)='';

   if indexw(citizenship_,'MAY') then
      substr(citizenship_,indexw(citizenship_,'MAY'),3)='';

   if indexw(citizenship_,'JUN') then
      substr(citizenship_,indexw(citizenship_,'JUN'),3)='';

   if indexw(citizenship_,'JUL') then
      substr(citizenship_,indexw(citizenship_,'JUL'),3)='';

   if indexw(citizenship_,'AUG') then
      substr(citizenship_,indexw(citizenship_,'AUG'),3)='';

   if indexw(citizenship_,'SEP') then
      substr(citizenship_,indexw(citizenship_,'SEP'),3)='';

   if indexw(citizenship_,'OCT') then
      substr(citizenship_,indexw(citizenship_,'OCT'),3)='';

   if indexw(citizenship_,'NOV') then
      substr(citizenship_,indexw(citizenship_,'NOV'),3)='';

   if indexw(citizenship_,'DEC') then
      substr(citizenship_,indexw(citizenship_,'DEC'),3)='';

   if indexw(citizenship_,'A)') then
      substr(citizenship_,indexw(citizenship_,'A)'),3)='';

   if indexw(citizenship_,'B)') then
      substr(citizenship_,indexw(citizenship_,'B)'),3)=',';

   if indexw(citizenship_,'C)') then
      substr(citizenship_,indexw(citizenship_,'C)'),3)=',';

   if indexw(citizenship_,'D)') then
      substr(citizenship_,indexw(citizenship_,'D)'),3)=',';

   if indexw(citizenship_,'E)') then
      substr(citizenship_,indexw(citizenship_,'E)'),3)=',';

   /*--------------------------------------------------------------------------------------
    * K Function note:  Stays compress() since kcompress doesn't support modifier argument
    *--------------------------------------------------------------------------------------*/
   citizenship_=compress(citizenship_,';/.()-:','dc');

   if kindex(citizenship_,',') then do;
       if kindex(citizenship_,',') then do;
         do i=1 to countc(citizenship_,',') + 1;
            citizenship_country_name=compbl(kstrip(kscan(citizenship_,i,',')));

            if not missing(citizenship_country_name) then
               output stg_wtch.wl_dfat_transform_07;
         end;
      end;
   end;
   else do;

      citizenship_country_name=compbl(kstrip(citizenship_));
      if not missing(citizenship_country_name) then
         output stg_wtch.wl_dfat_transform_07;

   end;

  run;

  proc sort data=stg_wtch.wl_dfat_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_07))
            out=stg_wtch.wl_dfat_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_07, output_ds=Y))
            nodupkey;
    by entity_watch_list_number
       citizenship_country_name;
   run;

  /****************************************************************************/
  /* STEP 4                                                                   */
  /*                                                                          */
  /* DESCRIPTION:                                                             */
  /*                                                                          */
  /* Process FULL_ADDRESS                                                     */
  /*                                                                          */
  /* NOTES:                                                                   */
  /*                                                                          */
  /* INPUTS(S);                                                               */
  /*                                                                          */
  /* STG_WTCH.wl_dfat_transform_05 - Individuals-Organizations (address)      */
  /*                                                                          */
  /* OUTPUT(S):                                                               */
  /*                                                                          */
  /* STG_WTCH.wl_dfat_transform_08 - Individuals-Organizations (full_address) */
  /*                                                                          */
  /****************************************************************************/
  data
   stg_wtch.wl_dfat_transform_08(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_08, output_ds=Y) keep=
      entity_watch_list_number
      full_address
      );

   /* FSC_ENTITY_WATCH_LIST_DIM */
   /* Sets variable attribs to match variables in DBMS */
   if 1 eq 0 then
         set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

   set stg_wtch.wl_dfat_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_05)
                                     where=(upcase(address_) ne 'NA' and
                                                   address_ ne ''));

   /*--------------------------------------------------------------------------------------
    * K Function note:  Stays compress() since kcompress doesn't support modifier argument
    *--------------------------------------------------------------------------------------*/
   address_=upcase(compress(address_,,'c'));

   /* Reformat delimiters in form A-Z) to '|' */
   if indexw(address_,'A)') then
      substr(address_,indexw(address_,'A)'),2)='';

   if indexw(address_,'B)') then
      substr(address_,indexw(address_,'B)'),2)='|';

   if indexw(address_,'C)') then
      substr(address_,indexw(address_,'C)'),2)='|';

   if indexw(address_,'D)') then
      substr(address_,indexw(address_,'D)'),2)='|';

   if indexw(address_,'E)') then
      substr(address_,indexw(address_,'E)'),2)='|';

   if indexw(address_,'F)') then
      substr(address_,indexw(address_,'F)'),2)='|';

   if indexw(address_,'G)') then
      substr(address_,indexw(address_,'G)'),2)='|';

   if indexw(address_,'H)') then
      substr(address_,indexw(address_,'H)'),2)='|';

   if indexw(address_,'I)') then
      substr(address_,indexw(address_,'I)'),2)='|';

   if indexw(address_,'J)') then
      substr(address_,indexw(address_,'J)'),2)='|';

   if indexw(address_,'K)') then
      substr(address_,indexw(address_,'K)'),2)='|';

   if indexw(address_,'L)') then
      substr(address_,indexw(address_,'L)'),2)='|';

   if indexw(address_,'M)') then
      substr(address_,indexw(address_,'M)'),2)='|';

   if indexw(address_,'N)') then
      substr(address_,indexw(address_,'N)'),2)='|';

   if indexw(address_,'O)') then
      substr(address_,indexw(address_,'O)'),2)='|';

   if indexw(address_,'P)') then
      substr(address_,indexw(address_,'P)'),2)='|';

   if indexw(address_,'Q)') then
      substr(address_,indexw(address_,'Q)'),2)='|';

   if indexw(address_,'R)') then
      substr(address_,indexw(address_,'R)'),2)='|';

   if indexw(address_,'S)') then
      substr(address_,indexw(address_,'S)'),2)='|';

   if indexw(address_,'T)') then
      substr(address_,indexw(address_,'T)'),2)='|';

   if indexw(address_,'U)') then
      substr(address_,indexw(address_,'U)'),2)='|';

   if indexw(address_,'V)') then
      substr(address_,indexw(address_,'V)'),2)='|';

   if indexw(address_,'W)') then
      substr(address_,indexw(address_,'W)'),2)='|';

   if indexw(address_,'X)') then
      substr(address_,indexw(address_,'X)'),2)='|';

   if indexw(address_,'Y)') then
      substr(address_,indexw(address_,'Y)'),2)='|';

   if indexw(address_,'Z)') then
      substr(address_,indexw(address_,'Z)'),2)='|';

   /* Reformat delimiters in form (A-Z) to '|' */
   if indexw(address_,'(A)') then
      substr(address_,indexw(address_,'(A)'),3)='';

   if indexw(address_,'(B)') then
      substr(address_,indexw(address_,'(B)'),3)='|';

   if indexw(address_,'(C)') then
      substr(address_,indexw(address_,'(C)'),3)='|';

   if indexw(address_,'(D)') then
      substr(address_,indexw(address_,'(D)'),3)='|';

   if indexw(address_,'(E)') then
      substr(address_,indexw(address_,'(E)'),3)='|';

   if indexw(address_,'(F)') then
      substr(address_,indexw(address_,'(F)'),3)='|';

   if indexw(address_,'(G)') then
      substr(address_,indexw(address_,'(G)'),3)='|';

   if indexw(address_,'(H)') then
      substr(address_,indexw(address_,'(H)'),3)='|';

   if indexw(address_,'(I)') then
      substr(address_,indexw(address_,'(I)'),3)='|';

   if indexw(address_,'(J)') then
      substr(address_,indexw(address_,'(J)'),3)='|';

   if indexw(address_,'(K)') then
      substr(address_,indexw(address_,'(K)'),3)='|';

   if indexw(address_,'(L)') then
      substr(address_,indexw(address_,'(L)'),3)='|';

   if indexw(address_,'(M)') then
      substr(address_,indexw(address_,'(M)'),3)='|';

   if indexw(address_,'(N)') then
      substr(address_,indexw(address_,'(N)'),3)='|';

   if indexw(address_,'(O)') then
      substr(address_,indexw(address_,'(O)'),3)='|';

   if indexw(address_,'(P)') then
      substr(address_,indexw(address_,'(P)'),3)='|';

   if indexw(address_,'(Q)') then
      substr(address_,indexw(address_,'(Q)'),3)='|';

   if indexw(address_,'(R)') then
      substr(address_,indexw(address_,'(R)'),3)='|';

   if indexw(address_,'(S)') then
      substr(address_,indexw(address_,'(S)'),3)='|';

   if indexw(address_,'(T)') then
      substr(address_,indexw(address_,'(T)'),3)='|';

   if indexw(address_,'(U)') then
      substr(address_,indexw(address_,'(U)'),3)='|';

   if indexw(address_,'(V)') then
      substr(address_,indexw(address_,'(V)'),3)='|';

   if indexw(address_,'(W)') then
      substr(address_,indexw(address_,'(W)'),3)='|';

   if indexw(address_,'(X)') then
      substr(address_,indexw(address_,'(X)'),3)='|';

   if indexw(address_,'(Y)') then
      substr(address_,indexw(address_,'(Y)'),3)='|';

   if indexw(address_,'(Z)') then
      substr(address_,indexw(address_,'(Z)'),3)='|';

   /* Reformat delimiters in form >> to '|' */
   if kindex(address_,'>') and not kindex(address_,'|') then
      address_=tranwrd(address_,'>>','|');

   /* Reformat delimiters in form ; to '|' */
   if kindex(address_,';') and not kindex(address_,'|') then
      address_=translate(address_,'|',';');

   if kindex(address_,'|') then do;
      do i=1 to countc(address_,'|') + 1;
         full_address=compbl(kstrip(kscan(address_,i,'|')));

         if not missing(full_address) then
            output stg_wtch.wl_dfat_transform_08;
      end;
   end;
   else do;
      full_address=compbl(kstrip(address_));
      if not missing(full_address) then
         output stg_wtch.wl_dfat_transform_08;

   end;

  run;

  proc sort data=stg_wtch.wl_dfat_transform_08(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_08))
            out=stg_wtch.wl_dfat_transform_08(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_08, output_ds=Y))
            nodupkey;
    by entity_watch_list_number
       full_address;
   run;

  /****************************************************************************/
  /* STEP 4                                                                   */
  /*                                                                          */
  /* DESCRIPTION:                                                             */
  /*                                                                          */
  /* Create distinct indvidual and organization records for each combination  */
  /* of date_of_birth/year_of_birth, citizenship_country_name and             */
  /* full_address.                                                            */
  /*                                                                          */
  /* NOTES:                                                                   */
  /*                                                                          */
  /* INPUTS(S);                                                               */
  /*                                                                          */
  /* STG_WTCH.wl_dfat_transform_01 - Individuals                              */
  /* STG_WTCH.wl_dfat_transform_02 - Organizations                            */
  /* STG_WTCH.wl_dfat_transform_06 - Individuals (date_of_birth/year_of_birth)*/
  /* STG_WTCH.wl_dfat_transform_07 - Individuals (citizenship_country_name)   */
  /* STG_WTCH.wl_dfat_transform_08 - Individuals-Organizations (full_address) */
  /*                                                                          */
  /* OUTPUT(S):                                                               */
  /*                                                                          */
  /* STG_WTCH.wl_dfat_transform_09 - Individuals (date_of_birth/year_of_birth)*/
  /* STG_WTCH.wl_dfat_transform_10 - Individuals                              */
  /* (date_of_birth/year_of_birth citizenship_country_name)                   */
  /* STG_WTCH.wl_dfat_transform    - Individuals-Organizations                */
  /* (date_of_birth/year_of_birth citizenship_country_name full_address)      */
  /*                                                                          */
  /****************************************************************************/
  proc sql noprint;
   create table stg_wtch.wl_dfat_transform_09 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_09, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.date_of_birth,
             b.year_of_birth
        from stg_wtch.wl_dfat_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_01)) as a full join
             stg_wtch.wl_dfat_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_06)) as b
             on a.entity_watch_list_number=b.entity_watch_list_number;
 
   create table stg_wtch.wl_dfat_transform_09a %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_09a, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.place_of_birth
        from stg_wtch.wl_dfat_transform_09(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_09)) as a full join
             stg_wtch.wl_dfat_transform_06a(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_06a)) as b
             on a.entity_watch_list_number=b.entity_watch_list_number;

   create table stg_wtch.wl_dfat_transform_10 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_10, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.citizenship_country_name
        from stg_wtch.wl_dfat_transform_09a(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_09a)) as a full join
             stg_wtch.wl_dfat_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_07)) as b
             on a.entity_watch_list_number=b.entity_watch_list_number;
  quit;

  proc append base=stg_wtch.wl_dfat_transform_10(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_10, output_ds=Y))
              data=stg_wtch.wl_dfat_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_02))
              force;
  run;

  proc sql noprint;
   create table stg_wtch.wl_dfat_transform %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.full_address
        from stg_wtch.wl_dfat_transform_10(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_10)) as a full join
             stg_wtch.wl_dfat_transform_08(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dfat_transform_08)) as b
             on a.entity_watch_list_number=b.entity_watch_list_number;
  quit;

  /****************************************************************************/
  /* STEP 5                                                                   */
  /*                                                                          */
  /* DESCRIPTION:                                                             */
  /*                                                                          */
  /* Process data errors                                                      */
  /*                                                                          */
  /* NOTES:                                                                   */
  /*                                                                          */
  /* INPUTS(S);                                                               */
  /*                                                                          */
  /* STG_ERR.wl_dfat_errors_01 - Invalid data values                          */
  /* STG_ERR.wl_dfat_errors_02 - Invalid date_of_birth values                 */
  /*                                                                          */
  /* OUTPUT(S):                                                               */
  /*                                                                          */
  /* STG_WTCH.wl_dfat_errors - Combined invalid data values                   */
  /*                                                                          */
  /****************************************************************************/

  proc append base=stg_err.wl_dfat_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_dfat_errors, output_ds=Y))
              data=stg_err.wl_dfat_errors_01(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_dfat_errors_01))
              force;
   run;

  proc append base=stg_err.wl_dfat_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_dfat_errors, output_ds=Y)))
              data=stg_err.wl_dfat_errors_02(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_dfat_errors_02)))
              force;
   run;

  proc append base=stg_err.wl_dfat_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_dfat_errors, output_ds=Y)))
              data=stg_err.wl_dfat_errors_02a(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_dfat_errors_02a)))
              force;
   run;

  proc sort data=stg_err.wl_dfat_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_dfat_errors)))
            out=stg_err.wl_dfat_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_dfat_errors, output_ds=Y))
            nodupkey;
    by entity_watch_list_number
       error_value;
   run;


 %MACRO_END:

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
     %fcf_execute_macro(fcf_error);
  %else %do;
     %fcf_execute_macro(post&_fcfmodule_);
     %fcf_execute_macro(fcf_success);
  %end;

%mend fcf_wl_dfat_transform;
