/*==============================================================================
| NAME:    fcf_wl_dwjn_extract
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

WATCH LIST PROVIDER - Dow Jones
WATCH LIST NAME     - Watchlist
WATCH LIST CODE     - DWJN
WEB LINK            - http://fis.dowjones.com/products/watchlist.html

MODULE TYPE         - EXTRACT
MODULE PURPOSE      - Read extract file(s) and output to SAS data set(s).

MODULE NOTES
------------
None

==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_wl_dwjn_extract;

  /*------------
   * Set module
   *------------*/
  %let _fcfmodule_=FCF_WL_DWJN_EXTRACT;

  /*---------------------
   * Call pre macro stub
   *---------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*------------------------------------------
   * Set error code to 0 and message to blank
   *------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;

  /* Make sure SAS is set for UTF-8 encoding     */
  /* Dow Jones data file is in UTF-16LE encoding */
  data _null_;
    CHK_ENCODING=getoption('ENCODING');
    if CHK_ENCODING ^=: 'UTF' then do;
       call symputx('trans_rc', 999999);
    end;
   run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_= ERROR: FCF: Encoding value is not set properly for processing this data file.;
     %fcf_put_error;
     %let _fcfmsg_= ERROR: FCF: Dow Jones watchlist data requires SAS to use UTF-8 encoding.;
     %put &_fcfmsg_;
     %let _fcfmsg_= ERROR: FCF: Restart SAS using the correct encoding options to import the Dow Jones Watchlist data.;
     %put &_fcfmsg_;
     /* Reset message for e-mail subject */
     %let _fcfmsg_= ERROR: FCF: Dow Jones watchlist data requires SAS to use UTF-8 encoding.;
     %goto MACRO_END;
  %end;


  /****************************************************************************/
  /* STEP 1                                                                   */
  /*                                                                          */
  /* DESCRIPTION: Extract individual and organization data from csv file.     */
  /* NOTES:                                                                   */
  /* INPUTS(S):   Dow Jones Watchlist - Comma Separated Values (CSV)          */
  /* OUTPUT(S):   stg_wtch.wl_dwjn_extract                                    */
  /****************************************************************************/
  filename file "&dwjn_file" encoding="UTF-16LE";

  data stg_wtch.wl_dwjn_extract(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_dwjn_extract, output_ds=Y));

    attrib
      person_entity_id         informat=best32. format=best32.
      action                   informat=$3.     format=$3.
      gender                   informat=$7.     format=$7.
      deceased                 informat=$3.     format=$3.
      record_status            informat=$8.     format=$8.
      name_type                informat=$255.   format=$255.
      name_sub_type            informat=$255.   format=$255.
      title_honorific          informat=$255.   format=$255.
      firstname                informat=$510.   format=$510.
      middlename               informat=$255.   format=$255.
      surname                  informat=$255.   format=$255.
      suffix                   informat=$255.   format=$255.
      description_1            informat=$255.   format=$255.
      description_2            informat=$255.   format=$255.
      description_3            informat=$255.   format=$255.
      role_type                informat=$255.   format=$255.
      occupation_category      informat=$255.   format=$255.
      title                    informat=$500.   format=$500.
      since_date               informat=$2.     format=$2.
      since_month              informat=$3.     format=$3.
      since_year               informat=$4.     format=$4.
      to_date                  informat=$2.     format=$2.
      to_month                 informat=$3.     format=$3.
      to_year                  informat=$4.     format=$4.
      date_type                informat=$255.   format=$255.
      date                     informat=$2.     format=$2.
      month                    informat=$3.     format=$3.
      year                     informat=$4.     format=$4.
      date_notes               informat=$255.   format=$255.
      place_of_birth           informat=$100.   format=$100.
      sanctions_list_reference informat=$255.   format=$255.
      slr_since_date           informat=$2.     format=$2.
      slr_since_month          informat=$3.     format=$3.
      slr_since_year           informat=$4.     format=$4.
      slr_to_date              informat=$2.     format=$2.
      slr_to_month             informat=$3.     format=$3.
      slr_to_year              informat=$4.     format=$4.
      address_line             informat=$255.   format=$255.
      address_city             informat=$255.   format=$255.
      address_country          informat=$255.   format=$255.
      url                      informat=$255.   format=$255.
      vessel_call_sign         informat=$255.   format=$255.
      vessel_type              informat=$255.   format=$255.
      vessel_tonnage           informat=$255.   format=$255.
      vessel_grt               informat=$255.   format=$255.
      vessel_owner             informat=$255.   format=$255.
      vessel_flag              informat=$255.   format=$255.
      country_type             informat=$255.   format=$255.
      country                  informat=$255.   format=$255.
      identification_type      informat=$255.   format=$255.
      identification_number    informat=$255.   format=$255.
      identification_notes     informat=$255.   format=$255.
      source_name              informat=$255.   format=$255.
      profile_notes            informat=$255.   format=$255.
      images                   informat=$255.   format=$255.
      related_person_entity_id informat=$20.    format=$20.   
      relationship             informat=$255.   format=$255.
      is_ex                    informat=$3.     format=$3.
      addition_date            informat=$11.    format=$11.
      last_reviewed_date       informat=$11.    format=$11.
    ;

    infile file delimiter=',' dsd truncover pad lrecl=40000 end=eof;

    input
         person_entity_id
         action
         gender
         deceased
         record_status
         name_type
         name_sub_type
         title_honorific
         firstname
         middlename
         surname
         suffix
         description_1
         description_2
         description_3
         role_type
         occupation_category
         title
         since_date
         since_month
         since_year
         to_date
         to_month
         to_year
         date_type
         date
         month
         year
         date_notes
         place_of_birth
         sanctions_list_reference
         slr_since_date
         slr_since_month
         slr_since_year
         slr_to_date
         slr_to_month
         slr_to_year
         address_line
         address_city
         address_country
         url
         vessel_call_sign
         vessel_type
         vessel_tonnage
         vessel_grt
         vessel_owner
         vessel_flag
         country_type
         country
         identification_type
         identification_number
         identification_notes
         source_name
         profile_notes
         images
         related_person_entity_id
         relationship
         is_ex
         addition_date
         last_reviewed_date
    ;
   
    if upcase("&allow_invalid") eq "N" then
       if _error_ ne 0 then do;
          call symputx('trans_rc', 999999);
          stop;
       end;

    if eof then
       call symputx('num_recs', _N_ );

   run;


  %MACRO_END:

    %if &syserr eq 1012 or &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let trans_rc=5;
      %fcf_execute_macro(fcf_error);
   %end;
   %else %do;
      %fcf_execute_macro(post&_fcfmodule_);
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_wl_dwjn_extract;
