/*==============================================================================
| NAME:    fcf_wl_eufs_extract
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

WATCH LIST PROVIDER - European Union, Common Foreign and Security Policy (CFSP)
WATCH LIST NAME     - Consolidated list of persons, groups and entities subject
                      to EU financial sanctions
WATCH LIST CODE     - EUFS
WEB LINK            - http://ec.europa.eu/external_relations/cfsp/sanctions/list/consol-list.htm
                    - http://eeas.europa.eu/cfsp/sanctions/consol-list_en.htm

MODULE TYPE         - EXTRACT
MODULE PURPOSE      - Read extract file(s) and output to SAS data set(s).

MODULE NOTES
------------
None

==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_wl_eufs_extract;

  /*------------
   * Set module
   *------------*/
  %let _fcfmodule_=fcf_WL_EUFS_EXTRACT;

  /*---------------------
   * Call pre macro stub
   *---------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*------------------------------------------
   * Set error code to 0 and message to blank
   *------------------------------------------*/
  %let trans_rc =0;
  %let _fcfmsg_=;

  /* Make sure SAS is set for UTF-8 encoding     */
  /* Watch list data file contains characters in UTF-8 encoding */
  data _null_;
    CHK_ENCODING=getoption('ENCODING');
    if CHK_ENCODING ^=: 'UTF' then do;
       call symputx('trans_rc', 999999);
    end;
   run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_= ERROR: FCF: Encoding value is not set properly for processing this data file.;
     %fcf_put_error;
     %let _fcfmsg_= ERROR: FCF: Eurpoean Union watchlist data requires SAS to use UTF-8 encoding.;
     %put &_fcfmsg_;
     %let _fcfmsg_= ERROR: FCF: Restart SAS using the correct encoding options to import the watch list data.;
     %put &_fcfmsg_;
     /* Reset message for e-mail subject */
     %let _fcfmsg_= ERROR: FCF: Eurpoean Union watchlist data requires SAS to use UTF-8 encoding.;
     %goto MACRO_END;
  %end;
   
  /******************************************************************************/
  /* STEP 1                                                                     */
  /*                                                                            */
  /* DESCRIPTION:                                                               */
  /*                                                                            */
  /* Extract entity, name, address, birth, citizen and passport data from xml   */
  /* file.                                                                      */
  /*                                                                            */
  /* NOTES:                                                                     */
  /*                                                                            */
  /*                                                                            */
  /* INPUTS(S);                                                                 */
  /*                                                                            */
  /* global.xml - EUFS current consolidated list (xml)                          */
  /*                                                                            */
  /* OUTPUT(S):                                                                 */
  /*                                                                            */
  /* stg_wtch.wl_eufs_extract_entity   - Entity data                            */
  /* stg_wtch.wl_eufs_extract_name     - Name data                              */
  /* stg_wtch.wl_eufs_extract_address  - Address data                           */
  /* stg_wtch.wl_eufs_extract_birth    - Birth data                             */
  /* stg_wtch.wl_eufs_extract_citizen  - Citizen data                           */
  /* stg_wtch.wl_eufs_extract_passport - Passport data                          */
  /*                                                                            */
  /******************************************************************************/
  filename in_eufs "&eufs_file";
  filename eufs_map "&eufs_map";
  libname  in_eufs xml xmlmap=eufs_map access=readonly;

  data stg_wtch.wl_eufs_extract_entity(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_entity, output_ds=Y));
   set in_eufs.entity;
   run;

  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: &_fcfmodule_: Unable to extract the watch list file into FCF data sets.;
     %put &_fcfmsg_;
     %goto MACRO_END;
  %end;

  data stg_wtch.wl_eufs_extract_name(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_name, output_ds=Y));
   set in_eufs.name;
   run;

  data stg_wtch.wl_eufs_extract_address(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_address, output_ds=Y));
   set in_eufs.address;
   run;

  data stg_wtch.wl_eufs_extract_birth(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_birth, output_ds=Y));
   set in_eufs.birth;
   run;

  data stg_wtch.wl_eufs_extract_citizen(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_citizen, output_ds=Y));
   set in_eufs.citizen;
   run;

  data stg_wtch.wl_eufs_extract_passport(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_eufs_extract_passport, output_ds=Y));
   set in_eufs.passport;
   run;


  %MACRO_END:
   
   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(post&_fcfmodule_);
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_wl_eufs_extract;
