/*==============================================================================
| NAME:    fcf_wl_wchk_transform
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

WATCH LIST PROVIDER - World-Check
WATCH LIST NAME     - Data File
WATCH LIST CODE     - WCHK
WEB LINK            - https://www.world-check.com/login/

MODULE TYPE         - TRANSFORM
MODULE PURPOSE      - Transform data in prepartion for data load.

MODULE NOTES
------------
None

==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2017 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_wl_wchk_transform;

  /*------------
   * Set module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=fcf_WL_WCHK_TRANSFORM;

  /*---------------------
   * Call pre macro stub
   *---------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*------------------------------------------
   * Set error code to 0 and message to blank
   *------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;


/* BEGIN MACRO DESCRIPTOR
+-------------+----------------------------------------------------------------+
|MACRO:       | fcf_wl_wchk_transform_locations
|             |
|PURPOSE:     | The purpose of this macro is to process the LOCATIONS
|             | field located in the World-Check data file. The LOCATIONS
|             | field format is multiple entry. As such, multiple locations
|             | may exist and each distinct location is required to be
|             | processed and output as a distinct record. The secondary
|             | purpose of this macro is to output completed records
|             | to STG_WTCH.wl_wchk_entity_transform table.
|             |
|PARAMETERS:  | None
|             |
|USAGE:       | %fcf_wl_wchk_transform_locations
+-------------+----------------------------------------------------------------+
|NOTES:       | Output fields parsed and output by this macro:
|             | - address
|             | - city_name
|             | - state_name
|             | - country_name
|             | - full_address
|             |
|             | NOTE: The World-Check format for single entry in the
|             | multiple entry LOCATIONS field is: Freetext~CITY,STATE~COUNTRY.
|             | Any or all tokens within a single entry could be missing.
|             | This program will validate the format of a single non-blank
|             | (at least one token is available) entry. If the entry is found
|             | to violate the published format (i.e. missing/incorrect number
|             | delimiters) then an error record is output and no further
|             | processing is performed. This program also checks for digits
|             | within location_city, location_state, location_country_name.
|             | If at least one digit is located an error record is ouput.
|             | For these cases, however, processing will not terminate.
+-------------+--------+------------+------------------------------------------+
END MACRO DESCRIPTOR */

%macro fcf_wl_wchk_transform_locations;

/* LOCATIONS is not blank - proceed */
if kcompress(locations,'~,;') ne '' then do;
   num_locations=countc(locations,';') + 1;
   do j=1 to num_locations;
      location_text=kscan(locations,j,';');
      /* location_text is not blank - proceed */
      if kcompress(location_text,'~,') ne '' then do;
         /* ERROR: entry does not contain exactly 2 '~' delimiters */
         if countc(location_text,'~') ne 2 then do;

            error_file="&wchk_file";
            error_record_number=_N_;
            error_field='LOCATIONS';
            error_value=kstrip(location_text);
            error_message="WARNING: Incorrect field format. Field requires 2 '~' delimiters.";
            error_datetime=datetime();

            putlog "WARNING: FCF: Incorrect field format Field requires 2 '~' delimiters. "
            entity_watch_list_number= locations=;
       
            error_flag = 1;

            output STG_ERR.wl_wchk_errors;
         end;
         else do;
            /* location_text format = ~ location_city, state ~ country */
            /* no Freetext (address) at beginning */
            if kindex(kstrip(location_text),'~') eq 1 then do;
               /* ERROR: entry does not contain exactly 1 ',' delimiter */
               if countc(kscan(location_text,1,'~'),',') ne 1 then do;

                  error_file="&wchk_file";
                  error_record_number=_N_;
                  error_field='LOCATIONS';
                  error_value=kstrip(location_text);
                  error_message="WARNING: Incorrect field format. Field requires 1 ',' delimiter.";
                  error_datetime=datetime();

                  putlog "WARNING: FCF: Incorrect field format. Field requires 1 ',' delimiter. "
                  entity_watch_list_number= locations=;
       
                  error_flag = 1;

                  output STG_ERR.wl_wchk_errors;
               end;
               else do;
                  /* set location_address value=blank due to missing freetext */
                  address="";
                  /* city_name */
                  city_name=upcase(compbl(kscan(kscan(location_text,1,'~'),1,',')));
                  /* state_name */
                  state_name=upcase(compbl(kscan(kscan(location_text,1,'~'),2,',')));
               end;
            end; /* end locations format = ~location_city,state,country */

            /* location_text format = freetext~location_city,state~country */
            else if kindex(kstrip(location_text),'~') ne 1 then do;
               /* ERROR: entry does not contain exactly 1 ',' delimiter */
               if countc(kscan(location_text,2,'~'),',') ne 1 then do;

                  error_file="&wchk_file";
                  error_record_number=_N_;
                  error_field='LOCATIONS';
                  error_value=kstrip(location_text);
                  error_message="WARNING: Incorrect field format. Field requires 1 ',' delimiter.";
                  error_datetime=datetime();

                  putlog "WARNING: FCF: Incorrect field format. Field requires 1 ',' delimiter. "
                  entity_watch_list_number= location_text=;
       
                  error_flag = 1;

                  output STG_ERR.wl_wchk_errors;
               end;
               else do;
                  /* address */
                  address=upcase(compbl(kscan(location_text,1,'~')));
                  /* city_name */
                  city_name=upcase(compbl(kscan(kscan(location_text,2,'~'),1,',')));
                  /* state_name */
                  state_name=upcase(compbl(kscan(kscan(location_text,2,'~'),2,',')));
               end;
            end; /* end locations format=freetext~location_city,state,country */

            /* country_name */

            /* missing country_name */
            if kindex(kreverse(kstrip(location_text)),'~') eq 1 then
              country_name='';
            else do;
              country_name=upcase(kstrip(compbl(kscan(location_text,-1,'~'))));
            end;
            
            /* full_address */
            full_address=catx(';',address,city_name,state_name,postal_code,country_name);

            if not missing(error_field) then
               output STG_WTCH.wl_wchk_entity_transform
                      STG_ERR.wl_wchk_errors;
            else
               output STG_WTCH.wl_wchk_entity_transform;

         end; /* end no format issues */
      end; /* end location_text is not blank */
   end; /* end do loop */
end; /* end location is not blank */
else output STG_WTCH.wl_wchk_entity_transform;

%mend fcf_wl_wchk_transform_locations;

/*--------------------------------------
*  get column names for watch_list_dim
*--------------------------------------*/

%fcf_get_column_names(table=seg_kc.fsc_entity_watch_list_dim);

/******************************************************************************/
/* BEGIN:                                                                     */
/*        STG_WTCH.wl_wchk_entity_transform                                   */
/*        STG_WTCH.wl_wchk_location_transform                                 */
/*        STG_ERROR.wl_wchk_errors                                            */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*        Output final transformation and error table(s).                     */
/******************************************************************************/
data

STG_WTCH.wl_wchk_entity_transform(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_wchk_entity_transform, output_ds=Y)
                                  keep=
   entity_watch_list_key
   entity_watch_list_number
   watch_list_name
   category_desc
   type_desc
   programs
   tax_id
   tax_id_type_code
   identification_id
   identification_type_desc
   date_of_birth
   year_of_birth
   place_of_birth
   deceased_ind
   first_name
   last_name
   middle_name
   entity_name
   match_code_individual
   match_code_organization
   entity_title
   occupation_desc
   address
   city_name
   state_name
   postal_code
   country_code
   country_name
   full_address
   match_code_full_address
   citizenship_country_code
   citizenship_country_name
   match_code_citizenship
   nationality_country_code
   nationality_country_name
   match_code_nationality
   org_country_of_business_code
   org_country_of_business_name
   match_code_org_country
   politically_exposed_person_ind
   create_date
   update_date
   exclude_ind
   change_begin_date
   change_end_date
   change_current_ind
)

STG_WTCH.wl_wchk_location_transform(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_wchk_location_transform, output_ds=Y)
                                    keep=
   location_watch_list_key
   location_watch_list_number
   watch_list_name
   program
   country_code
   country_name
   match_code_country_name
   create_date
   update_date
   exclude_ind
   change_begin_date
   change_end_date
   change_current_ind
)

STG_ERR.wl_wchk_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_wchk_errors, output_ds=Y)
                       keep=
   entity_watch_list_number
   watch_list_name
   error_file
   error_record_number
   error_field
   error_value
   error_message
   error_datetime
);


   attrib
      /* ERROR FIELDS */
      error_file          length=$200
                          informat=$200.
                          format=$200.
                          label='Error File'

      error_record_number length=8
                          informat=best32.
                          format=best32.
                          label='Error Record Number'

      error_field         length=$32
                          informat=$32.
                          format=$32.
                          label='Error Field'

      error_value         length=$1000
                          informat=$1000.
                          format=$1000.
                          label='Error Value'

      error_message       length=$200
                          informat=$200.
                          format=$200.
                          label='Error Message'

      error_datetime      length=8
                          informat=nldatm.
                          format=nldatm.
                          label='Error Datetime'

      /* TEMP FIELDS */
      location_text                  length=$32767
      aliases_text                   length=$32767
      alternative_spelling_text      length=$32767
   ;

/******************************************************************************/
/* NOTE:                                                                      */
/*                                                                            */
/* The following fields are must be retained to avoid being set to null via   */
/* do loop processing.                                                        */
/******************************************************************************/
   retain
      entity_watch_list_number
      watch_list_name
      category_desc
      type_desc
      tax_id
      tax_id_type_code
      first_name
      last_name
      middle_name
      entity_name
      date_of_birth
      year_of_birth
      org_country_of_business_name
      politically_exposed_person_ind
      exclude_ind
   ;

   /* Sets variable attribs to match variables in DBMS */
   /* FSC_ENTITY_WATCH_LIST_DIM   */
   /* FSC_LOCATION_WATCH_LIST_DIM */
   if 1 eq 0 then do;
      set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;
      set seg_kc.FSC_LOCATION_WATCH_LIST_DIM;
   end;

   set STG_WTCH.wl_wchk_extract(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_wchk_extract)
                                where=(upcase(category) not contains "VESSEL")
                                rename=(first_name=wchk_first_name
                                        last_name=wchk_last_name
                                        place_of_birth=wchk_place_of_birth))
                                end=eof;

/******************************************************************************/
/* NOTE:                                                                      */
/*                                                                            */
/* The following fields are set to null as this watch list extract does not   */
/* currently include this data.                                               */
/******************************************************************************/
   middle_name='';
   postal_code='';
   citizenship_country_name='';

/******************************************************************************/
/* NOTE:                                                                      */
/*                                                                            */
/* The following fields are set to null to avoid retained values.             */
/******************************************************************************/
   entity_watch_list_number='';
   type_desc='';
   tax_id='';
   tax_id_type_code='';
   date_of_birth=.;
   year_of_birth=.;
   country_code='';
   nationality_country_name='';
   nationality_country_code='';
   org_country_of_business_name='';
   org_country_of_business_code='';
   politically_exposed_person_ind='';


   /* ENTITY_WATCH_LIST_NUMBER */
   entity_watch_list_number=kstrip(put(uid,best32.));

   /* WATCH_LIST_NAME*/
   watch_list_name='WCHK';

   /* CATEGORY_DESC */
   category_desc=upcase(compbl(kstrip(category)));

   /* PROGRAM */
      programs=upcase(compbl(kstrip(keywords)));

   /* POLITICALLY_EXPOSED_PERSON_IND */
   if kindex(kstrip(sub_category),'PEP') > 0 then 
      politically_exposed_person_ind='Y';
   else
      politically_exposed_person_ind='N';

   /* CREATE_DATE */
   if not missing(entered) then
      create_date=dhms(input(entered,yymmdd10.),00,00,00);

   /* UPDATE_DATE */
   if not missing(updated) then
      update_date=dhms(input(updated,yymmdd10.),00,00,00);

   /* EXCLUDE_IND */
   exclude_ind='N';

   /***************************************************************************/
   /* BEGIN: Process STG_WTCH.wl_wchk_location_transform                      */
   /***************************************************************************/

   /* Identify blacklisted and embargoed countries */
   if upcase(e_i) eq 'E' and
      upcase(category) in ('COUNTRY','EMBARGO') and
      kindex (keywords , 'WORLD BANK' ) = 0 and
      kindex (keywords , 'EUAC' ) = 0 and
      kindex (further_information , 'EUAC' ) = 0  
      then do;

      /* ERROR: Country name is missing */
      if missing(countries) then do;

         error_file="&wchk_file";
         error_record_number=_N_;
         error_field='COUNTRIES';
         error_value=kstrip(countries);
         error_message='WARNING: Country name is missing.';
         error_datetime=datetime();

         putlog 'WARNING: FCF: Country name is missing. ' entity_watch_list_number=;
         error_flag = 1;
         output STG_ERR.wl_wchk_errors;
      end;

      /* ERROR: Country name is unknown */
      /* NOTE: World-Check codes unknown countries as UNKNOWN */
      else if not missing(countries) then do;
         if upcase(compbl(kstrip(countries))) eq 'UNKNOWN' then do;

            error_file="&wchk_file";
            error_record_number=_N_;
            error_field='COUNTRIES';
            error_value=kstrip(countries);
            error_message='WARNING: Country name is unknown.';
            error_datetime=datetime();

            putlog 'WARNING: FCF: Country name is unknown. ' entity_watch_list_number= countries=;
            error_flag = 1;

            output STG_ERR.wl_wchk_errors;
         end;

        /* Country name is not missing */
         else do;

            /* LOCATION_WATCH_LIST_NUMBER */
            location_watch_list_number=kstrip(put(uid,best32.));

            /* COUNTRY_NAME */
            country_name=upcase(compbl(kstrip(countries)));

            if kcompress(keywords,'~') eq '' then
               output STG_WTCH.wl_wchk_location_transform;
            else do;
               num_list=countc(keywords,'~','to') + 1;
               do i=1 to num_list;

                  /* PROGRAM */
                  program=upcase(kstrip(kscan(keywords,i,'~')));
                  if not missing(program) then
                     output STG_WTCH.wl_wchk_location_transform;
               end; /* end do-loop */
               return;
            end; /* end non-blank keyords */
         end; /* end countries ne UNKNOWN */
      end; /* end non-blank countries */
   end; /* end blacklisted/embargoed country */

   /***************************************************************************/
   /* END: Process STG_WTCH.wl_wchk_location_transform                        */
   /***************************************************************************/

   /***************************************************************************/
   /* BEGIN: Process STG_WTCH.wl_wchk_entity_transform                        */
   /***************************************************************************/

   /* BEGIN: type_desc=INDIVIDUAL */

   /* I = Individual - Gender Unknown - Profile not reviewed since introduction
          of new character set
      U = Unknown - Gender Unknown - Profile reviewed since introduction of new
          character set
      M = Male - Individual - Gender Known - Positive ID
      F = Female - Individual - Gender Known - Positive ID                    */

   if upcase(e_i) in ('I','F','M','U') then do;

/******************************************************************************/
/* NOTE: The following fields are not applicable where                        */
/*       type_desc=INDIVIDUAL. These fields are therefore set to null.        */
/******************************************************************************/
      org_country_of_business_name='';

      /* TYPE_DESC */
      type_desc='INDIVIDUAL';

      /* PROGRAM */
      programs=upcase(compbl(kstrip(keywords)));

      if not missing(ssn) then do;

         /* TAX_ID */
         tax_id=kstrip(ssn);

         /* TAX_ID_TYPE_CODE */
         tax_id_type_code='S';

      end;

/******************************************************************************/
/* NOTE:                                                                      */
/*                                                                            */
/* Additional transformations may be required in those cases where more than  */
/* one passport number exists within the PASSPORT field.                      */
/**************************************************************************** */
      if not missing(passports) then do;

         /* IDENTIFICATION_ID */
         identification_id=kscan(passports,1,'(');

         /* IDENTIFICATION_TYPE_DESC */
         identification_type_desc='PASSPORT';

      end;

      /* FIRST_NAME */
      first_name=upcase(compbl(kstrip(wchk_first_name)));

      /* LAST_NAME */
      last_name=upcase(compbl(kstrip(wchk_last_name)));

      /* NAME */
      entity_name=upcase(compbl(catx(' ',first_name,last_name)));

      /* ENTITY_TITLE */
      entity_title=upcase(compbl(kstrip(title)));

      /* POSITION */
      occupation_desc=upcase(compbl(kstrip(position)));

      /* DATE_OF_BIRTH
         YEAR_OF_BIRTH */
/******************************************************************************/
/* NOTE:                                                                      */
/*                                                                            */
/* World-Check DOB field is formatted as YYYY/MM/DD.                          */
/* Values formatted as YYYY/00/00 indicate missing month/day.                 */
/******************************************************************************/
      if missing(dob) then do;
         date_of_birth=.;
         year_of_birth=.;
      end;
      else if not missing(dob) then do;

         /* Valid date format YYYY/MM/DD */
         if lengthn(kstrip(kcompress(dob,'/'))) eq 8 and
            not anyalpha(kcompress(dob,'/')) then do;
            /* Only year_of_birth available */
            if index(dob,'/00') then do;
               date_of_birth=.;
               year_of_birth=input(kscan(dob,1,'/'),best32.);
            end;
            /* date_of_birth and year_of_birth available */
            else if not index(dob,'/00') then do;

               date_of_birth=input(dob,??yymmdd10.);

               if not missing(date_of_birth) then do;
                  date_of_birth=dhms(date_of_birth,00,00,00);
                  year_of_birth=year(datepart(date_of_birth));
               end;

               /* WARNING: Invalid date of birth. */
               else if missing(date_of_birth) then do;
                  year_of_birth=.;

                  error_file="&wchk_file";
                  error_record_number=_N_;
                  error_field='DOB';
                  error_value=kstrip(dob);
                  error_message='WARNING: Invalid date of birth.';
                  error_datetime=datetime();

                  putlog 'WARNING: FCF: Invalid date of birth. ' entity_watch_list_number= dob=;
                  error_flag = 1;
               end;
            end;
         end;

         /* WARNING: Invalid date of birth. */
         else if lengthn(kstrip(kcompress(dob,'/'))) ne 8 or
            anyalpha(kcompress(dob,'/')) then do;
            date_of_birth=.;
            year_of_birth=.;

            error_file="&wchk_file";
            error_record_number=_N_;
            error_field='DOB';
            error_value=kstrip(dob);
            error_message='WARNING: Invalid date of birth.';
            error_datetime=datetime();

            putlog 'WARNING: FCF: Invalid date of birth. ' entity_watch_list_number= dob=;
            error_flag = 1;

         end; /* end: Invalid DD/MM/YY format */
      end; /* end: Non-missing date of birth */

      /* PLACE_OF_BIRTH */
      place_of_birth=upcase(compbl(kstrip(wchk_place_of_birth)));

      /* DECEASED_IND */
      if not missing(deceased) then
         deceased_ind='Y';

      /* NATIONALITY_COUNTRY_NAME */
      if upcase(countries) not in(' ','UNKNOWN') then do;
         nationality_country_name=upcase(compbl(kstrip(countries)));
      end;

   end; /* end: type_desc=INDIVIDUAL */

   /* BEGIN: type_desc=ORGANIZATION */
   else if upcase(e_i) eq 'E' then do;

/******************************************************************************/
/* NOTE:                                                                      */
/*                                                                            */
/* The following fields are not applicable where type_desc='ORGANIZATION'.    */
/* These fields are therefore set to null.                                    */
/******************************************************************************/
      first_name='';
      last_name='';
      entity_title='';
      postion='';
      date_of_birth=.;
      year_of_birth=.;
      place_of_birth='';
      deceased_ind='';

      /* TYPE_DESC */
      type_desc='ORGANIZATION';
      if ssn ne '' then do;

         /* TAX_ID */
         tax_id=kstrip(ssn);

         /* TAX_ID_TYPE_CODE */
         tax_id_type_code='E';

      end;

      /* NAME */
/******************************************************************************/
/* NOTE:                                                                      */
/*                                                                            */
/* Where type_desc='ORGANIZATION', the organization name appears in the       */
/* LAST_NAME field.                                                           */
/******************************************************************************/
      entity_name=upcase(compbl(kstrip(wchk_last_name)));

      /* ORG_COUNTRY_OF_BUSINESS_NAME */
      if upcase(countries) not in(' ','UNKNOWN') then do;
        org_country_of_business_name=upcase(compbl(kstrip(countries)));
      end;

   end; /* end ORGANIZATION */


   /* process LOCATIONS field */
   %fcf_wl_wchk_transform_locations;

   /*** process ALIASES field ***/

   /* mulitiple entry field is not blank - process */
   if kcompress(aliases,',;') ne '' then do;
      num_aliases=countc(aliases,';') + 1;
      do i=1 to num_aliases;
         aliases_text=kscan(aliases,i,';');
         /* parsed multiple entry field is not blank - process */
         if kcompress(aliases_text,',') ne '' then do;
            /* process individual */
            if upcase(e_i) in ('I','F','M','U') then do;
               /* one ',' delimiter found - process accordingly */
               if countc(aliases_text,',') eq 1 then do;
                  if kindex(kstrip(aliases_text),',') eq 1 then do;
                     first_name=upcase(compbl(kscan(aliases_text,1,',')));
                     last_name='';
                  end;
                  else do;
                     first_name=upcase(compbl(kscan(aliases_text,2,',')));
                     last_name=upcase(compbl(kscan(aliases_text,1,',')));
                  end;
               end;
               /* zero ',' delimiter found - process accordingly */
               else if countc(aliases_text,',') eq 0 then do;
                  /* World-Check last names are in all upper case.  */
                  /* if we find no lower case then assume last name */
                  if anylower(aliases_text) eq 0 then do;
                     first_name='';
                     last_name=upcase(compbl(aliases_text));
                  end;
                  else do;
                     /* found lower case so assume first name */
                     first_name=upcase(compbl(aliases_text));
                     last_name='';
                  end;
               end;

               /* ERROR: Entry contains multiple ',' delimiters. */
               else do;

                  error_file="&wchk_file";
                  error_record_number=_N_;
                  error_field='ALIASES';
                  error_value=kstrip(aliases_text);
                  error_message="WARNING: Incorrect field format. More than 1 ',' delimiter found.";
                  error_datetime=datetime();

                  putlog "WARNING: FCF: Incorrect field format. More than 1 ',' delimiter found."
                         entity_watch_list_number= aliases_text=;
                  error_flag = 1;

                  output STG_ERR.WL_WCHK_ERRORS;
               end;
               middle_name='';
               entity_name=upcase(compbl(catx(' ',first_name,last_name)));
            end;
            /* process organization */
            else if upcase(e_i) eq 'E' then do;
               first_name='';
               last_name='';
               middle_name='';
               entity_name=upcase(compbl(kstrip(aliases_text)));
            end;
            
            /* Process LOCATIONS */
            %fcf_wl_wchk_transform_locations;
            
         end;
      end;
   end;

   /*** Process ALTERNATIVE_SPELLING field ***/

   /* mulitiple entry field is not blank - process */
   if kcompress(alternative_spelling,',;') ne '' then do;
      num_alternative_spelling=countc(alternative_spelling,';') + 1;
      do i=1 to num_alternative_spelling;
         alternative_spelling_text=kscan(alternative_spelling,i,';');
         /* parsed multiple entry field is not blank - process */
         if kcompress(alternative_spelling_text,',') ne '' then do;
            /* process individual */
            if upcase(e_i) in ('I','F','M','U') then do;
               /* one ',' delimiter found - process accordingly */
               if countc(alternative_spelling_text,',') eq 1 then do;
                  if kindex(kstrip(alternative_spelling_text),',') eq 1 then do;
                     first_name=upcase(compbl(kscan(alternative_spelling_text,1,',')));
                     last_name='';
                  end;
                  else do;
                     first_name=upcase(compbl(kscan(alternative_spelling_text,2,',')));
                     last_name=upcase(compbl(kscan(alternative_spelling_text,1,',')));
                  end;
               end;
               /* zero ',' delimiter found - process accordingly */
               else if countc(alternative_spelling_text,',') eq 0 then do;
                  /* World-Check last names are in all upper case.  */
                  /* if we find no lower case then assume last name */
                  if anylower(alternative_spelling_text) eq 0 then do;
                     first_name='';
                     last_name=upcase(compbl(alternative_spelling_text));
                  end;
                  else do;
                     /* found lower case so assume first name */
                     first_name=upcase(compbl(alternative_spelling_text));
                     last_name='';
                  end;
               end;

               /* ERROR: Entry contains multiple ',' delimiters. */
               else do;

                  error_file="&wchk_file";
                  error_record_number=_N_;
                  error_field='ALTERNATIVE_SPELLING';
                  error_value=kstrip(alternative_spelling_text);
                  error_message="WARNING: Incorrect field format. More than 1 ',' delimiter found.";
                  error_datetime=datetime();

                  putlog "WARNING: FCF: Incorrect field format. More than 1 ',' delimiter found. "
                         entity_watch_list_number= alternative_spelling_text=;
                  error_flag = 1;

                  output STG_ERR.WL_WCHK_ERRORS;
               end;
               middle_name='';
               entity_name=upcase(compbl(catx(' ',first_name,last_name)));
            end;
            /* process organization */
            else if upcase(e_i) eq 'E' then do;
               first_name='';
               last_name='';
               middle_name='';
               entity_name=upcase(compbl(kstrip(alternative_spelling_text)));
             end;
            /* process LOCATIONS */
            %fcf_wl_wchk_transform_locations;
         end;
      end;
   end;
   
   /*  Set all columns in if set 1=0 statement to missing */
   call missing(&col_name_list);

   if (eof) and (error_flag) then do;
      putlog ' ';
      putlog 'WARNING: FCF: Invalid values were encountered processing the watch list data';
      putlog 'WARNING: FCF: Data values were written to the STG_ERR.WL_WCHK_ERRORS table.';
   end;
run;


/******************************************************************************/
/*                                                                            */
/* DESCRIPTION:                                                               */
/* Remove duplicate records.                                                  */
/* Duplicates can occur due to aliases or alternative spelling of the         */
/* entity_name being in different case                                        */
/*                                                                            */
/******************************************************************************/
proc sort data=stg_wtch.wl_wchk_entity_transform(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_wchk_entity_transform))
          out=stg_wtch.wl_wchk_entity_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_wchk_entity_transform_01, output_ds=Y))
          nodupkey;
  by
     entity_watch_list_number
     entity_name
     tax_id
     full_address
     date_of_birth
     year_of_birth
  ;
run;

proc sort data=stg_wtch.wl_wchk_location_transform(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_wchk_location_transform))
          out=stg_wtch.wl_wchk_location_transform(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_wchk_location_transform, output_ds=Y))
          nodupkey;
  by
     location_watch_list_number
     program
     country_name
  ;
run;


/******************************************************************************/
/*                                                                            */
/* DESCRIPTION:                                                               */
/* Process multiple place of births                                           */
/*                                                                            */
/******************************************************************************/
data stg_wtch.wl_wchk_entity_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_wchk_entity_transform_02, output_ds=Y));
    set stg_wtch.wl_wchk_entity_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_wchk_entity_transform_01));
    length orig_place_of_birth $400 place_of_birth_text $100;
    orig_place_of_birth=place_of_birth;
      num_place_of_birth=countc(orig_place_of_birth,';') + 1;
      do i=1 to num_place_of_birth;
         place_of_birth_text=kscan(orig_place_of_birth,i,';');
         place_of_birth_text=kstrip(tranwrd(place_of_birth_text,'ALT.POB',''));
         place_of_birth_text=kstrip(tranwrd(place_of_birth_text,'ALT. POB',''));
         place_of_birth_text=kstrip(tranwrd(place_of_birth_text,'ALT POB',''));
         place_of_birth=place_of_birth_text;
         output;
      end;
    
    drop orig_place_of_birth num_place_of_birth place_of_birth_text i;
run;

 /****************************************************************************/
  /* DESCRIPTION:                                                             */
  /* Process sanctions_list_reference.                                        */
  /*                                                                          */
  /* NOTES:                                                                   */
  /* Each entity can have multiple program values.                            */
  /* Some entities are in so many lists that concatenating the values of the  */
  /* lists exceed the length of programs in the database, the following rules */
  /* are applied in building the programs list:                               */
  /* 1)  The programs field is checked agains the most important              */
  /*     lists based on current customer base.                                */
  /*          Text Checked             Special List Value                     */
  /*          ------------             ------------------                     */
  /*          OFAC                     OFAC                                   */
  /*          DFAT                     DFAT                                   */
  /*          begins with EC           EUFS                                   */
  /*          HM Treasury              HMT                                    */
  /*          New Zealand Police       NCBI                                   */
  /*          UN Security Council      UNSC                                   */
  /* 2) If the entity is one or more special lists, the Special List Values   */
  /*    are included in the programs field.   If the entity is also on        */
  /*    other lists, the count of other lists is included in the programs     */
  /* 3) If the entity is not on any special list, the programs field contains */
  /*    a concatentated list of all the reference lists providing the number  */
  /*    of distinct lists is <= 5 and the length of the concatenated list     */
  /*    is less than 250.                                                     */
  /* 4) If the number of lists constraint or the length constraint is violated*/
  /*    only a count is provided.                                             */
  /****************************************************************************/
data stg_wtch.wl_wchk_entity_transform(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_wchk_entity_transform, output_ds=Y));
    set stg_wtch.wl_wchk_entity_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_wchk_entity_transform_02));
    length orig_programs $1000 programs $1000 current_program $10 special_programs $1000;
    orig_programs=programs;
    
    /* Check Special Programs */
    special_programs='';
    special_prog_count=0;

    if kindex(orig_programs,"OFAC") > 0 then do;
       special_prog_count+1;
       special_programs="OFAC";
    end;
    if kindex(orig_programs,"DFAT") > 0 then do;
       special_prog_count+1;
       special_programs=kstrip(special_programs)||";DFAT ";
    end;
    if ksubstr(kleft(orig_programs), 1, 2)="EU" then do;
       special_prog_count+1;
       special_programs=kstrip(special_programs)||";EUFS ";
    end;
    if kindex(orig_programs,"UKHMT") > 0 then do;
       special_prog_count+1;
       special_programs=kstrip(special_programs)||";UKHMT ";
    end;
    if kindex(orig_programs,"NZ-MFAT") > 0 then do;
       special_prog_count+1;
       special_programs=kstrip(special_programs)||";NCBI ";
    end;
    else if substr(orig_programs,1,2)="UN" then do;
       special_prog_count+1;
       special_programs=kstrip(special_programs)||";UNSC ";
    end;
    else if kindex(orig_programs,"OSFI") > 0 then do;
       special_prog_count+1;
       special_programs=kstrip(special_programs)||";OSFI ";
    end;

    num_programs=countc(orig_programs,'~') + 1;
    if num_programs gt 1 then do;
       programs='';
       do i=1 to num_programs;
          current_program=kscan(orig_programs,i,'~');
          if missing(programs) then programs=compbl(kstrip(current_program));
              else programs=kstrip(programs) || ';' || compbl(kstrip(current_program));
        end; /* do loop */
      end;

      if special_prog_count then do;
           if num_programs > 5  then do;
              programs=special_programs;
              if (num_programs - special_prog_count) > 0 then 
                  programs=kstrip(special_programs)||";"||kstrip(put(num_programs - special_prog_count,best32.))||" additional lists";
           end;
      end;
      else if num_programs > 5 or length(programs) > 250 then programs="Entity on "||kstrip(put(num_programs, best32.))||" sanctions lists";
      drop current_program orig_programs I num_programs special_prog_count special_programs ;
run;

/* Ensure error file is de-dupped */
proc sort data=STG_ERR.wl_wchk_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_wchk_errors))
          out=STG_ERR.wl_wchk_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_wchk_errors, output_ds=Y))
          nodupkey;
  by
    entity_watch_list_number
    watch_list_name
    error_file
    error_record_number
    error_field
    error_value
    error_message
    error_datetime
    ;
run;

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
     %fcf_execute_macro(fcf_error);
  %else %do;
     %fcf_execute_macro(post&_fcfmodule_);
     %fcf_execute_macro(fcf_success);
  %end;

%mend fcf_wl_wchk_transform;
