/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_efile_fincen_detail.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    Transform the report data for e-filing (detail record level)
|          
+------------------------------------------------------------------------------+
| USAGE:   form_config_key = form key
|          form_type_code  = form type code (defined in FSK_RC_FORM_CONFIG)
|          report_list     = report key list of reports to be processed 
|                            Keys are separated by ' ' 
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

  

%macro rc_efile_fincen_detail(
                              form_config_key=, 
                              form_type_code=,
                              report_list=,
                             );                                        

 
   
    %let trans_rc=0;
    %let _fcfmsg_=;

    %let record_id_cnt=0;
    proc sql noprint;
         select distinct trim(form_efile_record_id) into
                :record_id_1 - 
         from rc_cfg.form_efile_record_config(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=form_efile_record_config))
         where form_config_key=&form_config_key
         ;
         %let record_id_cnt=&sqlobs;
        
    quit;
    
     /* remove existing tables */
     proc datasets lib=work nolist nowarn;
         
     %do irecord=1 %to &record_id_cnt; 
         %if %sysfunc(exist(&form_type_code._&&&record_id_&irecord)) %then
        %do;
            delete &form_type_code._&&&record_id_&irecord;
        %end; 
     %end;
     run;
     quit;
    /*---------------------------------------------
     * Transpose report node data into records
     *---------------------------------------------*/


   %rc_efile_transpose_data(report_list=%superq(report_list),
                            form_config_key=&form_config_key, 
                            form_type_code=&form_type_code);

  
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end;


    /*---------------------------------------------
     * Assign the file date
     *---------------------------------------------*/
    proc sql;
         update &form_type_code._3A
         set filed_date = today()
     ;
    quit;

     %fcf_rcset(&sqlrc);

      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
           %let _fcfmsg_=ERROR: RC: Unable to update file_date in WORK.SAR_3A;
           %fcf_put_error;
           %goto MACRO_END;
      %end;

      /*---------------------------------------------
     * Assign Prior Agency Reference ID 
     *---------------------------------------------*/
    proc sql;
         
          update &form_type_code._3A as rec_3a
          set prior_agency_ref_id=(select  prior_agency_ref_id    
                                   from seg_kc.fsk_rc_report rpt
                                   where rec_3a.report_key=rpt.report_key)
     where type_filing_code ne 'A'
     ;
    quit;

     %fcf_rcset(&sqlrc);

      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
           %let _fcfmsg_=ERROR: RC: Unable to update PRIOR_AGENCY_REF_ID in WORK.SAR_3A;
           %fcf_put_error;
           %goto MACRO_END;
      %end;

      /* create a empty 4A table if no party exists in the report */
       %if %sysfunc(exist(&form_type_code._4A))=0 and %sysfunc(exist(rc_db.&form_type_code._4A))%then 
        %do;
             data &form_type_code._4A;
                  if 0 then set rc_db.&form_type_code._4A(%fcf_get_ds_opts(lib_name=rc_db, ds_name=&form_type_code._4A));
                  stop; 
             run;
        %end;



    /*--------------------------------------------------------------
     * Run additional SAR only transformation
     *--------------------------------------------------------------*/

    %if &form_type_code=SAR %then
    %do;
        
    /*--------------------------------------------------------------
     * Convert the birth date format from yyyy-mm-dd to mmddyyyy.
     * This date conversion is different from other date fields
     * because of the support of partial date. 
     * Birth dates are entered as string in java date format, not date, in UI.
     * Below is the instruction from FINCEN about partial date
     * If the birth day and or month is unknown, enter �00 for the unknown day or month.  
         *--------------------------------------------------------------*/
    
        

        /*--------------------------------------------------------------
         * Set if entity flag to A when the subject is an entity
         * and make up 4A records as critical info missing if there isn't any subject for the report. 
         *--------------------------------------------------------------*/
        
             proc sql;
                  update sar_4A
                      set birth_date=substrn(birth_date,6,2)||
                                     substrn(birth_date,9,2)||
                                     substrn(birth_date,1,4)
                      where birth_date > ''
                      ;
                  update sar_4A
                      set entity_flg='A'
                      where entity_flg='X'
              ;
          
        /*--------------------------------------------------------------
         * Remove unwanted 4H records 
         *--------------------------------------------------------------*/
        %if %sysfunc(exist(SAR_4H)) %then
        %do;
            proc sql;
                 delete from SAR_4H
                 where subject_relation_type='' 
                       and subject_relation_type_other=''
                       and subject_relation_status_code=''
                       and subject_relation_status_date = .
                ;
           quit;
        %end; 
             
 
       
      

         %fcf_rcset(&sqlrc);

         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=ERROR: RC: Unable to remove null relation from WORK.SAR_4H;
              %fcf_put_error;
              %goto MACRO_END;
          %end;
  

        /*--------------------------------------------------------------
         *  Break up long narrative column into      
         *  $850 strings                            
         *  up to 20 rows are reported  (5A)         
         *--------------------------------------------------------------*/
         

        %rc_efile_fincen_narrative  (report_list=%superq(report_list),
                                     form_type_code=&form_type_code,
                                     outsize=850,
                                     outseqmax=20
                                    );
         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %goto MACRO_END;
         %end;

 %end;

 
    /*--------------------------------------------------------------
     * Run additional CTR only transformation
     *--------------------------------------------------------------*/

    %if &form_type_code=CTR %then
    %do;

        /* Change the total amount to missing when it is 0 */
        
        proc sql;
             update CTR_3A
             set us_cash_in_amount=.
             where us_cash_in_amount =0;
             update CTR_3A
             set us_cash_out_amount=.
             where us_cash_out_amount =0;
        
        quit;
         %fcf_rcset(&sqlrc);

         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=ERROR: RC: Unable to set 0 amount to missing in  WORK.CTR_3A;
              %fcf_put_error;
              %goto MACRO_END;
          %end;

    %end;



%MACRO_END:
  

    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
       %fcf_execute_macro(fcf_error);
    %else 
       %fcf_execute_macro(fcf_success);
    


%mend;
 
