/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_efile_fincen_sum.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    This macro
|          1. determines the number of e-files are needed to file all the reports.
|          2. assigns efile key and sequence numbers to the report and the associated entities
|             such as branch and parties. 
|          3. pull the transmitter information and derive coverage period
|          4. pull the institution and branch details
|          5. create all the counts for 9<x> records. 
|          
+------------------------------------------------------------------------------+
| USAGE:   form_config_key = form key 
|          form_type_code  = form type code (defined in FSK_RC_FORM_CONFIG)
|
| HISTORY: For 7.1HF3, the usage of rc_cfg.form_default is removed. Also,
|          This macro is changed to support different institution for 2A and 2B 
|          in SAR and different transmitter for different form. 
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro rc_efile_fincen_sum(form_config_key=,
                           form_type_code=,
                           efile_key=
                           );

     
    %let trans_rc=0;
    %let _fcfmsg_=;

    /*-------------------------------------------------------------------------
     * Get e-file information from configuration files and 
     * derive the new e-file keys based on the max of the existing e-file keys
     *-------------------------------------------------------------------------*/
     
    %let efile_max_report_count=0;
    %let last_efile_key=0;
   
    %if %nrbquote(&efile_key) ne %nrbquote() %then
    %do;
        %let efile_max_report_count=10000;
        %let last_efile_key=%eval(&efile_key-1);
    %end;
    %else 
    %do;

        proc sql noprint;
             select efile_max_report_count into : efile_max_report_count
             from seg_kc.fsk_rc_form_config
             where form_config_key=&form_config_key;
              %fcf_rcset(&sqlrc);
              %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                  %goto MACRO_END;
              %end;  

             select max(efile_key) into :last_efile_key
             from seg_kc.fsk_rc_efile;
             %fcf_rcset(&sqlrc);
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                  %goto MACRO_END;
             %end;  
        quit;

    %end;

    %let record_id_cnt=0;


    proc sql noprint;       
  
         select distinct form_efile_record_id into :record_id_1 -  
         from rc_cfg.form_efile_record_config(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=form_efile_record_config))
         where form_config_key=&form_config_key 
           ;
    quit; 
        %let record_id_cnt=&sqlobs;  

        %if &record_id_cnt=0 %then %do;
            %let _fcfmsg_=Note: RC: No &form_type_code record configuration found.;
            %fcf_put_error;
            %fcf_rcset(3);
            %goto MACRO_END;
        %end;

        %if &efile_max_report_count < 1 %then %let efile_max_report_count=100; 
        %if &last_efile_key < 1 %then %let last_efile_key=0; 
        

       /*-------------------------------------------------------------------------
        * Define activity dates. They will be used to derive coverage period. 
        *-------------------------------------------------------------------------*/
        
        %let activity_start_date_var=;
        %let activity_end_date_var=;

        %if &form_type_code=SAR %then 
        %do;
              %let activity_start_date_var=SAR_begin_date;
              %let activity_end_date_var=SAR_end_date;
        %end;
                
        %else %if &form_type_code=CTR %then
        %do;
               %let activity_start_date_var=transaction_date;
               %let activity_end_date_var=transaction_date;
        %end;

        %else  %if &form_type_code=DOEP %then
        %do;
               %let activity_start_date_var=effective_date;
               %let activity_end_date_var=effective_date;
        %end;

   /*-------------------------------------------------------------------------
    * Assign e-file keys and report sequence number. 
    * Also derive coverage period using the earliest activity start date and the
    * the latest end date.
    * If any of the activity end date is empty (i.e. activity is on-going) 
    * use today date as coverage end date.  
    * To apply efile key and subjectsequence number to the records in all submission tables. 
    * SAS formats are created:
    *   format report2efile format for assigning report sequence number
    *   subject_seq_number format for subject sequence numbers in 4<x> tables.  
    * 
    *-------------------------------------------------------------------------*/

    data seq_report (keep=efile_key report_key report_seq_number index=(report_key))
         /* report table with sequence numbers and efile key */
 
         seq_batch_date  (keep=efile_key coverage_beginning_date coverage_ending_date 
                          index=(efile_key))
         /* report level data with batch statistics. This table will be rolled in the next data step. */

         seq_format (keep=fmtname type start label)
         /* report sequence table for creating format */
         ;
         retain fmtname 'report2efile'
                type 'N'
          ;
         if 0 then set rc_db.&form_type_code._1A (
                                                  %fcf_get_ds_opts(lib_name=rc_db, ds_name=&form_type_code._1A)
                                                  keep= 
                                                       coverage_beginning_date 
                                                       coverage_ending_date
                                                 );
         set &form_type_code._3A (keep=report_key &activity_start_date_var &activity_end_date_var);
         retain report_seq_number;
         retain coverage_beginning_date coverage_ending_date;
                  
         /* use ceil() to derive batch number */
         efile_key=&last_efile_key+ceil(_n_/&efile_max_report_count);

         if lag(efile_key) ne efile_key then
         do; /* when it is a change batch */        
             link INIT_BATCH;         
         end;

         link REPORT_SUM; /* roll up report data to batch level */

         output seq_report;
         output seq_batch_date;    
         start=report_key;
         label=strip(put(efile_key,best.))||'-'||strip(put(report_seq_number,best.));
         output seq_format;
        
         return;

    INIT_BATCH:
        report_seq_number=0;
        coverage_beginning_date=today();
        coverage_ending_date=0;
    return;

    REPORT_SUM:
            report_seq_number+1;
            activity_start_date=&activity_start_date_var;
            activity_end_date=&activity_end_date_var;
            if activity_end_date <= 0 then
               activity_end_date=today();
       
           coverage_beginning_date = min( coverage_beginning_date, activity_start_date);
           coverage_ending_date = max( coverage_ending_date, activity_end_date);
    return;
     
    run;
    %fcf_rcset(&syserr);
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
       %goto MACRO_END;
   %end;  
    /* Extract the last record of each batch */
    data seq_batch;
         set seq_batch_date;
         by efile_key;
         if last.efile_key then output;
    run;

   %fcf_rcset(&syserr);
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
       %goto MACRO_END;
   %end; 

    /* create report sequence format */
    proc format cntlin=seq_format;
    run;
    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
       %goto MACRO_END;
    %end; 


    %if %sysfunc(exist(&form_type_code._4A)) %then
    %do;
        /* create subject sequence number format */
        data seq_subject_format (keep=fmtname type start label)
             ;
             retain fmtname 'subject_seq_number'
                    type 'C'
              ;
             
             set &form_type_code._4A (keep=report_key entity_key entity_seq_number);
             
             start=strip(put(report_key,best.))||'-'||strip(put(entity_key,best.));
             label=strip(put(entity_seq_number,best.));
             output seq_subject_format;
             
         
        run;

        %fcf_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
           %goto MACRO_END;
        %end;
     
        proc format cntlin=seq_subject_format;
        run;

        %fcf_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
           %goto MACRO_END;
       %end;
   %end;
 
   /* get the latest transmitter and institution data */
    
   %rc_efile_fincen_entity_info(form_config_key=&form_config_key, form_type_code=&form_type_code);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end;


 
  /*------------------------------------------------------------------------- 
   * Transmitter Data  1A        
   * One record per e-file        
   * -------------------------------------------------------------------------*/
    
    
    data &form_type_code._1A;
         if 0 then set rc_db.&form_type_code._1A(%fcf_get_ds_opts(lib_name=rc_db, ds_name=&form_type_code._1A));
         merge seq_batch (keep=efile_key coverage_beginning_date coverage_ending_date )
               fincen_1A
          ;                               
          by efile_key;                             
         record_type='1A';
         entity_key=efile_key;
         drop transmitter_key;
         report_seq_number=0;
         entity_seq_number=0;
         row_seq_number=1;
         user_field=put(mod(entity_key,10000000000),z10.); /* get the last 10 digits of entity key */          
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
       %goto MACRO_END;
   %end;  
 

  /*------------------------------------------------------------------------- 
   * Filing Institution  Data  2A        
   * One record per e-file        
   * -------------------------------------------------------------------------*/
   
       
    data &form_type_code._2A ;
         if 0 then set rc_db.&form_type_code._2A(%fcf_get_ds_opts(lib_name=rc_db, ds_name=&form_type_code._2A));
         merge seq_batch (keep=efile_key)
             fincen_2A                
          ;
         by efile_key;                              
                                       
         record_type='2A';
         report_seq_number=0;
         entity_seq_number=0;
         row_seq_number=1;
         entity_level_code='INSTITUTION';
         entity_key=institution_key;
         user_field=put(mod(entity_key,10000000000),z10.); /* get the last 10 digits of entity key */          
    
         drop institution_key default_ind;
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
       %goto MACRO_END;
   %end;  
 
    
  /*------------------------------------------------------------------------- 
   *  Assign efile and sequence number to all files except 1A and 2A      
   * -------------------------------------------------------------------------*/
   
      proc sql ;

     %do itable=3 %to  &record_id_cnt ;
         %let record_id=&&&record_id_&itable;
         %let table_name=%trim(&form_type_code._&record_id);
         %if %sysfunc(exist(&table_name)) %then
         %do;
             update &table_name 
                    set efile_key=input(scan(put(report_key,report2efile.),1,'-'),best.),
                        report_seq_number=input(scan(put(report_key,report2efile.),2,'-'),best.),
                        user_field=put(mod(entity_key,10000000000),z10.)         
   
            %if %substr(&record_id,1,1)=4 and &record_id ne '4A' %then
            %do;
                       , entity_seq_number=input(put(STRIP(PUT(REPORT_KEY, BEST.)) || '-' ||
                                                     STRIP(PUT(ENTITY_KEY, BEST.)) 
                                                     ,$subject_seq_number.)
                                                ,best.)

            %end;
             ; 
             %fcf_rcset(&sqlrc);
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
              %end;

         %end;
      %end;
     quit;

  /*------------------------------------------------------------------------- 
   * Branch Data         
   * SAR - 2C
   * CTR - 2B 
   * -------------------------------------------------------------------------*/
  
     

    %if &form_type_code=SAR %then %let branch_record_type=2C;
    %else %if &form_type_code=CTR %then %let branch_record_type=2B;
    %else %let branch_record_type=;
      
    %if %str(&branch_record_type) ne %str() and %sysfunc(exist(&form_type_code._&branch_record_type)) %then
    %do;
             proc sql ;
                 create table branch_info as
                 select rpt.report_key, 
                        rpt.entity_key,                     
                        cfg.*
                 from  &form_type_code._&branch_record_type as rpt, 
                       rc_cfg.fincen_branch(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_branch)) as cfg
                 where rpt.entity_key=cfg.branch_key                    
                 ;
             quit;
              %fcf_rcset(&sqlrc);
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
              %end;

             
             data &form_type_code._&branch_record_type ;              
                 modify &form_type_code._&branch_record_type 
                        branch_info 
                  ;
                 by report_key entity_key;
                 row_seq_number=entity_seq_number;
                 entity_seq_number=0;
                 branch_number=put(input(branch_number,best.),z10.);
                 record_type="%upcase(&branch_record_type)";
                 if _iorc_=0 then replace;
                 else _error_=0;
                 
             run;
             %fcf_rcset(&syserr);
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
              %end;

             
     %end;  
      
 /*------------------------------------------------------------------------- 
   * Institution where activity occurred         
   * SAR only - 2B
   * repeat 2A data into 2B    
   * There will be one 2A per e-file.
   * There will be one 2B per report.
   * -------------------------------------------------------------------------*/
      
      
    %if &form_type_code=SAR %then 
    %do;
            
           

             data &form_type_code._2B ; 
                
                 modify &form_type_code._2B        
                        fincen_sar_2B   
                  ;
                 by report_key; 
                 record_type='2B';
                 entity_seq_number = 0;
                 row_seq_number = 1;
                 if _iorc_=0 then
                 do;                    
                    entity_level_code = 'INSTITUTION';
                    institution_number = put(input(entity_number,best.),z10.);
                    entity_key = institution_key;
                    user_field = put(mod(entity_key,10000000000),z10.); 
                    link SET_UNK_FLG;
                    replace;
                 end;
                 else 
                 do;
                    _error_ = 0;                   
                 end;
                 return;
             SET_UNK_FLG:
                 if INSTITUTION_TAX_ID="" then INSTITUTION_TIN_UNK_FLG="X";
                 if INSTITUTION_LEGAL_NAME="" then INSTITUTION_LEGAL_NAME_UNK_FLG="X";
                 if INSTITUTION_ADDRESS="" then INSTITUTION_ADDRESS_UNK_FLG="X";
                 if INSTITUTION_CITY_NAME="" then INSTITUTION_CITY_UNKN_FLG="X";
                 if INSTITUTION_ZIP_CODE="" then INSTITUTION_ZIP_UNK_FLG="X";
                 if INSTITUTION_COUNTRY_CODE="" then INSTITUTION_COUNTRY_UNK_FLG="X";
             return;
             run;
             %fcf_rcset(&syserr);
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
              %end; 

             %if %sysfunc(exist(&form_type_code._2C)) %then
             %do;
                   proc sql;
                    create table count_2C as
                    select distinct efile_key, report_key, count(*) as BRANCH_COUNT
                    from &form_type_code._2C
                    group by efile_key, report_key
                    order by efile_key, report_key;
                   quit;
            %end;
            %else
            %do;
                    data count_2c;
                         set seq_report;
                         branch_count=0;
                    run;

            %end;

             proc sql;
                     update &form_type_code._2B
                     set institution_report_no_branch_flg='X' 
                     where report_key in (select report_key from count_2C where branch_count=0)
                 ;
             quit;
             %fcf_rcset(&sqlrc);
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
              %end;
              
    %end;
    


  
     %rc_efile_fincen_sum_&form_type_code(form_config_key=&form_config_key,
                                          form_type_code=&form_type_code
                                          );
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %goto MACRO_END;
     %end;
    
  

%MACRO_END:
  

    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
       %fcf_execute_macro(fcf_error);
    %else 
       %fcf_execute_macro(fcf_success);
    
  
%mend;



