/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_resp_fincen_report
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    report on the response processing result   
|          
+------------------------------------------------------------------------------+
| PARAMETERS:
|              INPUT_TYPE=type of input parameter.                                
|                valid values are; FROM_DTTM, RESPONSE_ID, EFILE_KEY, REPORT_KEY, BSA_ID 
|                If it is blank, all response data will be reported.               
|              FROM_DTTM=filter the response data by this SAS timestamp           
|              RESPONSE_ID=filter the response data by this response ID           
|              EFILE_KEY  = filter the response data by this e-file key            
|              REPORT_KEY     = filter the response data by this report key            
|              BSA_ID    = filter the response data by this BSA ID               
|              RESPONSE_TYPE_CODE=asc for ASCII acknowledgment receipt
|                                 xml for XML acknowledgment receipt 
|              FORM_CONFIG_KEY=form key (301 for SAR, 311 for SARX, and 312 for CTRX)           
|                                       (303 for DOEPX)
|                  This is a required parameter.                                  
|              PRINTYN=Y to generate report.                                      
|                      N to prepare report data only. 
| NOTE:       CTRX and SARX  and DOEPX response type is XML.
|             Since rc_resp_report stored process hardcoded
|               response_type_code as ASC, response_type_code will change to XML when   
|               the form is CTRX (form_config_key=312) and SARX (form_config_key=311)
|               and DOEPX (form_config_key=303)
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/


%macro rc_resp_fincen_report(   input_type=
                                 , from_dttm=
                                 , response_id=
                                 , efile_key=
                                 , report_key=
                                 , bsa_id=
                                 , response_type_code=ASC
                                 , form_config_key=
                                 , printYN=Y
                               
                             );

    
%let trans_rc=0;
%let _fcfmsg_=; 

 
    %local filter_response_id filter_efile_key filter_report_key form_type_code error_code_table;
 
    /* Get the form type code. e.g. SAR or CTR */
    proc sql noprint;
         select strip(form_type_code) 
                into
                :form_type_code 
         from seg_kc.fsk_rc_form_config
         where form_config_key=&form_config_key
    ;
	quit;
	
	/* fix response_type_code when the form is CTRX or SARX or DOEPX */
    %if (
         (&form_type_code.=CTRX) or
         (&form_type_code.=SARX) or
         (&form_type_code.=DOEPX)
        ) and %upcase(&response_type_code)=ASC %then
    %do;
        %let response_type_code=XML;
    %end;  

    %fcf_rcset(&sqlrc);
    
    %let form_type_code=%trim(&form_type_code);
                        

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to get form_type_code;
         %fcf_put_error;
         %goto MACRO_END;
     %end;
     
    quit;

    /* Define the error code table name based on the form type code */
    %let error_code_table=rc_cfg.%trim(%left(&form_type_code))_fincen_error_code;

    /* Initialize filter lists */
    %let filter_response_id=;
    %let filter_efile_key=;
    %let filter_report_key=;


    /* If no input type is specified, all response data for the form will be reported */
    %if %upcase(&input_type)=%str() %then
    %do;
            proc sql noprint;
            select distinct quote(strip(response_id)) 
                   into :filter_response_id
                   separated by ' ' 
            from  rc_db.resp_fincen_summary(%fcf_get_ds_opts(lib_name=rc_db, ds_name=resp_fincen_summary))
            where  form_config_key=&form_config_key
             %if %str(&response_type_code) ne %str() %then 
             %do;
                   and upcase(response_type_code)="%upcase(&response_type_code)"
             %end; 

            %fcf_rcset(&sqlrc);
                        

         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: RC: Unable to get form_type_code;
             %fcf_put_error;
             %goto MACRO_END;
         %end;
     


                  
            ;  
                
    %end;

    /* Define the list of response ID of the response files which are processed after the FROM_DTTM time stamp */

    %if %upcase(&input_type)=FROM_DTTM %then
    %do;
        %if %str(&from_dttm) ne %str() %then
        %do; 
            %let filter_response_id=;
            proc sql noprint;
            select distinct quote(strip(response_id))
            into :filter_response_id
                  separated by ' '
            from  rc_db.resp_fincen_summary(%fcf_get_ds_opts(lib_name=rc_db, ds_name=resp_fincen_summary))
            where form_config_key=&form_config_key
                  and processed_date >= &from_dttm
            ;
           %fcf_rcset(&sqlrc);
                        

         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: RC: Unable to get response list.;
             %fcf_put_error;
             %goto MACRO_END;
         %end;
                 
        %end;
    %end;


    %else %if %upcase(&input_type)=RESPONSE_ID %then
    %do;
        %if "%nrbquote(&response_id)" ne %nrbquote() %then
        %do; 
            %let filter_response_id="&response_id";
            %let filter_efile_key=;
            %let filter_report_key=;     
        %end;
    %end;

    %else %if %upcase(&input_type)=EFILE_KEY %then
    %do;
        %if %str(&efile_key) ne %str() %then
        %do; 
            %let filter_report_key=;
            %let filter_response_id=;
            %let filter_efile_key=&efile_key;
 
             /* Find the IDs of the response files that contain the efile */
             /* If the process is done right, there should only be one response file per an efile. */
             /* However, the same efile may be submitted multiple times during testing.   */
             proc sql noprint;
             select  distinct quote(strip(response_id))               
             into   :filter_response_id   
                     separated by ' '    
             from rc_db.resp_fincen_summary(%fcf_get_ds_opts(lib_name=rc_db, ds_name=resp_fincen_summary))
             where efile_key=&efile_key
              %if %str(&response_type_code) ne %str() %then 
             %do;
                   and upcase(response_type_code)="%upcase(&response_type_code)"
             %end; 
     
            ; 
            %fcf_rcset(&sqlrc);
                            

         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: RC: Unable to get response list by e-file key;
             %fcf_put_error;
             %goto MACRO_END;
         %end;
     


        
        %end;
    %end;

    %else %if %upcase(&input_type)=REPORT_KEY  %then
    %do;
        %if %str(&report_key) ne %str() %then
        %do; 
            %let filter_report_key=&report_key;
            %let filter_response_id=;
            %let filter_efile_key=;
             /* Find the IDs of the response files that contain the report */
             /* If the process is done right, there should only be one response file per report */
             /* However, more response files may be created for the same report during testing.   */
             proc sql noprint ;
             select distinct  quote(strip(rpt.response_id)) 
                    , put(rpt.efile_key,best.)
             into   :filter_response_id
                  , :filter_efile_key 
             separated by ' '
             from   rc_db.resp_fincen_report_list(%fcf_get_ds_opts(lib_name=rc_db, ds_name=resp_fincen_report_list)) as rpt
                  , rc_db.resp_fincen_summary(%fcf_get_ds_opts(lib_name=rc_db, ds_name=resp_fincen_summary)) as sum
             where rpt.report_key=&report_key
                   and rpt.response_id=sum.response_id
             %if %str(&response_type_code) ne %str() %then 
             %do;
                  and upcase(sum.response_type_code)="%upcase(&response_type_code)"
             %end; 
             
        ; 
            %fcf_rcset(&sqlrc);
                        

         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: RC: Unable to get the response list by report key;
             %fcf_put_error;
             %goto MACRO_END;
         %end;
     

        
        %end;
    %end;

    %else %if %upcase(&input_type)=BSA_ID  %then
    %do;
        %if %str(&bsa_id) ne %str() %then
        %do; 
             /* Find the ID of the response files that has reports with the same BSA_ID */
             /* Multiple reports may have the same BSA_ID when they are for correction or update */
           
            %let filter_report_key=;
            %let filter_response_id=;
            %let filter_efile_key=;
            proc sql noprint;
             select distinct quote(strip(rpt.response_id)) 
                    , put(rpt.efile_key,best.)
                    , put(rpt.report_key,best.) 
             into   :filter_response_id
                  , :filter_efile_key
                  , :filter_report_key
             separated by ' '
             from   rc_db.resp_fincen_report_list(%fcf_get_ds_opts(lib_name=rc_db, ds_name=resp_fincen_report_list)) as rpt
                  , rc_db.resp_fincen_summary(%fcf_get_ds_opts(lib_name=rc_db, ds_name=resp_fincen_summary)) as sum
             where rpt.bsa_id="&bsa_id"
                   and rpt.response_id=sum.response_id
             %if %str(&response_type_code) ne %str() %then 
             %do;
                  and upcase(sum.response_type_code)="%upcase(&response_type_code)"
             %end; 
     
             ; 
            %fcf_rcset(&sqlrc);
                        

         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: RC: Unable to get response list by BSA-ID;
             %fcf_put_error;
             %goto MACRO_END;
         %end;
     
        
        %end;
    %end;


   
    proc sql noprint;

          /* subset the RESP_FINCEN_SUMMARY with the list of response IDs */
         create table resp_fincen_summary_print as
         select   sum.*
                , ef.efile_id
         from   rc_db.resp_fincen_summary(%fcf_get_ds_opts(lib_name=rc_db, ds_name=resp_fincen_summary)) as sum
         left join seg_kc.fsk_rc_efile as ef
         on   sum.efile_key=ef.efile_key
         where 1
         %if %str(&filter_response_id) ne %str() %then
         %do;
              and sum.response_id in (&filter_response_id)
         %end;
              
        ;

        %fcf_rcset(&sqlrc);
                        

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to create response summary table.;
         %fcf_put_error;
         %goto MACRO_END;
     %end;
     

        
         /* subset the RESP_FINCEN_REPORT_LIST with the list of REPORT_KEY */

         create table resp_fincen_report_print as
         select   rpt.*
                , rr.report_desc
         from   rc_db.resp_fincen_report_list(%fcf_get_ds_opts(lib_name=rc_db, ds_name=resp_fincen_report_list)) as rpt
         left join seg_kc.fsk_rc_report as rr
         on  rpt.report_key=rr.report_key
         where 1
         %if %str(&filter_response_id) ne %str() %then
         %do;
               and rpt.response_id in (&filter_response_id)
         %end;
         

         %if %str(&filter_efile_key) ne %str() %then
         %do;
                and rpt.efile_key in (&filter_efile_key)
          %end;      
          %if %str(&filter_report_key) ne %str() %then 
          %do;
                and rpt.report_key in (&filter_report_key)
          %end;
          order by response_id, trans_seq_number
    ;
         
          %fcf_rcset(&sqlrc);
                        

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to create report list. ;
         %fcf_put_error;
         %goto MACRO_END;
     %end;
     

         /* subset the RESP_FINCEN_ERROR_LIST with the list of Reponse ID, EFILE_KEY or REPORT_KEY */


         create table resp_fincen_error_print as
         select a.*,
                rr.report_desc
         from
         (
         select     err.*
                  , error_desc
                  %if %upcase(&response_type_code) = XML %then
                  %do;
                    , ref.field_reference
                  %end;
         from      rc_db.resp_fincen_error_list(%fcf_get_ds_opts(lib_name=rc_db, ds_name=resp_fincen_error_list)) as err
         left join &error_code_table(%fcf_get_ds_opts(ds_name=&error_code_table)) as ref
         on  err.error_code=ref.error_code
         %if %upcase(&response_type_code) ne XML %then
         %do;  
             and err.record_type=ref.error_record_type
         %end;
         where 1 
         %if %str(&filter_response_id) ne %str() %then
         %do;
                and err.response_id in (&filter_response_id)
         %end;
         %if %str(&filter_efile_key) ne %str() %then
         %do;
                and err.efile_key in (&filter_efile_key)
          %end;      
          %if %str(&filter_report_key) ne %str() %then 
          %do;
                and err.report_key in (&filter_report_key)
          %end;
          ) as a
          left join seg_kc.fsk_rc_report as rr
          on a.report_key=rr.report_key
          order by response_id, trans_seq_number, line_number, error_seq_number
    ;

          %fcf_rcset(&sqlrc);
                        

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to create error list.;
         %fcf_put_error;
         %goto MACRO_END;
     %end;
     


    quit;

  
  

%if %upcase(&printYN)=Y %then 
%do;
    title;
    %if %str(&filter_response_id)=%str() %then
    %do;
        title; 
        options ls=72;
        data _null_;
             file print notitles;
             put "No processed %trim(&form_type_code) response found.";
        run;
        %goto MACRO_END;
    %end;

     %let response_msg_found=0;

         /* Check if there is any processing messages to be displayed */
         proc sql noprint;
         select '1'  
                into :response_msg_found

         from resp_fincen_summary_print  
         where response_processing_msg ne ''
         ; 
    quit;

    proc sql noprint;
         create view resp_fincen_summary_view as
         select   resp.*
                , ef.efile_status_code  
         from resp_fincen_summary_print as resp 
         left join seg_kc.fsk_rc_efile as ef
         on resp.efile_key=ef.efile_key
         ; 
  
         create view resp_fincen_report_view as
         select   resp.*
                , rr.report_agency_ref_id
                , rr.file_status_code
         from resp_fincen_report_print as resp 
         left join seg_kc.fsk_rc_efile_report as rr
         on resp.report_key=rr.report_key and resp.efile_key=rr.efile_key
         ; 

    proc report data=resp_fincen_summary_view nowindows center ls=120 ;
         column response_id  
                efile_id 
                efile_status_code
                %if %upcase(&response_type_code) = XML %then
                %do;
                submission_status_code
                %end;               
                record_3A_count primary_error_count secondary_error_count processed_date
         %if %str(&response_msg_found) = %str(1) %then
         %do;
              response_processing_msg
         %end; 
          ;
         define response_id /width=30 flow;
         define efile_id /width=10 flow "E-File ID";
         define efile_status_code /width=10 flow "E-File Status"  format=$FCF_EFILE_STATUS. ;
         %if %str(&response_msg_found) = %str(1) %then
         %do;
             define response_processing_msg /flow width=30;
         %end;
         
         %if %upcase(&response_type_code) = XML %then
         %do;
            define record_3A_count /"Number of*Activities";
            define submission_status_code /"Submission Status Code";
         %end;

         title "&form_type_code Response Summary";      
    run;
    quit;
      %fcf_rcset(&syserr);
                        

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to get generate summary report;
         %fcf_put_error;
         %goto MACRO_END;
     %end;
     
    

     %let report_msg_found=0;
         /* Check if there is any processing messages to be displayed */
     proc sql noprint;
         select '1'  
                into :report_msg_found

         from resp_fincen_report_print  
         where response_processing_msg ne ''
         ; 
    quit;

    
     

    proc report data=resp_fincen_report_view nowindows center split='*' ls=120 ;
         column response_id report_key file_status_code 
         %if %upcase(&response_type_code) ne XML %then
         %do;
               trans_seq_number
         %end;        
         bsa_id  primary_error_count secondary_error_count
         %if %str(&report_msg_found) = %str(1) %then
         %do;
              user_field
              response_processing_msg
         %end; 
          ;
         define response_id /width=10 flow;
         define report_key /format=best. "Report Key";
         %if %upcase(&response_type_code) ne XML %then
         %do;
             define trans_seq_number/"Report*Seq*Number" format=4.;
         %end;
         
         define primary_error_count/"# of*Primary*Errors" format=4.;
         define secondary_error_count/"# of*Secondary*Errors" format=4.;
         
        * define report_agency_ref_id /width=20 "DCN in Report";
         define file_status_code /width=20 "Report Status" format=$FCF_REPORT_STATUS.;
         
         %if %str(&report_msg_found) = %str(1) %then
         %do;
         define user_field /'User Field in E-File' width=10 flow;
         define response_processing_msg /flow width=30;
          %end;
         title "&form_type_code Response Report List";      
    run;
    quit;

    %fcf_rcset(&syserr);
                        

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to get form_type_code;
         %fcf_put_error;
         %goto MACRO_END;
     %end;
     




    proc report data=resp_fincen_error_print nowindows center split='*' ls=120 ;
    %if %upcase(&response_type_code) ne XML %then
    %do;
         column response_id report_key record_type line_number error_code error_type_code error_desc 
          ;
         define response_id/order flow width=20;
         define report_key /format=best. width=12 "Report Key";
         define error_desc /flow width=50;
         define line_number /'Line*Number' format=4.;
         define error_type_code /'Error Type' width=10 format=$fincen_error_type.;
    %end;
    %else
    %do;
         column response_id report_key error_context 
                error_code error_type_code error_desc field_reference 
          ;
         define response_id/order flow width=20;
         define report_key /format=best. width=12 "Report Key";
         define error_desc /flow width=50;          
         define error_type_code /'Error Type' width=10 format=$fincen_error_type.;
         define error_context /flow width=30 ;
         define field_reference /flow width=30 ;
     %end;  
         title "&form_type_code Response Error List"; 

          
    run;
    quit;

    %fcf_rcset(&syserr);
                        

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Unable to get form_type_code;
         %fcf_put_error;
         %goto MACRO_END;
     %end;
     

%end;
           
 %MACRO_END:    

   
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
      %fcf_execute_macro(fcf_error);
    %else %do;
      %fcf_execute_macro(fcf_success);
    %end;
  
    title;



%mend;
